<?php

namespace WPFormsHubspot\Api;

use WPFormsHubspot\Api\Http\Request;
use WPFormsHubspot\Api\Http\Response;

/**
 * WPForms HubSpot API class.
 *
 * @since 1.0.0
 */
class Api {

	/**
	 * URL for the HubSpot Objects API.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const API_OBJECTS_URL = 'https://api.hubapi.com';

	/**
	 * URL for the HubSpot Forms API.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const API_FORMS_URL = 'https://api.hsforms.com';

	/**
	 * Request instance.
	 *
	 * @since 1.0.0
	 *
	 * @var Request
	 */
	private $request;

	/**
	 * API constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param Request $request Request.
	 */
	public function __construct( Request $request ) {

		$this->request = $request;
	}

	/**
	 * Get contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email Email value.
	 *
	 * @return Response
	 */
	public function get_contact( $email ) {

		return $this->request->get( sprintf( self::API_OBJECTS_URL . '/crm/v3/objects/contacts/%s?idProperty=email', rawurlencode( $email ) ) );
	}

	/**
	 * Create contact.
	 *
	 * @since 1.0.0
	 *
	 * @param array $properties Properties.
	 *
	 * @return Response
	 */
	public function create_contact( $properties ) {

		$properties['company'] = '';

		return $this->request->post( self::API_OBJECTS_URL . '/crm/v3/objects/contacts', [ 'properties' => $properties ] );
	}

	/**
	 * Updating contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email      Existing contact email.
	 * @param array  $properties Properties.
	 *
	 * @return Response
	 */
	public function update_contact( $email, $properties ) {

		return $this->request->patch( sprintf( self::API_OBJECTS_URL . '/crm/v3/objects/contacts/%s?idProperty=email', rawurlencode( $email ) ), [ 'properties' => $properties ] );
	}

	/**
	 * Delete contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id Contact ID.
	 *
	 * @return Response
	 */
	public function delete_contact( $id ) {

		return $this->request->delete( sprintf( self::API_OBJECTS_URL . '/crm/v3/objects/contacts/%s', absint( $id ) ) );
	}

	/**
	 * Update contact note.
	 *
	 * @since 1.0.0
	 *
	 * @param string $contact_id Contact ID.
	 * @param string $note       Note.
	 *
	 * @return Response
	 */
	public function update_contact_note( $contact_id, $note ) {

		return $this->request->post(
			self::API_OBJECTS_URL . '/engagements/v1/engagements',
			[
				'engagement'   => [
					'active' => true,
					'type'   => 'NOTE',
				],
				'associations' => [
					'contactIds' => [ absint( $contact_id ) ],
				],
				'attachments'  => [],
				'metadata'     => [
					'body' => $note,
				],
			]
		);
	}

	/**
	 * Connect contact to the company.
	 *
	 * @since 1.0.0
	 *
	 * @param string $company_id Company ID.
	 * @param string $contact_id Contact ID.
	 *
	 * @return Response
	 */
	public function company_connect( $company_id, $contact_id ) {

		return $this->request->put(
			sprintf(
				self::API_OBJECTS_URL . '/crm/v3/objects/contacts/%d/associations/companies/%d/1',
				absint( $contact_id ),
				absint( $company_id )
			)
		);
	}

	/**
	 * Get connected contact company.
	 *
	 * @since 1.0.0
	 *
	 * @param string $contact_id Contact ID.
	 *
	 * @return Response
	 */
	public function get_connected_contact_company( $contact_id ) {

		return $this->request->get(
			sprintf(
				self::API_OBJECTS_URL . '/crm/v3/objects/contacts/%d/associations/companies',
				absint( $contact_id )
			)
		);
	}

	/**
	 * Disconnect contact from the company.
	 *
	 * @since 1.0.0
	 *
	 * @param string $company_id Company ID.
	 * @param string $contact_id Contact ID.
	 *
	 * @return Response
	 */
	public function company_disconnect( $company_id, $contact_id ) {

		return $this->request->delete(
			sprintf(
				self::API_OBJECTS_URL . '/crm/v3/objects/contacts/%d/associations/companies/%d/1',
				absint( $contact_id ),
				absint( $company_id )
			)
		);
	}

	/**
	 * Get lead status list.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_lead_status_list() {

		return $this->get_contact_property_options( 'hs_lead_status' );
	}

	/**
	 * Get lifecycle stage list.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_lifecycle_stage_list() {

		return $this->get_contact_property_options( 'lifecyclestage' );
	}

	/**
	 * Get contact owner list.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_contact_owner_list() {

		$response = $this->request->get( self::API_OBJECTS_URL . '/crm/v3/owners', [ 'limit' => 500 ] );

		if ( $response->has_errors() ) {
			return [];
		}

		$body = $response->get_body();

		if ( ! isset( $body['results'] ) ) {
			return [];
		}

		$owners = [];

		foreach ( $body['results'] as $owner ) {
			$owners[ $owner['id'] ] = $owner['firstName'] . ' ' . $owner['lastName'];
		}

		return $owners;
	}

	/**
	 * Get companies list.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_companies_list() {

		$response = $this->request->get( self::API_OBJECTS_URL . '/crm/v3/objects/companies', [ 'limit' => 100 ] );

		if ( $response->has_errors() ) {
			return [];
		}

		$body = $response->get_body();

		if ( ! isset( $body['results'] ) ) {
			return [];
		}

		$companies = [];

		foreach ( $body['results'] as $company ) {
			$companies[ $company['id'] ] = $company['properties']['name'];
		}

		return $companies;
	}

	/**
	 * Get contact properties data.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_contact_properties() {

		$response = $this->request->get( self::API_OBJECTS_URL . '/crm/v3/properties/contacts' );

		if ( $response->has_errors() ) {
			return [];
		}

		$body = $response->get_body();

		if ( ! isset( $body['results'] ) ) {
			return [];
		}

		$properties = [];

		foreach ( $body['results'] as $property ) {
			$properties[ $property['name'] ] = $property;
		}

		return $properties;
	}

	/**
	 * Get contact properties.
	 *
	 * @since 1.0.0
	 *
	 * @param bool $groups Group by group names.
	 *
	 * @return array
	 */
	public function get_contact_properties_options( $groups = false ) {

		$options = [];

		foreach ( $this->get_contact_properties() as $property ) {

			if (
				$property['hidden'] ||
				! $property['formField'] ||
				in_array( $property['name'], [ 'hs_lead_status', 'lifecyclestage' ], true ) ||
				in_array( $property['groupName'], [ 'analyticsinformation', 'conversioninformation' ], true ) ||
				! in_array( $property['type'], [ 'enumeration', 'date', 'string', 'number' ], true )
			) {
				continue;
			}

			if ( $groups ) {
				$options[ $property['groupName'] ][ $property['name'] ] = $property['label'];
			} else {
				$options[ $property['name'] ] = $property['label'];
			}
		}

		ksort( $options );

		return $options;
	}

	/**
	 * Get contact properties groups.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_contact_properties_groups() {

		$response = $this->request->get( self::API_OBJECTS_URL . '/crm/v3/properties/contacts/groups' );

		if ( $response->has_errors() ) {
			return [];
		}

		$body = $response->get_body();

		if ( ! isset( $body['results'] ) ) {
			return [];
		}

		$groups = [];

		foreach ( $body['results'] as $group ) {
			$groups[ $group['name'] ] = $group['label'];
		}

		return $groups;
	}

	/**
	 * Get contact property options.
	 *
	 * @since 1.0.0
	 *
	 * @param string $property Property name.
	 *
	 * @return array
	 */
	private function get_contact_property_options( $property ) {

		$response = $this->request->get( self::API_OBJECTS_URL . '/crm/v3/properties/contacts/' . $property );

		if ( $response->has_errors() ) {
			return [];
		}

		$property_data = $response->get_body();

		if ( ! isset( $property_data['options'] ) ) {
			return [];
		}

		$options = [];

		foreach ( $property_data['options'] as $lead_status ) {
			$options[ $lead_status['value'] ] = $lead_status['label'];
		}

		return $options;
	}

	/**
	 * Create form.
	 *
	 * @since 1.0.0
	 *
	 * @param array $form_data Form data.
	 *
	 * @return Response
	 */
	public function create_form( $form_data ) {

		return $this->request->post( self::API_OBJECTS_URL . '/forms/v2/forms', $form_data );
	}

	/**
	 * Get form.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id Form ID.
	 *
	 * @return Response
	 */
	public function get_form( $id ) {

		return $this->request->get( sprintf( self::API_OBJECTS_URL . '/forms/v2/forms/%s', sanitize_key( $id ) ) );
	}

	/**
	 * Update form.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id        Form ID.
	 * @param array  $form_data Form data.
	 *
	 * @return Response
	 */
	public function update_form( $id, $form_data ) {

		$exist_response = $this->get_form( $id );

		if ( $exist_response->has_errors() ) {
			return $this->create_form( $form_data );
		}

		return $this->request->put( sprintf( self::API_OBJECTS_URL . '/forms/v2/forms/%s', sanitize_key( $id ) ), $form_data );
	}

	/**
	 * Form submission.
	 *
	 * @since 1.0.0
	 *
	 * @param array $connection_data Connection data..
	 * @param array $hs_form_data    HubSpot Form data.
	 *
	 * @return Response
	 */
	public function form_submission( $connection_data, $hs_form_data ) {

		return $this->request->post(
			sprintf( self::API_FORMS_URL . '/submissions/v3/integration/secure/submit/%s/%s', $connection_data['hs_portal_id'], $connection_data['hs_form_id'] ),
			$hs_form_data
		);
	}
}
