<?php

namespace WPFormsHubspot\Api\Http;

/**
 * Wrapper class for HTTP requests.
 *
 * @since 1.0.0
 */
class Request {

	/**
	 * User-agent value.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const USER_AGENT = 'wpforms-hubspot/';

	/**
	 * Access token.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private $token;

	/**
	 * Request constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param string $token Access token.
	 */
	public function __construct( $token ) {

		$this->token = $token;
	}

	/**
	 * Send a GET request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function get( $url, $args = [] ) {

		$url = add_query_arg( $args, $url );

		return $this->request( 'GET', $url, [] );
	}

	/**
	 * Send a POST request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function post( $url, $args = [] ) {

		$args = ! empty( $args ) ? [ 'body' => $args ] : [];

		return $this->request( 'POST', $url, $args );
	}

	/**
	 * Send a PUT request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function put( $url, $args = [] ) {

		$args = ! empty( $args ) ? [ 'body' => $args ] : [];

		return $this->request( 'PUT', $url, $args );
	}

	/**
	 * Send a PATCH request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function patch( $url, $args = [] ) {

		$args = ! empty( $args ) ? [ 'body' => $args ] : [];

		return $this->request( 'PATCH', $url, $args );
	}

	/**
	 * Send a DELETE request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function delete( $url, $args = [] ) {

		return $this->request( 'DELETE', $url, $args );
	}


	/**
	 * Send a request based on method (main interface).
	 *
	 * @since 1.0.0
	 *
	 * @param string $method Request method.
	 * @param string $url    Request URL.
	 * @param array  $args   Options for the request.
	 *
	 * @return Response
	 */
	public function request( $method, $url, $args ) {

		// Merge options.
		$options = array_merge_recursive( $this->get_default_parameters(), $args );

		// Set a request method.
		$options['method'] = $method;

		// Prepare a request body, as API expect it in a JSON format.
		if (
			! empty( $options['headers']['Content-Type'] ) &&
			$options['headers']['Content-Type'] !== 'application/x-www-form-urlencoded' &&
			! empty( $options['body'] )
		) {
			$options['body'] = wp_json_encode( $options['body'] );
		}

		/**
		 * Filter a request options array before it's sent.
		 *
		 * @since 1.0.0
		 *
		 * @param array   $options  Request options.
		 * @param string  $method   Request method.
		 * @param string  $url      Request URL.
		 * @param Request $instance Instance of Request class.
		 */
		$options = (array) apply_filters( 'wpformshubspot_api_http_request_options', $options, $method, $url, $this );

		// Retrieve the raw response from a safe HTTP request.
		return new Response( wp_safe_remote_request( $url, $options ) );
	}

	/**
	 * Retrieve default parameters for request.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_default_parameters() {

		return [
			'method'  => 'GET',
			'timeout' => 5,
			'headers' => [
				'Authorization' => 'Bearer ' . $this->token,
				'User-Agent'    => self::USER_AGENT . WPFORMS_HUBSPOT_VERSION,
				'Accept'        => 'application/json',
				'Connection'    => 'keep-alive',
				'Content-Type'  => 'application/json',
			],
		];
	}
}
