<?php

namespace WPFormsHubspot;

use WPForms\Providers\Loader as ProvidersLoader;
use WPFormsHubspot\Api\Client;
use WPFormsHubspot\Provider\Account;
use WPFormsHubspot\Provider\OAuthPopup;
use WPFormsHubspot\Provider\Template;
use WPFormsHubspot\Tasks\ProcessActionTask;
use WPFormsHubspot\Provider\Core;
use WPForms_Updater;
use stdClass;

/**
 * WPForms HubSpot main class.
 *
 * @since 1.0.0
 */
class Plugin {

	/**
	 * Provider slug.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const SLUG = 'hubspot';

	/**
	 * Manage API accounts.
	 *
	 * @since 1.0.0
	 *
	 * @var Account
	 */
	private $account;

	/**
	 * Client for API.
	 *
	 * @since 1.0.0
	 *
	 * @var Client
	 */
	private $client;

	/**
	 * Get some addon templates.
	 *
	 * @since 1.0.0
	 *
	 * @var Template
	 */
	private $template;

	/**
	 * Constructor.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {}

	/**
	 * Initialize plugin.
	 *
	 * @since 1.0.0
	 * @since 1.1.0 Returns Plugin instance.
	 *
	 * @return Plugin
	 */
	public function init() {

		$this->hooks();
		$this->load_dependencies();

		return $this;
	}

	/**
	 * Plugin hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_updater', [ $this, 'updater' ] );
	}

	/**
	 * Get a single instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return Plugin
	 */
	public static function get_instance() {

		static $instance;

		if ( ! $instance ) {
			$instance = new self();

			$instance->init();
		}

		return $instance;
	}

	/**
	 * All the actual plugin loading is done here.
	 *
	 * @since 1.0.0
	 */
	public function load_dependencies() {

		$this->account  = new Account();
		$this->client   = new Client();
		$this->template = new Template();

		$this->account->hooks();
		$this->template->hooks();

		( new ProcessActionTask() )->hooks();

		ProvidersLoader::get_instance()->register(
			Core::get_instance()
		);

		( new OAuthPopup() )->hooks();
	}

	/**
	 * Get property.
	 *
	 * @since 1.0.0
	 *
	 * @param string $property_name Property name.
	 *
	 * @return mixed
	 */
	public function get( $property_name ) {

		return property_exists( $this, $property_name ) ? $this->{$property_name} : new stdClass();
	}

	/**
	 * Load the plugin updater.
	 *
	 * @since 1.0.0
	 *
	 * @param string $key License key.
	 */
	public function updater( $key ) {

		new WPForms_Updater(
			[
				'plugin_name' => 'WPForms HubSpot',
				'plugin_slug' => 'wpforms-hubspot',
				'plugin_path' => plugin_basename( WPFORMS_HUBSPOT_FILE ),
				'plugin_url'  => trailingslashit( WPFORMS_HUBSPOT_URL ),
				'remote_url'  => WPFORMS_UPDATER_API,
				'version'     => WPFORMS_HUBSPOT_VERSION,
				'key'         => $key,
			]
		);
	}
}
