<?php

namespace WPFormsHubspot\Provider;

use WPFormsHubspot\Plugin;

/**
 * Class Account.
 *
 * @since 1.0.0
 */
class Account {

	/**
	 * WPForms website URL.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const WPFORMS_URL = 'https://wpforms.com';

	/**
	 * HubSpot connect endpoint.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const HUBSPOT_CONNECT_ENDPOINT = '/oauth/hubspot-connect';

	/**
	 * Add hooks.
	 *
	 * @since 1.0.0
	 */
	public function hooks() {

		add_action( 'wp_ajax_wpforms_hubspot_popup_oauth', [ $this, 'save_popup_oauth_data' ] );
	}

	/**
	 * Save OAuth data after connecting in popup.
	 *
	 * @since 1.0.0
	 */
	public function save_popup_oauth_data() {

		// Run a security check.
		check_ajax_referer( 'wpforms-hubspot', 'nonce' );

		// Check for permissions.
		if ( empty( $_POST['data'] ) || ! wpforms_current_user_can() ) {
			return;
		}

		$this->save_credentials( wp_unslash( $_POST['data'] ), uniqid( '', true ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

		wp_safe_redirect( wp_get_referer() );
		exit;
	}

	/**
	 * Save user credentials.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $credentials Credentials.
	 * @param string $state       Unique ID.
	 */
	private function save_credentials( $credentials, $state ) {

		$required_keys = [ 'access_token', 'refresh_token', 'date', 'label', 'expires_in', 'code', 'hub_id' ];

		if ( 0 !== count( array_diff( $required_keys, array_keys( $credentials ) ) ) ) {
			return;
		}

		if ( ! $this->is_connected( $credentials['hub_id'] ) ) {
			wpforms_update_providers_options( Plugin::SLUG, $credentials, $state );
		}
	}

	/**
	 * Refresh token when expired.
	 *
	 * @since 1.0.0
	 *
	 * @param array $account_data Account info.
	 *
	 * @return array
	 */
	private function refresh_token( $account_data ) {

		$response = wp_remote_post(
			$this->get_server_url(),
			[
				'body' => [
					'action' => 'refresh',
					'code'   => $account_data['code'],
					'token'  => $account_data['refresh_token'],
				],
			]
		);

		if ( is_wp_error( $response ) ) {
			return [];
		}

		$body = json_decode( wp_remote_retrieve_body( $response ), true );

		return is_array( $body ) && ! empty( $body['success'] ) ? array_merge( $account_data, array_intersect_key( $body, $account_data ) ) : [];
	}

	/**
	 * Get list of available accounts.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_accounts_list() {

		$accounts = $this->get_accounts();

		foreach ( $accounts as $account_id => $account ) {

			$accounts[ $account_id ] = $account['label'];
		}

		return $accounts;
	}

	/**
	 * Get data.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 *
	 * @return array
	 */
	public function get_data( $account_id ) {

		$accounts = $this->get_accounts();

		return ! empty( $accounts[ $account_id ] ) ? $accounts[ $account_id ] : [];
	}

	/**
	 * Get connection.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 *
	 * @return null|\WPFormsHubspot\Api\Api
	 */
	public function get_connection( $account_id ) {

		$accounts = $this->get_accounts();

		if ( ! isset( $accounts[ $account_id ] ) ) {
			return null;
		}

		$account = $accounts[ $account_id ];

		if ( empty( $account ) || empty( $account['access_token'] ) ) {
			return null;
		}

		if ( time() > $account['date'] + $account['expires_in'] ) {

			$account = $this->refresh_token( $account );

			if ( empty( $account ) ) {

				$this->set_expired( $account_id );

				return null;
			}

			unset( $account['token_expired'] );

			wpforms_update_providers_options( Plugin::SLUG, $account, $account_id );
		}

		return wpforms_hubspot()
			->get( 'client' )
			->new_connection( $account['access_token'] );
	}

	/**
	 * Check if exists.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 *
	 * @return bool
	 */
	public function is_exists( $account_id ) {

		return ! empty( $this->get_data( $account_id ) );
	}

	/**
	 * Get expired accounts.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_expired_accounts() {

		$expired = [];

		foreach ( $this->get_accounts() as $account ) {
			if ( empty( $account['token_expired'] ) ) {
				continue;
			}

			$expired[] = $account['label'];
		}

		return $expired;
	}

	/**
	 * Get accounts.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_accounts() {

		$all_providers = wpforms_get_providers_options();

		if ( empty( $all_providers[ Plugin::SLUG ] ) ) {
			return [];
		}

		$update = false;

		foreach ( $all_providers[ Plugin::SLUG ] as $account_id => $account ) {
			if ( ! empty( $account['access_token'] ) && ! empty( $account['label'] ) ) {
				continue;
			}

			unset( $all_providers[ Plugin::SLUG ][ $account_id ] );

			$update = true;
		}

		if ( $update ) {
			update_option( 'wpforms_providers', $all_providers );
		}

		return $all_providers[ Plugin::SLUG ];
	}

	/**
	 * Check if already connected.
	 *
	 * @since 1.0.0
	 *
	 * @param int $hub_id Account hub ID.
	 *
	 * @return bool
	 */
	private function is_connected( $hub_id ) {

		foreach ( $this->get_accounts() as $account ) {

			if ( (int) $account['hub_id'] === (int) $hub_id ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Retrieve a connect server URL.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function get_server_url() {

		if ( defined( 'WPFORMS_HUBSPOT_LOCAL_CONNECT_SERVER' ) && WPFORMS_HUBSPOT_LOCAL_CONNECT_SERVER && wpforms_debug() ) {
			return home_url() . self::HUBSPOT_CONNECT_ENDPOINT;
		}

		return self::WPFORMS_URL . self::HUBSPOT_CONNECT_ENDPOINT;
	}

	/**
	 * Set account as expired.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 */
	private function set_expired( $account_id ) {

		$accounts = $this->get_accounts();

		if ( ! isset( $accounts[ $account_id ] ) ) {
			return;
		}

		wpforms_log(
			'HubSpot token error.',
			'Token is expired for account: ' . $accounts[ $account_id ]['label'],
			[
				'type' => [ 'provider', 'error' ],
			]
		);

		$accounts[ $account_id ]['token_expired'] = true;

		wpforms_update_providers_options( Plugin::SLUG, $accounts[ $account_id ], $account_id );
	}
}
