<?php

namespace WPFormsHubspot\Provider\Actions;

use WPFormsHubspot\Api\Exceptions\ResponseException;
use WPFormsHubspot\Api\Exceptions\RequiredArgumentMissingException;

/**
 * Class ContactAction.
 *
 * @since 1.0.0
 */
class ContactAction extends Action {

	/**
	 * HS form fields.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $hs_form_fields = [];

	/**
	 * Run action.
	 *
	 * @since 1.0.0
	 *
	 * @throws ResponseException                Invalid response.
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	public function run() {

		$this->update_contact_information();
	}

	/**
	 * Update contact information.
	 *
	 * @since 1.0.0
	 *
	 * @throws ResponseException                Invalid response.
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	private function update_contact_information() {

		$this->hs_form_fields = array_merge( $this->map_required_fields(), $this->field_mapper->get_custom_fields( $this->api->get_contact_properties() ) );

		if ( wpforms_is_empty_string( $this->hs_form_fields['email'] ) ) {
			throw new RequiredArgumentMissingException( 'email' );
		}

		$email = $this->hs_form_fields['email'];

		$this->hs_form_fields['email'] = $this->update_contact_email_value( $email );

		$contact_response = $this->api->get_contact( $email );

		if ( $contact_response->get_response_code() !== 200 ) {

			$contact_response = $this->api->create_contact(
				[ 'email' => $this->hs_form_fields['email'] ]
			);
		} else {

			$contact_response = $this->api->update_contact(
				$email,
				[ 'email' => $this->hs_form_fields['email'] ]
			);
		}

		if ( $contact_response->has_errors() ) {
			throw new ResponseException( $contact_response->get_response_message() );
		}

		$this->form_submission();

		$contact_data = $contact_response->get_body();

		$this->update_contact_note( $contact_data['id'] );
	}

	/**
	 * Update contact email value.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email Contact email.
	 *
	 * @return string
	 *
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	private function update_contact_email_value( $email ) {

		$new_email_id = $this->field_mapper->get_optional_field_value( 'new_email', 'field_id' );

		if ( $new_email_id === '' ) {
			return $email;
		}

		$new_email = sanitize_email( $this->field_mapper->get_field_value_by_id( $new_email_id ) );

		if ( empty( $new_email ) || $new_email === $email ) {
			return $email;
		}

		return $new_email;
	}

	/**
	 * Get default lifecycle stage value.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_default_lifecycle_stage() {

		$default = 'lead';

		/**
		 * Filter a lifecycle stage default value while creating new contact.
		 *
		 * @since 1.0.0
		 *
		 * @param string $value Lifecycle stage value.
		 */
		$lifecyclestage = apply_filters( 'wpformshubspot_provider_actions_contact_action_get_default_lifecycle_stage', $default );

		if ( array_key_exists( $lifecyclestage, $this->api->get_lifecycle_stage_list() ) ) {
			return $lifecyclestage;
		}

		return $default;
	}

	/**
	 * Update contact note.
	 *
	 * @since 1.0.0
	 *
	 * @param string $contact_id Contact ID.
	 *
	 * @throws ResponseException Invalid response.
	 */
	private function update_contact_note( $contact_id ) {

		$note = sanitize_textarea_field( $this->field_mapper->get_optional_field_value( 'note', 'value' ) );

		if ( wpforms_is_empty_string( $note ) ) {
			return;
		}

		$note = wpforms_process_smart_tags( $note, $this->form_data, $this->fields, $this->field_mapper->get_entry_id() );

		$contact_response = $this->api->update_contact_note( $contact_id, nl2br( $note ) );

		if ( $contact_response->has_errors() ) {
			throw new ResponseException( $contact_response->get_response_message() );
		}
	}

	/**
	 * Send form submission.
	 *
	 * @since 1.0.0
	 *
	 * @throws ResponseException Invalid response.
	 */
	private function form_submission() {

		$account_data = wpforms_hubspot()->get( 'account' )->get_data( $this->connection_data['account_id'] );

		if ( empty( $account_data ) || empty( $account_data['access_token'] ) ) {
			return;
		}

		$fields = [];

		foreach ( $this->hs_form_fields as $key => $value ) {
			$fields[] = [
				'name'  => $key,
				'value' => $value,
			];
		}

		$hs_form_data = [
			'fields'  => $fields,
			'context' => [
				'pageUri'  => $this->page_url,
				'pageName' => $this->connection_data['name'],
			],
		];

		$tracking_cookie = $this->get_tracking_cookie();

		if ( $tracking_cookie ) {
			$hs_form_data['context']['hutk'] = $tracking_cookie;
		}

		if ( wpforms_is_collecting_ip_allowed( $this->form_data ) ) {
			$hs_form_data['context']['ipAddress'] = wpforms_get_ip();
		}

		$form_response = $this->api->form_submission( $this->connection_data, $hs_form_data );

		if ( $form_response->has_errors() ) {
			throw new ResponseException( $form_response->get_response_message() );
		}
	}

	/**
	 * Get HubSpot cookie if set by embedded tracking code.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_tracking_cookie() {

		return ! empty( $_COOKIE['hubspotutk'] ) ? sanitize_key( $_COOKIE['hubspotutk'] ) : '';
	}

	/**
	 * Map required fields.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 *
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	private function map_required_fields() {

		$required_fields['email']            = sanitize_email( $this->field_mapper->get_field_value_by_id( $this->field_mapper->get_required_field_value( 'email', 'field_id' ) ) );
		$required_fields['hs_lead_status']   = sanitize_text_field( $this->field_mapper->get_optional_field_value( 'hs_lead_status', 'list_id' ) );
		$required_fields['hubspot_owner_id'] = sanitize_text_field( $this->field_mapper->get_optional_field_value( 'hubspot_owner_id', 'list_id' ) );
		$required_fields['lifecyclestage']   = sanitize_text_field( $this->get_default_lifecycle_stage() );

		return $required_fields;
	}
}
