<?php

namespace WPFormsHubspot\Provider;

use WPFormsHubspot\Api\Exceptions\RequiredArgumentMissingException;

/**
 * HubSpot Field Mapper class.
 *
 * @since 1.0.0
 */
class FieldMapper {

	/**
	 * List of fields.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $fields;

	/**
	 * Form data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $form_data;

	/**
	 * Connection data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $connection_data;

	/**
	 * Entry ID.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	private $entry_id;

	/**
	 * FieldMapper constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields          Array of form fields.
	 * @param array $form_data       Form data and settings.
	 * @param array $connection_data Connection data.
	 * @param int   $entry_id        ID of a saved entry.
	 */
	public function __construct( $fields, $form_data, $connection_data, $entry_id ) {

		$this->fields          = $fields;
		$this->form_data       = $form_data;
		$this->connection_data = $connection_data;
		$this->entry_id        = $entry_id;
	}

	/**
	 * Get required field value.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Field name.
	 * @param string $key  Field key.
	 *
	 * @return string
	 *
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	public function get_required_field_value( $name, $key ) {

		if ( ! isset( $this->connection_data['fields_required'][ $name ][ $key ] ) ) {
			throw new RequiredArgumentMissingException( $name );
		}

		return (string) $this->connection_data['fields_required'][ $name ][ $key ];
	}

	/**
	 * Get optional field value.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Field name.
	 * @param string $key  Field key.
	 *
	 * @return string
	 */
	public function get_optional_field_value( $name, $key ) {

		if ( ! isset( $this->connection_data['fields_required'][ $name ][ $key ] ) ) {
			return '';
		}

		return (string) $this->connection_data['fields_required'][ $name ][ $key ];
	}

	/**
	 * Get field value by field ID.
	 *
	 * @since 1.0.0
	 *
	 * @param int $field_id Field ID.
	 *
	 * @return string
	 *
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	public function get_field_value_by_id( $field_id ) {

		if ( empty( $this->form_data['fields'][ $field_id ] ) ) {
			throw new RequiredArgumentMissingException( $field_id );
		}

		return (string) $this->fields[ $field_id ]['value'];
	}

	/**
	 * Get custom fields formatted values.
	 *
	 * @since 1.0.0
	 *
	 * @param array $properties Contact properties.
	 *
	 * @return array
	 */
	public function get_custom_fields( $properties ) {

		if ( empty( $this->connection_data['fields_meta'] ) ) {
			return [];
		}

		$custom_fields = [];

		foreach ( $this->connection_data['fields_meta'] as $field ) {

			$value = $this->get_formatted_value( $field['field_id'], $properties[ $field['name'] ] );

			if ( $value === null || wpforms_is_empty_string( $value ) ) {
				continue;
			}

			$custom_fields[ $field['name'] ] = $value;
		}

		return $custom_fields;
	}

	/**
	 * Get current entry ID.
	 *
	 * @since 1.0.0
	 *
	 * @return int
	 */
	public function get_entry_id() {

		return $this->entry_id;
	}

	/**
	 * Get custom field value formatted to HubSpot properties.
	 *
	 * @since 1.0.0
	 *
	 * @param int   $field_id Field ID.
	 * @param array $property Property data.
	 *
	 * @return string|null
	 */
	private function get_formatted_value( $field_id, $property ) {

		if ( ! isset( $this->fields[ $field_id ] ) || $this->fields[ $field_id ]['type'] === 'password' || ! $property['type'] ) {
			return null;
		}

		$mapping_method = 'map_' . $property['type'] . '_format';

		if ( method_exists( $this, $mapping_method ) ) {

			$value = wpforms_decode_string( $this->$mapping_method( $this->fields[ $field_id ]['value'], $property ) );

			return str_replace( [ "\r\n", "\r", "\n" ], ', ', $value );
		}

		return wpforms_decode_string( $this->map_string_format( $this->fields[ $field_id ]['value'], $property ) );
	}

	/**
	 * String format.
	 *
	 * @since 1.0.0
	 *
	 * @param string $value    Field value.
	 * @param array  $property Property data.
	 *
	 * @return string
	 */
	private function map_string_format( $value, $property ) {

		if ( $property['fieldType'] === 'file' ) {
			return esc_url_raw( (string) $value );
		}

		return (string) $value;
	}

	/**
	 * Date format.
	 *
	 * @since 1.0.0
	 *
	 * @param string $value    Field value.
	 * @param array  $property Property data.
	 *
	 * @return string|null
	 */
	private function map_date_format( $value, $property ) {

		$time = strtotime( $value );

		return $time ? gmdate( 'Y-m-d', $time ) : null;
	}

	/**
	 * Number format.
	 *
	 * @since 1.0.0
	 *
	 * @param string $value    Field value.
	 * @param array  $property Property data.
	 *
	 * @return float|null
	 */
	private function map_number_format( $value, $property ) {

		return is_numeric( $value ) ? (float) $value : null;
	}

	/**
	 * Category format.
	 *
	 * @since 1.0.0
	 *
	 * @param string $value    Field value.
	 * @param array  $property Property data.
	 *
	 * @return int|null
	 */
	private function map_enumeration_format( $value, $property ) {

		if ( empty( $property['options'] ) ) {
			return null;
		}

		$mapped_checkbox = $this->map_multi_value_checkbox( $value, $property );

		if ( $mapped_checkbox ) {
			return $mapped_checkbox;
		}

		foreach ( $property['options'] as $option ) {
			if ( ! isset( $option['label'], $option['value'] ) ) {
				continue;
			}

			if ( in_array( $value, [ $option['label'], $option['value'] ], true ) ) {
				return $option['value'];
			}
		}

		return null;
	}

	/**
	 * Map multiple values from the string with newline characters to values separated by semicolon.
	 *
	 * @since 1.0.1
	 *
	 * @param string $value    Field value.
	 * @param array  $property Property data.
	 *
	 * @return string|null
	 */
	private function map_multi_value_checkbox( $value, $property ) {

		$joined = [];

		if ( $property['fieldType'] === 'checkbox' ) {

			$multi = explode( PHP_EOL, $value );

			if ( count( $multi ) > 1 ) {
				foreach ( $multi as $one ) {
					$joined = $this->join_option( $property['options'], $one, $joined );
				}
			}

			if ( ! empty( $joined ) ) {
				return implode( ';', $joined );
			}
		}

		return null;
	}

	/**
	 * Join one option to array of multiple values.
	 *
	 * @since 1.0.1
	 *
	 * @param array  $options Multiple values.
	 * @param string $value   Field value.
	 * @param array  $joined  Processed values.
	 *
	 * @return array
	 */
	private function join_option( $options, $value, $joined ) {

		foreach ( $options as $option ) {
			if ( in_array( $value, [ $option['label'], $option['value'] ], true ) ) {
				$joined[] = $option['value'];
			}
		}

		return $joined;
	}
}
