<?php

namespace WPFormsHubspot\Provider\Settings;

use WPForms\Providers\Provider\Settings\FormBuilder as FormBuilderAbstract;
use WPForms\Providers\Provider\Status;
use WPFormsHubspot\Plugin;

/**
 * HubSpot Form Builder related functionality.
 *
 * @since 1.0.0
 */
class FormBuilder extends FormBuilderAbstract {

	/**
	 * Register all hooks (actions and filters).
	 *
	 * @since 1.0.0
	 */
	protected function init_hooks() { // phpcs:ignore WPForms.PHP.HooksMethod.InvalidPlaceForAddingHooks

		parent::init_hooks();

		static $ajax_events = [
			'ajax_accounts_get',
			'ajax_action_data_get',
			'ajax_connections_get',
		];

		array_walk(
			$ajax_events,
			static function ( $ajax_event, $key, $instance ) {

				add_filter(
					"wpforms_providers_settings_builder_{$ajax_event}_{$instance->core->slug}",
					[ $instance, $ajax_event ]
				);
			},
			$this
		);

		add_filter( 'wpforms_save_form_args', [ $this, 'save_form' ], 11, 3 );

		add_filter( 'wpforms_builder_save_form_response_data', [ $this, 'maybe_add_invalid_connection' ], 10, 3 );
	}

	/**
	 * Maybe add invalid connection identifier.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $response_data Response data.
	 * @param string $form_id       Form ID.
	 * @param array  $data          Form data.
	 */
	public function maybe_add_invalid_connection( $response_data, $form_id, $data ) {

		$saved_form   = wpforms()->get( 'form' )->get( $form_id, [ 'content_only' => true ] );
		$hubspot_data = isset( $saved_form['providers'][ Plugin::SLUG ] ) ? $saved_form['providers'][ Plugin::SLUG ] : [];

		foreach ( $hubspot_data as $connection ) {
			// If connection is saved without errors skip it.
			if ( ! isset( $connection['invalid'] ) || ! wpforms_hubspot()->get( 'account' )->is_exists( $connection['account_id'] ) ) {
				continue;
			}

			// Add flag to trigger warning in modal after form save.
			$response_data['hubspot_invalid'] = true;

			return $response_data;
		}

		return $response_data;
	}

	/**
	 * Pre-process provider data before saving it in form_data when editing form.
	 *
	 * @since 1.0.0
	 *
	 * @param array $form Form array, usable with wp_update_post.
	 * @param array $data Data retrieved from $_POST and processed.
	 * @param array $args Update form arguments.
	 *
	 * @return array
	 */
	public function save_form( $form, $data, $args ) {

		$form_data = json_decode( stripslashes( $form['post_content'] ), true );

		if ( ! empty( $form_data['providers'][ Plugin::SLUG ] ) ) {
			$modified_form_data = $this->modify_form_data( $form_data );

			if ( ! empty( $modified_form_data ) ) {
				$form['post_content'] = wpforms_encode( $modified_form_data );

				return $form;
			}
		}

		/*
		 * This part works when modification is locked or current filter was called on NOT Providers panel.
		 * Then we need to restore provider connections from the previous form content.
		 */

		// Get a "previous" form content (current content is still not saved).
		$prev_form = ! empty( $data['id'] ) ? wpforms()->get( 'form' )->get( $data['id'], [ 'content_only' => true ] ) : [];

		if ( ! empty( $prev_form['providers'][ Plugin::SLUG ] ) ) {
			$provider = $prev_form['providers'][ Plugin::SLUG ];

			if ( ! isset( $form_data['providers'] ) ) {
				$form_data = array_merge( $form_data, [ 'providers' => [] ] );
			}

			$form_data['providers'] = array_merge( (array) $form_data['providers'], [ Plugin::SLUG => $provider ] );
			$form['post_content']   = wpforms_encode( $form_data );
		}

		return $form;
	}

	/**
	 * Prepare modifications for the form content, if it's not locked.
	 *
	 * @since 1.0.0
	 *
	 * @param array $form_data Form content.
	 *
	 * @return array
	 */
	protected function modify_form_data( $form_data ) {

		$lock = '__lock__';

		/**
		 * Connection is locked.
		 * Why? User clicked the "Save" button when one of AJAX requests
		 * for retrieving data from API was in progress or failed.
		 */
		if (
			isset( $form_data['providers'][ Plugin::SLUG ][ $lock ] ) &&
			absint( $form_data['providers'][ Plugin::SLUG ][ $lock ] ) === 1
		) {
			return [];
		}

		foreach ( $form_data['providers'][ Plugin::SLUG ] as $connection_id => $connection ) {
			if ( $connection_id === $lock ) {
				unset( $form_data['providers'][ Plugin::SLUG ][ $lock ] );
				continue;
			}

			$connection_data = $this->sanitize_connection( $connection );

			$form_data['providers'][ Plugin::SLUG ][ $connection_id ] = HubspotFormHelper::send_form_connection_to_hubspot( $connection_data, $form_data['id'] );
		}

		return $form_data;
	}

	/**
	 * Sanitize connection.
	 *
	 * @since 1.0.0
	 *
	 * @param array $connection_data Connection data.
	 *
	 * @return array
	 */
	private function sanitize_connection( $connection_data ) {

		if ( ! empty( $connection_data['fields_required'] ) ) {
			$connection_data['fields_required'] = $this->sanitize_required_fields( $connection_data['fields_required'] );
		}

		if ( ! empty( $connection_data['fields_meta'] ) ) {
			$connection_data['fields_meta'] = $this->sanitize_custom_fields( $connection_data['fields_meta'] );
		}

		return $connection_data;
	}

	/**
	 * Sanitize required fields.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields List of fields.
	 *
	 * @return array
	 */
	private function sanitize_required_fields( $fields ) {

		foreach ( $fields as $key => $field ) {

			if ( isset( $field['field_id'] ) && $field['field_id'] === '' ) {
				continue;
			}

			if ( isset( $field['field_id'] ) ) {

				$fields[ $key ]['field_id'] = absint( $field['field_id'] );

				continue;
			}

			if ( isset( $field['list_id'] ) ) {
				$fields[ $key ]['list_id'] = sanitize_text_field( $field['list_id'] );

				continue;
			}

			if ( empty( $field['value'] ) ) {
				continue;
			}

			$fields[ $key ]['value'] = $field['name'] === 'note' ? sanitize_textarea_field( $field['value'] ) : sanitize_text_field( $field['value'] );
		}

		return $fields;
	}

	/**
	 * Sanitize custom fields.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields List of fields.
	 *
	 * @return array
	 */
	private function sanitize_custom_fields( $fields ) {

		foreach ( $fields as $key => $field ) {
			// Remove field if name or field_id is empty.
			if ( wpforms_is_empty_string( $field['name'] ) || wpforms_is_empty_string( $field['field_id'] ) ) {
				unset( $fields[ $key ] );
				continue;
			}

			$fields[ $key ]['name']     = sanitize_text_field( $field['name'] );
			$fields[ $key ]['field_id'] = absint( $field['field_id'] );
		}

		return $fields;
	}

	/**
	 * Get the list of all saved connections.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function ajax_connections_get() {

		$connections = [
			'connections'  => ! empty( $this->get_connections_data() ) ? array_reverse( $this->get_connections_data(), true ) : [],
			'conditionals' => [],
		];

		foreach ( $connections['connections'] as $connection ) {
			if ( empty( $connection['id'] ) ) {
				continue;
			}

			// This will either return an empty placeholder or complete set of rules, as a DOM.
			$connections['conditionals'][ $connection['id'] ] = wpforms_conditional_logic()
				->builder_block(
					[
						'form'       => $this->form_data,
						'type'       => 'panel',
						'parent'     => 'providers',
						'panel'      => Plugin::SLUG,
						'subsection' => $connection['id'],
						'reference'  => esc_html__( 'Marketing provider connection', 'wpforms-hubspot' ),
					],
					false
				);
		}

		$accounts = $this->ajax_accounts_get();

		return array_merge( $connections, $accounts );
	}

	/**
	 * Get the list of all accounts.
	 *
	 * @since 1.0.0
	 *
	 * @return array May return an empty sub-array.
	 */
	public function ajax_accounts_get() {

		return [
			'accounts' => $this->get_accounts(),
			'actions'  => $this->get_actions(),
		];
	}

	/**
	 * Retrieve saved provider accounts data.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_accounts() {

		return wpforms_hubspot()->get( 'account' )->get_accounts_list();
	}

	/**
	 * Retrieve saved provider connections data.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_connections_data() {

		return isset( $this->form_data['providers'][ Plugin::SLUG ] )
			? $this->form_data['providers'][ Plugin::SLUG ]
			: [];
	}

	/**
	 * Get list of actions.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_actions() {

		return [
			'contact'            => esc_html__( 'Create/Update Contact', 'wpforms-hubspot' ),
			'contact_delete'     => esc_html__( 'Delete Contact', 'wpforms-hubspot' ),
			'company_connect'    => esc_html__( 'Company Connect', 'wpforms-hubspot' ),
			'company_disconnect' => esc_html__( 'Company Disconnect', 'wpforms-hubspot' ),
		];
	}

	/**
	 * Retrieve HubSpot action data.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function ajax_action_data_get() {

		// phpcs:disable WordPress.Security.NonceVerification.Missing
		if ( empty( $_POST['account_id'] ) ) {
			return [];
		}

		$account_id = sanitize_text_field( wp_unslash( $_POST['account_id'] ) );

		if ( ! wpforms_hubspot()->get( 'account' )->is_exists( $account_id ) ) {
			return [];
		}
		// phpcs:enable WordPress.Security.NonceVerification.Missing

		return [
			'contact'            => $this->get_contact_fields( $account_id ),
			'contact_delete'     => $this->get_contact_delete_fields(),
			'company_connect'    => $this->get_company_connect_fields( $account_id ),
			'company_disconnect' => $this->get_company_disconnect_fields(),
		];
	}

	/**
	 * Get fields for the contact related actions.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 *
	 * @return array
	 */
	private function get_contact_fields( $account_id ) {

		$connection = wpforms_hubspot()->get( 'account' )->get_connection( $account_id );

		if ( empty( $connection ) ) {
			return [];
		}

		return [
			'required' => [
				'email'            => [
					'label'    => esc_html__( 'Email', 'wpforms-hubspot' ),
					'type'     => 'email_select',
					'required' => true,
				],
				'new_email'        => [
					'label' => esc_html__( 'New Email', 'wpforms-hubspot' ),
					'type'  => 'email_select',
				],
				'form_name'        => [
					'label'    => esc_html__( 'Form Name', 'wpforms-hubspot' ),
					'type'     => 'text',
					'default'  => $this->form_data['settings']['form_title'],
					'required' => true,
				],
				'hs_lead_status'   => [
					'label'       => esc_html__( 'Lead Status', 'wpforms-hubspot' ),
					'type'        => 'select',
					'options'     => $connection->get_lead_status_list(),
					'tooltip'     => esc_html__( 'The contact\'s sales, prospecting or outreach status.', 'wpforms-hubspot' ),
					'placeholder' => esc_html__( '--- Select Lead Status ---', 'wpforms-hubspot' ),
				],
				'hubspot_owner_id' => [
					'label'       => esc_html__( 'Contact Owner', 'wpforms-hubspot' ),
					'type'        => 'select',
					'options'     => $connection->get_contact_owner_list(),
					'tooltip'     => esc_html__( 'The owner of a contact.', 'wpforms-hubspot' ),
					'placeholder' => esc_html__( '--- Select Contact Owner ---', 'wpforms-hubspot' ),
				],
				'note'             => [
					'label' => esc_html__( 'Note', 'wpforms-hubspot' ),
					'type'  => 'paragraph',
				],
			],
			'optional' => $connection->get_contact_properties_options( true ),
			'groups'   => $connection->get_contact_properties_groups(),
		];
	}

	/**
	 * Get fields for the contact delete action.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_contact_delete_fields() {

		return [
			'required' => [
				'email' => [
					'label'    => esc_html__( 'Email', 'wpforms-hubspot' ),
					'type'     => 'email_select',
					'required' => true,
				],
			],
			'optional' => [],
		];
	}

	/**
	 * Get fields for the company connect action.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 *
	 * @return array
	 */
	private function get_company_connect_fields( $account_id ) {

		$connection = wpforms_hubspot()->get( 'account' )->get_connection( $account_id );

		if ( empty( $connection ) ) {
			return [];
		}

		return [
			'required' => [
				'email'   => [
					'label'    => esc_html__( 'Email', 'wpforms-hubspot' ),
					'type'     => 'email_select',
					'required' => true,
				],
				'company' => [
					'label'       => esc_html__( 'Company', 'wpforms-hubspot' ),
					'type'        => 'select',
					'required'    => true,
					'options'     => $connection->get_companies_list(),
					'placeholder' => esc_html__( '--- Select Company ---', 'wpforms-hubspot' ),
				],
			],
			'optional' => [],
		];
	}

	/**
	 * Get fields for the company disconnect action.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_company_disconnect_fields() {

		return [
			'required' => [
				'email' => [
					'label'    => esc_html__( 'Email', 'wpforms-hubspot' ),
					'type'     => 'email_select',
					'required' => true,
				],
			],
			'optional' => [],
		];
	}

	/**
	 * Use this method to register own templates for form builder.
	 * Make sure, that you have `tmpl-` in template name in `<script id="tmpl-*">`.
	 *
	 * @since 1.0.0
	 */
	public function builder_custom_templates() {

		// phpcs:disable WordPress.Security.EscapeOutput.OutputNotEscaped
		?>
		<!-- Single HubSpot connection. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'connection' ); ?>
		</script>

		<!-- Single HubSpot connection sub-block: CUSTOM FIELDS -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-fields">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'fields' ); ?>
		</script>

		<!-- Single HubSpot connection block: EMAIL SELECT FIELD. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-email-select-field">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'email-select-field' ); ?>
		</script>

		<!-- Single HubSpot connection block: SELECT FIELD. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-select-field">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'select-field' ); ?>
		</script>

		<!-- Single HubSpot connection block: PARAGRAPH FIELD. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-paragraph-field">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'paragraph-field' ); ?>
		</script>

		<!-- Single HubSpot connection block: TEXT FIELD. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-text-field">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'text-field' ); ?>
		</script>

		<!-- Single HubSpot connection block: ERROR. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-error">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'error' ); ?>
		</script>

		<!-- Single HubSpot connection block: LOCK. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-lock">
			<?php echo wpforms_hubspot()->get( 'template' )->get_builder_template( 'lock' ); ?>
		</script>
		<?php
		// phpcs:enable WordPress.Security.EscapeOutput.OutputNotEscaped
	}

	/**
	 * Enqueue JavaScript and CSS files.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_assets() {

		parent::enqueue_assets();

		$min = wpforms_get_min_suffix();

		wp_enqueue_script(
			'wpforms-hubspot-admin-builder',
			WPFORMS_HUBSPOT_URL . "assets/js/hubspot-builder$min.js",
			[ 'wpforms-admin-builder-providers' ],
			WPFORMS_HUBSPOT_VERSION,
			true
		);

		$invalid_connection_message = sprintf(
			wp_kses( /* translators: %s - HubSpot Account suspension documentation page URL. */
				__( 'We noticed that your <a href="%s" target="_blank" rel="noopener noreferrer">HubSpot account has been suspended.</a> While your form will still function normally, contacts won’t be created in your HubSpot account.<br>When HubSpot have restored your account, please click the Save button in the form builder to remove this alert.', 'wpforms-hubspot' ),
				[
					'a'  => [
						'href'   => [],
						'rel'    => [],
						'target' => [],
					],
					'br' => [],
				]
			),
			'https://knowledge.hubspot.com/account-settings/account-suspension-faq'
		);

		wp_localize_script(
			'wpforms-hubspot-admin-builder',
			'wpformsHubSpotBuilderVars',
			[
				'l10n' => [
					'provider_placeholder' => esc_html__( '--- Select HubSpot Field ---', 'wpforms-hubspot' ),
					'invalid_connection'   => $invalid_connection_message,
				],
			]
		);

		wp_enqueue_style(
			'wpforms-hubspot-admin-builder',
			WPFORMS_HUBSPOT_URL . "assets/css/wpforms-hubspot-admin-builder$min.css",
			[],
			WPFORMS_HUBSPOT_VERSION
		);
	}

	/**
	 * Display provider screen.
	 *
	 * @since 1.0.0
	 */
	public function display_content() {

		if ( $this->is_configured() ) {

			parent::display_content();

			return;
		}
		?>

		<div class="wpforms-panel-content-section wpforms-builder-provider wpforms-panel-content-section-<?php echo esc_attr( $this->core->slug ); ?>"
			 id="<?php echo esc_attr( $this->core->slug ); ?>-provider"
			 data-provider="<?php echo esc_attr( $this->core->slug ); ?>">

			<?php $this->display_content_header(); ?>

			<div class="wpforms-builder-provider-connections-default">
				<img src="<?php echo esc_url( $this->core->icon ); ?>" alt="<?php esc_attr_e( 'Connect WPForms to HubSpot.', 'wpforms-hubspot' ); ?>">
				<div class="wpforms-builder-provider-settings-default-content">
					<p><?php esc_html_e( 'Get the most out of WPForms — use it with an active HubSpot account.', 'wpforms-hubspot' ); ?></p>
					<button type="button" class="wpforms-btn wpforms-btn-md wpforms-btn-orange wpforms-hubspot-auth"><?php esc_html_e( 'Get HubSpot Free', 'wpforms-hubspot' ); ?></button>
				</div>
			</div>
		</div>

		<?php
	}

	/**
	 * Section content header.
	 *
	 * @since 1.0.0
	 */
	protected function display_content_header() {

		if ( $this->is_configured() ) {
			parent::display_content_header();

			$this->check_connection();

			return;
		}
		?>

		<div class="wpforms-builder-provider-title wpforms-panel-content-section-title">
			<?php echo esc_html( $this->core->name ); ?>

			<button type="button" class="wpforms-builder-provider-title-add wpforms-hubspot-auth">
				<?php esc_html_e( 'Add New Account', 'wpforms-hubspot' ); ?>
			</button>
		</div>

		<?php
		$this->check_connection();
	}

	/**
	 * Check if account configured.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function is_configured() {

		return Status::init( $this->core->slug )->is_configured() && ! empty( wpforms_hubspot()->get( 'account' )->get_accounts_list() );
	}

	/**
	 * Check connection.
	 *
	 * @since 1.0.0
	 */
	private function check_connection() {

		foreach ( wpforms_hubspot()->get( 'account' )->get_expired_accounts() as $account_label ) {

			echo '<p class="wpforms-alert wpforms-alert-warning wpforms-hubspot-expired-accounts-warning">';

			printf(
				/* translators: %s - Account label. */
				esc_html__( 'Heads up! HubSpot account %s connection is expired. Tokens must be refreshed.', 'wpforms-hubspot' ),
				'<strong>' . esc_html( $account_label ) . '</strong>'
			);
			echo '</p>';
		}
	}
}
