<?php

namespace WPFormsHubspot\Provider\Settings;

use WPFormsHubspot\Plugin;

/**
 * HubSpot Form related functionality.
 *
 * @since 1.0.0
 */
class HubspotFormHelper {

	/**
	 * Send form connection to HubSpot.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $connection_data Connection data.
	 * @param string $form_id         Form ID.
	 *
	 * @return array
	 */
	public static function send_form_connection_to_hubspot( $connection_data, $form_id ) {

		if ( empty( $connection_data['action'] ) || $connection_data['action'] !== 'contact' ) {
			return $connection_data;
		}

		$connection = wpforms_hubspot()->get( 'account' )->get_connection( $connection_data['account_id'] );

		if ( empty( $connection ) ) {
			return $connection_data;
		}

		$saved_form      = ! empty( $form_id ) ? wpforms()->get( 'form' )->get( $form_id, [ 'content_only' => true ] ) : [];
		$saved_form_data = isset( $saved_form['providers'][ Plugin::SLUG ][ $connection_data['id'] ] ) ? $saved_form['providers'][ Plugin::SLUG ][ $connection_data['id'] ] : [];

		$hs_form_data['name']                        = 'WPForms - ' . $connection_data['fields_required']['form_name']['value'] . ' - ' . $connection_data['name'] . ' [' . $connection_data['id'] . '] ';
		$hs_form_data['formType']                    = 'HUBSPOT';
		$hs_form_data['formFieldGroups'][]['fields'] = self::generate_hs_form_fields( $connection_data, $connection->get_contact_properties() );

		$hs_form_hash = md5( wp_json_encode( array_merge( $hs_form_data, [ 'account_id' => $connection_data['account_id'] ] ) ) );

		if ( isset( $saved_form_data['hs_hash'] ) && $saved_form_data['hs_hash'] === $hs_form_hash ) {
			$connection_data['hs_form_id']   = $saved_form_data['hs_form_id'];
			$connection_data['hs_portal_id'] = $saved_form_data['hs_portal_id'];

			return $connection_data;
		}

		$hs_form_data['createdAt'] = time();

		// Maybe set HS form id from already saved form.
		$connection_data['hs_form_id'] = ! empty( $saved_form_data['hs_form_id'] ) ? $saved_form_data['hs_form_id'] : $connection_data['hs_form_id'];

		$hs_form_response = ! empty( $connection_data['hs_form_id'] ) ? $connection->update_form( $connection_data['hs_form_id'], $hs_form_data ) : $connection->create_form( $hs_form_data );

		if ( $hs_form_response->has_errors() ) {

			wpforms_log(
				'HubSpot form response error.',
				esc_html( $hs_form_response->get_response_message() ),
				[
					'type'    => [ 'provider', 'error' ],
					'form_id' => $form_id,
				]
			);

			if ( $hs_form_response->get_response_code() === 403 ) {
				$connection_data['invalid'] = true;
			}

			return $connection_data;
		}

		$hs_form_data = $hs_form_response->get_body();

		$connection_data['hs_form_id']   = $hs_form_data['guid'];
		$connection_data['hs_portal_id'] = $hs_form_data['portalId'];
		$connection_data['hs_hash']      = $hs_form_hash;

		unset( $connection_data['invalid'] );

		return $connection_data;
	}

	/**
	 * Generate HubSpot form fields data.
	 *
	 * @since 1.0.0
	 *
	 * @param array $connection_data Connection data.
	 * @param array $properties      Field properties.
	 *
	 * @return array
	 */
	private static function generate_hs_form_fields( $connection_data, $properties ) {

		$fields[] = self::generate_hs_form_field(
			'email',
			[
				'type'       => 'text',
				'required'   => true,
				'validation' => [
					'useDefaultBlockList'   => false,
					'blockedEmailAddresses' => [],
				],
			],
			$properties
		);
		$fields[] = self::generate_hs_form_field(
			'hs_lead_status',
			[
				'type'     => 'text',
				'required' => false,
				'hidden'   => true,
				'default'  => isset( $connection_data['fields_required']['hs_lead_status']['list_id'] ) ? $connection_data['fields_required']['hs_lead_status']['list_id'] : '',
			],
			$properties
		);
		$fields[] = self::generate_hs_form_field(
			'hubspot_owner_id',
			[
				'type'     => 'text',
				'required' => false,
				'hidden'   => true,
				'default'  => isset( $connection_data['fields_required']['hubspot_owner_id']['list_id'] ) ? $connection_data['fields_required']['hubspot_owner_id']['list_id'] : '',
			],
			$properties
		);

		foreach ( $connection_data['fields_meta'] as $field ) {

			if ( ! isset( $properties[ $field['name'] ] ) ) {
				continue;
			}

			$fields[] = self::generate_hs_form_field( $field['name'], [], $properties );
		}

		return $fields;
	}

	/**
	 * Generate HubSpot form field data.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name           Field name.
	 * @param array  $field_settings Field settings.
	 * @param array  $properties     Field properties.
	 *
	 * @return array
	 */
	private static function generate_hs_form_field( $name, $field_settings, $properties ) {

		$hs_field_data['name']         = $name;
		$hs_field_data['label']        = isset( $properties[ $name ]['label'] ) ? $properties[ $name ]['label'] : '';
		$hs_field_data['description']  = isset( $properties[ $name ]['description'] ) ? $properties[ $name ]['description'] : '';
		$hs_field_data['options']      = isset( $properties[ $name ]['options'] ) ? $properties[ $name ]['options'] : [];
		$hs_field_data['validation']   = isset( $field_settings['validation'] ) ? $field_settings['validation'] : [];
		$hs_field_data['fieldType']    = self::convert_field_type( isset( $field_settings['type'] ) ? $field_settings['type'] : $properties[ $name ]['fieldType'] );
		$hs_field_data['defaultValue'] = isset( $field_settings['default'] ) ? $field_settings['default'] : '';
		$hs_field_data['required']     = isset( $field_settings['required'] ) ? $field_settings['required'] : false;
		$hs_field_data['hidden']       = isset( $field_settings['hidden'] ) ? $field_settings['hidden'] : false;
		$hs_field_data['objectTypeId'] = '0-1';

		return $hs_field_data;
	}

	/**
	 * Convert field type to HubSpot form type.
	 *
	 * @since 1.0.0
	 *
	 * @param string $type Field type.
	 *
	 * @return string
	 */
	private static function convert_field_type( $type ) {

		if ( empty( $type ) ) {
			return 'text';
		}

		if ( $type === 'select' ) {
			return 'dropdown';
		}

		return $type;
	}
}
