<?php

namespace WPFormsHubspot\Provider\Settings;

use WPForms\Providers\Provider\Settings\PageIntegrations as PageIntegrationsAbstract;
use WPFormsHubspot\Api\Api;
use WPForms\Admin\Notice;

/**
 * HubSpot addon settings.
 *
 * @since 1.0.0
 */
class PageIntegrations extends PageIntegrationsAbstract {

	/**
	 * Integrations hooks.
	 *
	 * @since 1.0.0
	 */
	public function hooks() {

		add_action( 'wpforms_settings_init', [ $this, 'display_notice' ] );
	}

	/**
	 * Display admin error notice if something wrong with the settings.
	 *
	 * @since 1.0.0
	 */
	public function display_notice() {

		if ( ! wpforms_is_admin_page( 'settings', 'integrations' ) ) {
			return;
		}

		foreach ( wpforms_hubspot()->get( 'account' )->get_expired_accounts() as $account_label ) {

			Notice::error(
				sprintf(
					/* translators: %s - Account label. */
					esc_html__( 'Heads up! HubSpot account %s connection is expired. Tokens must be refreshed.', 'wpforms-hubspot' ),
					'<strong>' . esc_html( $account_label ) . '</strong>'
				)
			);
		}

	}

	/**
	 * AJAX to disconnect a provider from the settings integrations tab.
	 *
	 * @since 1.0.0
	 */
	public function ajax_disconnect() {

		$providers = wpforms_get_providers_options();

		if (
			! empty( $_POST['provider'] ) &&
			! empty( $_POST['key'] ) &&
			! empty( $providers[ $_POST['provider'] ][ $_POST['key'] ] ) &&
			check_ajax_referer( 'wpforms-admin', 'nonce', false ) &&
			wpforms_current_user_can()
		) {

			wp_safe_remote_request(
				Api::API_OBJECTS_URL . '/oauth/v1/refresh-tokens/' . $providers[ sanitize_key( $_POST['provider'] ) ][ sanitize_text_field( wp_unslash( $_POST['key'] ) ) ]['access_token'], // do not use sanitize_key because it will remove '.' from the key.
				[
					'method' => 'DELETE',
				]
			);
		}

		parent::ajax_disconnect();
	}

	/**
	 * Display HubSpot new account connection button.
	 *
	 * @since 1.0.0
	 */
	protected function display_add_new() {
		?>
		<p>
			<a class="wpforms-btn wpforms-btn-md wpforms-btn-light-grey wpforms-hubspot-auth" href="#">
				<i class="fa fa-plus"></i> <?php esc_html_e( 'Add New Account', 'wpforms-hubspot' ); ?>
			</a>
		</p>
		<p>
			<?php
			printf(
				'<a href="%s" target="_blank" rel="noopener noreferrer" class="secondary-text">%s</a>',
				esc_url( wpforms_utm_link( 'https://wpforms.com/docs/how-to-install-and-use-the-hubspot-addon-in-wpforms/', 'Settings - Integration', 'HubSpot Documentation' ) ),
				esc_html__( 'Click here for documentation on connecting WPForms with HubSpot.', 'wpforms-hubspot' )
			);
			?>
		</p>
		<?php
	}
}
