/* global WPForms, wpforms_builder, Choices, wpforms_builder_providers, wpformsMailerLiteBuilderVars, wpf */
'use strict';

/**
 * WPForms Providers Builder MailerLite module.
 *
 * @since 1.0.0
 */
WPForms.Admin.Builder.Providers.MailerLite = WPForms.Admin.Builder.Providers.MailerLite || ( function( document, window, $ ) {

	/**
	 * Private functions and properties.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	var __private = {

		/**
		 * jQuery object for holder.
		 *
		 * @since 1.0.0
		 *
		 * @type {jQuery}
		 */
		$holder: null,

		/**
		 * Config contains all configuration properties.
		 *
		 * @since 1.0.0
		 *
		 * @type {object}
		 */
		config: {

			/**
			 * List of MailerLite templates that should be compiled.
			 *
			 * @since 1.0.0
			 *
			 * @type {Array}
			 */
			templates: [
				'wpforms-mailerlite-builder-content-connection',
				'wpforms-mailerlite-builder-content-connection-required-email-select-field',
				'wpforms-mailerlite-builder-content-connection-required-select-field',
				'wpforms-mailerlite-builder-content-connection-required-multiselect-field',
				'wpforms-mailerlite-builder-content-connection-required-text-field',
				'wpforms-mailerlite-builder-content-connection-error',
				'wpforms-mailerlite-builder-content-connection-lock',
				'wpforms-mailerlite-builder-content-connection-conditionals',
				'wpforms-mailerlite-builder-content-connection-custom-fields-label',
			],
		},

		/**
		 * Sometimes in DOM we might have placeholders or temporary connection IDs.
		 * We need to replace them with actual values.
		 *
		 * @since 1.0.0
		 *
		 * @param {string} connectionId New connection ID to replace to.
		 * @param {object} $connection  jQuery DOM connection element.
		 */
		replaceConnectionIds: function( connectionId, $connection ) {

			// Replace old temporary %connection_id% from PHP code with the new one.
			$connection.find( 'input, select, label' ).each( function() {

				var $this = $( this );

				if ( $this.attr( 'name' ) ) {
					$this.attr( 'name', $this.attr( 'name' ).replace( /%connection_id%/gi, connectionId ) );
				}

				if ( $this.attr( 'id' ) ) {
					$this.attr( 'id', $this.attr( 'id' ).replace( /%connection_id%/gi, connectionId ) );
				}

				if ( $this.attr( 'for' ) ) {
					$this.attr( 'for', $this.attr( 'for' ).replace( /%connection_id%/gi, connectionId ) );
				}

				if ( $this.attr( 'data-name' ) ) {
					$this.attr( 'data-name', $this.attr( 'data-name' ).replace( /%connection_id%/gi, connectionId ) );
				}
			} );
		},

		/**
		 * Whether we have an account ID in a list of all available accounts.
		 *
		 * @since 1.0.0
		 *
		 * @param {string} connectionAccID  Connection account ID to check.
		 * @param {object}  accounts        Array of objects, usually received from API.
		 *
		 * @returns {boolean} True if connection already exists.
		 */
		connectionAccountExists: function( connectionAccID, accounts ) {

			if ( _.isEmpty( accounts ) ) {
				return false;
			}

			// New connections, that have not been saved don't have the account ID yet.
			if ( _.isEmpty( connectionAccID ) ) {
				return true;
			}

			return _.has( accounts, connectionAccID );
		},
	};

	/**
	 * Public functions and properties.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	var app = {

		/**
		 * Current provider slug.
		 *
		 * @since 1.0.0
		 *
		 * @type {string}
		 */
		provider: 'mailerlite',

		/**
		 * This is a shortcut to the WPForms.Admin.Builder.Providers object,
		 * that handles the parent all-providers functionality.
		 *
		 * @since 1.0.0
		 *
		 * @type {object}
		 */
		Providers: {},

		/**
		 * This is a shortcut to the WPForms.Admin.Builder.Templates object,
		 * that handles all the template management.
		 *
		 * @since 1.0.0
		 *
		 * @type {object}
		 */
		Templates: {},

		/**
		 * This is a shortcut to the WPForms.Admin.Builder.Providers.cache object,
		 * that handles all the cache management.
		 *
		 * @since 1.0.0
		 *
		 * @type {object}
		 */
		Cache: {},

		/**
		 * This is a flag for ready state.
		 *
		 * @since 1.0.0
		 *
		 * @type {boolean}
		 */
		isReady: false,

		/**
		 * Start the engine.
		 *
		 * Run initialization on providers panel only.
		 *
		 * @since 1.0.0
		 */
		init: function() {

			// We are requesting/loading a Providers panel.
			if ( wpf.getQueryString( 'view' ) === 'providers' ) {
				$( '#wpforms-panel-providers' ).on( 'WPForms.Admin.Builder.Providers.ready', app.ready );
			}

			// We have switched to Providers panel.
			$( document ).on( 'wpformsPanelSwitched', function( _event, panel ) {

				if ( panel === 'providers' ) {
					app.ready();
				}
			} );
		},

		/**
		 * Initialized once the DOM and Providers are fully loaded.
		 *
		 * @since 1.0.0
		 */
		ready: function() {

			if ( app.isReady ) {
				return;
			}

			// Done by reference, so we are not doubling memory usage.
			app.Providers = WPForms.Admin.Builder.Providers;
			app.Templates = WPForms.Admin.Builder.Templates;
			app.Cache     = app.Providers.cache;

			// Save a jQuery object for the provider holder.
			__private.$holder = app.Providers.getProviderHolder( app.provider ).find( '.wpforms-builder-provider-connections' );

			// Register custom Underscore.js templates.
			app.Templates.add( __private.config.templates );

			// Register a handler for the "Add New Account" process.
			app.Providers.ui.account.registerAddHandler( app.provider, app.processAccountAdd );

			// Events registration.
			app.bindUIActions();
			app.bindTriggers();

			app.processInitial();

			$( document ).on( 'wpformsSaved', app.processEmptyGroupsError );

			// Save a flag for ready state.
			app.isReady = true;
		},

		/**
		 * Process various events as a response to UI interactions.
		 *
		 * @since 1.0.0
		 */
		bindUIActions: function() {

			app
				.Providers.getProviderHolder( app.provider )
				.on( 'connectionCreate', app.connection.callbacks.create )
				.on( 'connectionDelete', app.connection.callbacks.delete )
				.on( 'change', '.js-wpforms-builder-mailerlite-provider-connection-account', app.ui.account.changeCallback )
				.on( 'change', '.js-wpforms-builder-mailerlite-provider-connection-action', app.ui.action.changeCallback );
		},

		/**
		 * Fire certain events on certain actions, specific for related connections.
		 * These are not directly caused by user manipulations.
		 *
		 * @since 1.0.0
		 */
		bindTriggers: function() {

			// Connection data was loaded.
			__private.$holder.on( 'connectionsDataLoaded', function( _event, data ) {

				if ( _.isEmpty( data.connections ) ) {
					return;
				}

				for ( var connectionId in data.connections ) {
					app.connection.callbacks.generate( {
						connection: data.connections[ connectionId ],
						conditional: data.conditionals[ connectionId ],
					} );
				}
			} );

			// Connection was generated.
			__private.$holder.on( 'connectionGenerated', function( _event, data ) {

				var $connection = app.connection.getById( data.connection.id );

				if ( _.has( data.connection, 'isNew' ) && data.connection.isNew ) {

					// Run replacing temporary connection ID, if it's a new connection.
					__private.replaceConnectionIds( data.connection.id, $connection );
					return;
				}

				$( '.js-wpforms-builder-mailerlite-provider-connection-action', $connection ).trigger( 'change', [ $connection ] );
			} );
		},

		/**
		 * Compile template with data if any and display them on a page.
		 *
		 * @since 1.0.0
		 */
		processInitial: function() {

			__private.$holder.prepend( app.tmpl.callbacks.commonsHTML() );
			app.connection.callbacks.dataLoad();
		},

		/**
		 * Process the account creation in FormBuilder.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} modal jQuery-Confirm modal object.
		 *
		 * @returns {boolean} False.
		 */
		processAccountAdd: function( modal ) {

			var $confirmBtn  = modal.$$add;

			if ( $confirmBtn.prop( 'disabled' ) ) {
				return false;
			}

			var $error       = modal.$content.find( '.form-error' ),
				$apiKeyField = modal.$content.find( 'input[name="api_key"]' ),
				$apiKeyError = $apiKeyField.siblings( '.error' ),
				apiKey       = $apiKeyField.val().trim(),
				accountName  = modal.$content.find( 'input[name="account_name"]' ).val().trim();

			$confirmBtn.prop( 'disabled', 'disabled' );

			// Checking a required data.
			if ( _.isEmpty( apiKey ) ) {
				modal.setType( 'red' );

				$apiKeyField.addClass( 'wpforms-error' );
				$confirmBtn.removeProp( 'disabled' ).removeAttr( 'disabled' );

				$apiKeyError.text( wpforms_builder_providers.required_field ).show();

				return false;
			}

			// Required data exists and not empty.
			modal.$content.find( '.wpforms-error' ).removeClass( 'wpforms-error' );
			$apiKeyError.hide();
			$error.hide();
			modal.setType( 'blue' );

			app
				.Providers.ajax
				.request( app.provider, {
					data: {
						'task': 'account_save',
						data: {
							'api_key': apiKey,
							'account_name': accountName,
						},
					},
				} )
				.done( function( response ) {

					if ( response.success ) {
						app.Providers.getProviderHolder( app.provider )
							.find( '.wpforms-builder-provider-title-add' )
							.toggleClass( 'hidden' );
						modal.close();

						return;
					}

					if ( _.has( response, 'data' ) && _.has( response.data, 'error_msg' ) ) {
						$error.html( response.data.error_msg );
					}

					$error.show();
					$confirmBtn.removeProp( 'disabled' ).removeAttr( 'disabled' );
				} );

			return false;
		},

		/**
		 * Transpose Choices.js select errors to it's parent container.
		 *
		 * @since 1.0.0
		 */
		processEmptyGroupsError: function() {

			var $select = $( '.js-wpforms-builder-mailerlite-provider-item-select.choices__input' );

			$select.each( function( _index, element ) {

				$( this ).hasClass( 'wpforms-required' ) && $( this ).val().length === 0 ?
					$( element ).parent().parent().addClass( 'wpforms-error' ) :
					$( element ).parent().parent().removeClass( 'wpforms-error' );
			} );
		},

		/**
		 * Connection property.
		 *
		 * @since 1.0.0
		 */
		connection: {

			/**
			 * Sometimes we might need to a get a connection DOM element by its ID.
			 *
			 * @since 1.0.0
			 *
			 * @param {string} connectionId Connection ID to search for a DOM element by.
			 *
			 * @returns {jQuery} jQuery object for connection.
			 */
			getById: function( connectionId ) {

				return __private.$holder.find( '.wpforms-builder-provider-connection[data-connection_id="' + connectionId + '"]' );
			},

			/**
			 * Connection methods.
			 *
			 * @since 1.0.0
			 */
			callbacks: {

				/**
				 * Create a connection using the user entered name.
				 *
				 * @since 1.0.0
				 *
				 * @param {object} _event Event object.
				 * @param {string} name   Connection name.
				 */
				create: function( _event, name ) {

					var connectionId = new Date().getTime().toString( 16 ),
						connection   = {
							id: connectionId,
							name: name,
							isNew: true,
						};

					app.Cache.addTo( app.provider, 'connections', connectionId, connection );

					app.connection.callbacks.generate( {
						connection: connection,
					} );
				},

				/**
				 * Connection is deleted - delete a cache as well.
				 *
				 * @since 1.0.0
				 *
				 * @param {object} _event      Event object.
				 * @param {object} $connection jQuery DOM element for a connection.
				 */
				delete: function( _event, $connection ) {

					var $eHolder = app.Providers.getProviderHolder( app.provider );

					if ( ! $connection.closest( $eHolder ).length ) {
						return;
					}

					var connectionId = $connection.data( 'connection_id' );

					if ( _.isString( connectionId ) ) {
						app.Cache.deleteFrom( app.provider, 'connections', connectionId );
					}
				},

				/**
				 * Get the template and data for a connection and process it.
				 *
				 * @since 1.0.0
				 *
				 * @param {object} data Connection data.
				 *
				 * @returns {Function} Connection render method.
				 */
				generate: function( data ) {

					var accounts = app.Cache.get( app.provider, 'accounts' ),
						actions  = app.Cache.get( app.provider, 'actions' );

					/*
					 * We may or may not receive accounts previously.
					 * If yes - render instantly, if no - request them via AJAX.
					 */
					if ( ! _.isEmpty( accounts ) && __private.connectionAccountExists( data.connection.account_id, accounts ) ) {

						return app.connection.callbacks.renderConnections( accounts, actions, data );
					}

					// We need to get the live list of accounts, as nothing is in cache.
					app
						.Providers.ajax
						.request( app.provider, {
							data: {
								task: 'accounts_get',
							},
						} )
						.done( function( response ) {

							if (
								! response.success ||
								! _.has( response.data, 'accounts' ) ||
								! _.has( response.data, 'groups' )  ||
								! _.has( response.data, 'fields' )
							) {
								return;
							}

							app.Cache.set( app.provider, 'accounts', response.data.accounts );
							app.Cache.set( app.provider, 'groups', response.data.groups );
							app.Cache.set( app.provider, 'fields', response.data.fields );

							app.connection.callbacks.renderConnections( response.data.accounts, actions, data );
						} );
				},

				/**
				 * Render connections.
				 *
				 * @since 1.0.0
				 *
				 * @param {object} accounts List of accounts.
				 * @param {object} actions List of actions.
				 * @param {object} data Connection data.
				 */
				renderConnections: function( accounts, actions, data ) {

					var tmplConnection  = app.Templates.get( 'wpforms-' + app.provider + '-builder-content-connection' ),
						tmplConditional = $( '#tmpl-wpforms-' + app.provider + '-builder-content-connection-conditionals' ).length ?
							app.Templates.get( 'wpforms-' + app.provider + '-builder-content-connection-conditionals' ) :
							app.Templates.get( 'wpforms-providers-builder-content-connection-conditionals' ),
						conditional     = _.has( data.connection, 'isNew' ) && data.connection.isNew ? tmplConditional() : data.conditional;

					if ( ! __private.connectionAccountExists( data.connection.account_id, accounts ) ) {
						return;
					}

					__private
						.$holder
						.prepend(
							tmplConnection( {
								accounts: accounts,
								actions: actions,
								connection: data.connection,
								conditional: conditional,
								provider: app.provider,
							} ) );

					// When we are done adding a new connection with its accounts - trigger next steps.
					__private.$holder.trigger( 'connectionGenerated', [ data ] );
				},

				/**
				 * Fire AJAX-request to retrieve the list of all saved connections.
				 *
				 * @since 1.0.0
				 */
				dataLoad: function() {

					app
						.Providers.ajax
						.request( app.provider, {
							data: {
								task: 'connections_get',
							},
						} )
						.done( function( response ) {

							if (
								! response.success ||
								! _.has( response.data, 'connections' )
							) {
								return;
							}

							// Save response data to "cache" as a copy.
							app.Cache.set( app.provider, 'connections', jQuery.extend( {}, response.data.connections ) );
							app.Cache.set( app.provider, 'conditionals', jQuery.extend( {}, response.data.conditionals ) );
							app.Cache.set( app.provider, 'actions', jQuery.extend( {}, response.data.actions ) );
							app.Cache.set( app.provider, 'actions_fields', jQuery.extend( {}, response.data.actions_fields ) );

							if ( ! _.isEmpty( response.data.accounts ) ) {
								app.Cache.set( app.provider, 'accounts', jQuery.extend( {}, response.data.accounts ) );
							}

							if ( ! _.isEmpty( response.data.groups ) ) {
								app.Cache.set( app.provider, 'groups', jQuery.extend( {}, response.data.groups ) );
							}

							if ( ! _.isEmpty( response.data.fields ) ) {
								app.Cache.set( app.provider, 'fields', jQuery.extend( {}, response.data.fields ) );
							}

							__private.$holder.trigger( 'connectionsDataLoaded', [ response.data ] );
						} );
				},
			},
		},

		/**
		 * All methods that modify UI of a page.
		 *
		 * @since 1.0.0
		 */
		ui: {

			/**
			 * Account methods.
			 *
			 * @since 1.0.0
			 */
			account: {

				/**
				 * Callback-function on change event.
				 *
				 * @since 1.0.0
				 */
				changeCallback: function() {

					var $this       = $( this ),
						$connection = $this.closest( '.wpforms-builder-provider-connection' ),
						$actionName = $( '.js-wpforms-builder-mailerlite-provider-connection-action', $connection );

					// Clear all connection data if account was changed.
					$( '.wpforms-builder-mailerlite-provider-actions-data', $connection ).empty();
					$actionName.prop( 'selectedIndex', 0 );

					// If account is empty.
					if ( _.isEmpty( $this.val() ) ) {

						// Block `Action` select box.
						$actionName.prop( 'disabled', true );
					} else {

						// Unblock `Action` select box.
						$actionName.prop( 'disabled', false );
						$this.removeClass( 'wpforms-error' );
					}
				},
			},

			/**
			 * Action methods.
			 *
			 * @since 1.0.0
			 */
			action: {

				/**
				 * Callback-function on change event.
				 *
				 * @since 1.0.0
				 */
				changeCallback: function() {

					var $this       = $( this ),
						$connection = $this.closest( '.wpforms-builder-provider-connection' ),
						$account    = $( '.js-wpforms-builder-mailerlite-provider-connection-account', $connection ),
						$action     = $( '.js-wpforms-builder-mailerlite-provider-connection-action', $connection );

					$( '.wpforms-builder-mailerlite-provider-actions-data', $connection ).empty();
					$this.removeClass( 'wpforms-error' );

					app.action.create.render( {
						'action': 'action',
						'target': $this,
						'account_id': $account.val(),
						'action_name': $action.val(),
						'connection_id': $connection.data( 'connection_id' ),
					} );
				},
			},
		},

		/**
		 * Action property.
		 *
		 * @since 1.0.0
		 */
		action: {

			/**
			 * Create action.
			 *
			 * @since 1.0.0
			 */
			create: {

				/**
				 * Render HTML.
				 *
				 * @since 1.0.0
				 *
				 * @param {object} args Arguments.
				 */
				render: function( args ) {

					var fields         = wpf.getFields(),
						optionalFields = app.tmpl.callbacks.optionalFieldsHTML( args ),
						requiredFields = app.tmpl.callbacks.requiredFieldsHTML( args, fields ),
						$connection    = app.connection.getById( args.connection_id );

					$connection
						.find( '.wpforms-builder-mailerlite-provider-actions-data' )
						.html( requiredFields );

					// Display compiled template with custom data.
					if ( 'subscribe' === args.action_name ) {
						$connection
							.find( '.wpforms-builder-mailerlite-provider-actions-data' )
							.append( optionalFields );
					}

					__private.$holder.trigger( 'connectionRendered', [ app.provider,
						args.connection_id ] );

					// Load ChoicesJS fields.
					app.loadChoicesJS( $connection );
				},
			},
		},

		/**
		 * Load Choices.js library.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} $connection jQuery connection selector.
		 */
		loadChoicesJS: function( $connection ) {

			// Load if function exists.
			if ( typeof window.Choices !== 'function' ) {
				return;
			}

			$connection.find( '.choicesjs-select' ).each( function( _index, element ) {

				var $this = $( element );

				// Return if already initialized.
				if ( 'undefined' !== typeof $this.data( 'choicesjs' ) ) {
					return;
				}

				$this.data( 'choicesjs', new Choices( $this[0], {
					shouldSort: false,
					removeItemButton: true,
					callbackOnInit: function() {

						wpf.initMultipleSelectWithSearch( this );
					},
				} ) );
			} );
		},

		/**
		 * All methods for JavaScript templates.
		 *
		 * @since 1.0.0
		 */
		tmpl: {

			/**
			 * Wrap functions for compile JavaScript templates and receive their HTML.
			 *
			 * @since 1.0.0
			 */
			callbacks: {

				/**
				 * Compile and retrieve an HTML for common elements.
				 *
				 * @since 1.0.0
				 *
				 * @returns {string} Compiled HTML.
				 */
				commonsHTML: function() {

					var tmplError = app.Templates.get( 'wpforms-' + app.provider + '-builder-content-connection-error' ),
						tmplLock  = app.Templates.get( 'wpforms-' + app.provider + '-builder-content-connection-lock' );

					return tmplError() + tmplLock( { provider: app.provider } );
				},

				/**
				 * Compile and retrieve an HTML for Custom Fields.
				 *
				 * @since 1.0.0
				 *
				 * @param {object} args Arguments
				 *
				 * @returns {string} Compiled HTML.
				 */
				optionalFieldsHTML: function( args ) {

					// Delete action don't require Custom Fields.
					if ( args.target.val() === 'delete' ) {
						return '';
					}

					var tmplFields  = app.Templates.get( 'wpforms-providers-builder-content-connection-fields' ),
						tmplLabel   = app.Templates.get( 'wpforms-mailerlite-builder-content-connection-custom-fields-label' ),
						connection  = app.Cache.getById( app.provider, 'connections', args.connection_id ),
						fields      = app.Cache.getById( app.provider, 'fields', args.account_id );

					return tmplLabel() + tmplFields( {
						'connection': connection,
						'fields': wpf.getFields(),
						'provider': {
							'placeholder': wpformsMailerLiteBuilderVars.i18n.provider_placeholder,
							'slug': app.provider,
							'fields': fields,
						},
					} );
				},

				/**
				 * Compile and retrieve an HTML for "Custom Fields Table".
				 *
				 * @since 1.0.0
				 *
				 * @param {object} args Arguments
				 * @param {object} fields Fields
				 *
				 * @returns {string} Compiled HTML.
				 */
				requiredFieldsHTML: function( args, fields ) {

					var actionsFields = app.Cache.get( app.provider, 'actions_fields' ),
						groups       = app.Cache.getById( app.provider, 'groups', args.account_id ),
						fieldHTML    = '';

					if ( ! _.has( actionsFields[ args.target.val() ], 'required' ) ) {
						return '';
					}

					$.each( actionsFields[ args.target.val() ].required, function( key, field ) {

						if ( key === 'groups' ) {
							field.options = groups;
						}

						var tmplField = app.Templates.get( 'wpforms-' + app.provider + '-builder-content-connection-required-' + field.type.replace( '_', '-' ) + '-field' );

						fieldHTML += tmplField( {
							'connection': app.Cache.getById( app.provider, 'connections', args.connection_id ),
							'name': key,
							'field': field,
							'provider': {
								'slug': app.provider,
								'fields': actionsFields[ args.target.val() ].required,
							},
							'formFields': fields,
						} );
					} );

					return fieldHTML;
				},
			},
		},

		/**
		 * Modal.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} data Modal data (type and message).
		 */
		modal: function( data ) {

			// Checking required data.
			if ( ! _.has( data, 'type' ) || ! _.has( data, 'message' ) ) {
				return;
			}

			var title = wpforms_builder.heads_up,
				icon = 'fa fa-exclamation-circle',
				color = 'orange';

			// Change modal attributes for success type.
			if ( data.type === 'success' ) {
				title = wpforms_builder.saved;
				icon = 'fa fa-check-circle';
				color = 'green';
			}

			$.alert( {
				title: title,
				content: data.message,
				icon: icon,
				type: color,
				buttons: {
					confirm: {
						text: wpforms_builder.ok,
						btnClass: 'btn-confirm',
						keys: [ 'enter' ],
					},
				},
			} );
		},
	};

	// Provide access to public functions/properties.
	return app;

}( document, window, jQuery ) );

// Initialize.
WPForms.Admin.Builder.Providers.MailerLite.init();
