<?php

namespace WPFormsMailerlite\Api;

use WPFormsMailerlite\Api\Http\Request;
use WPFormsMailerlite\Api\Http\Response;

/**
 * ConnectAPI class.
 *
 * @since 1.0.0
 */
class ConnectAPI implements PlatformInterface {

	/**
	 * URL for the ConnectAPI.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const API_URL = 'https://connect.mailerlite.com/api';

	/**
	 * Request.
	 *
	 * @since 1.0.0
	 *
	 * @var Request
	 */
	private $request;

	/**
	 * List of allowed keys for custom fields.
	 *
	 * @since 1.0.0
	 *
	 * @var string[]
	 */
	private $allowed_keys = [ 'id', 'name', 'key', 'type' ];

	/**
	 * API key.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private $api_key;

	/**
	 * ConnectAPI constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param string $api_key API key.
	 */
	public function __construct( $api_key ) {

		$this->api_key = $api_key;
		$this->request = new Request( $this->get_default_parameters() );
	}

	/**
	 * Retrieve an account information.
	 *
	 * @since 1.0.0
	 *
	 * @return Response
	 */
	public function get_account() {

		return $this->request->get( self::API_URL . '/account/' );
	}

	/**
	 * Get contact by email.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email Email.
	 *
	 * @return Response
	 */
	public function get_subscriber( $email ) {

		return $this->request->get( self::API_URL . sprintf( '/subscribers/%s', $email ) );
	}

	/**
	 * Create contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email  Email.
	 * @param array  $groups Group.
	 * @param array  $fields Fields.
	 * @param string $type   Type.
	 *
	 * @return Response
	 */
	public function create_contact( $email, $groups, $fields, $type ) {

		$data = [
			'email'           => $email,
			'fields'          => (object) $fields,
			'status'          => $type,
			'groups'          => $groups,
			'unsubscribed_at' => $type === 'unsubscribed' ? gmdate( 'Y-m-d H:i:s' ) : null,
		];

		return $this->request->post(
			self::API_URL . '/subscribers/',
			$data
		);
	}

	/**
	 * Updating contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email      Existing contact email.
	 * @param array  $properties Properties.
	 *
	 * @return Response
	 */
	public function update_contact( $email, $properties ) {

		return $this->request->post(
			self::API_URL . '/subscribers/',
			[
				'email'  => $email,
				'fields' => (object) $properties,
			]
		);
	}

	/**
	 * Delete contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id Contact ID.
	 *
	 * @return Response
	 */
	public function delete_contact( $id ) {

		return $this->request->delete( sprintf( self::API_URL . '/subscribers/%s', $id ) );
	}

	/**
	 * Delete contact from a group.
	 *
	 * @since 1.0.0
	 *
	 * @param int $id       Contact ID.
	 * @param int $group_id Group ID.
	 *
	 * @return Response
	 */
	public function delete_contact_from_group( $id, $group_id ) {

		return $this->request->post(
			sprintf( self::API_URL . '/subscribers/%s/groups/unassign', $id ),
			[
				'id' => $group_id,
			]
		);
	}

	/**
	 * Get a list of groups.
	 *
	 * @since 1.0.0
	 *
	 * @param int $limit  Limit of records.
	 * @param int $offset Offset for lists select.
	 *
	 * @return Response
	 */
	public function get_groups( $limit, $offset = 0 ) {

		return $this->request->get(
			self::API_URL . '/groups/',
			[
				'limit' => absint( $limit ),
				'page'  => absint( $offset ),
			]
		);
	}

	/**
	 * Get all groups.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_all_groups() {

		$groups   = [];
		$page     = 1;
		$per_page = 100; // It's possible to get a maximum of 100 groups per request.

		do {
			$data = $this->get_groups( $per_page, $page )->get_data();

			if ( ! is_array( $data ) ) {
				break;
			}

			$count    = count( $data );
			$groups[] = $data;

			$page ++;
		} while ( $count !== 0 );

		return array_merge( [], ...$groups );
	}

	/**
	 * Get list of custom fields.
	 *
	 * @since 1.0.0
	 */
	public function get_fields() {

		$response = $this->request->get( self::API_URL . '/fields/' );
		$body     = $response->get_data();
		$fields   = [];

		if ( empty( $body ) ) {
			return [];
		}

		foreach ( $body as $field ) {

			$arr          = array_intersect_key( $field, array_flip( $this->allowed_keys ) );
			$arr['title'] = $arr['name'];

			unset( $arr['name'] );

			$fields[] = $arr;
		}

		return $fields;
	}


	/**
	 * Retrieve default parameters for request.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_default_parameters() {

		return [
			'method'  => 'GET',
			'timeout' => 5,
			'headers' => [
				'Authorization' => 'Bearer ' . $this->api_key,
				'Accept'        => 'application/json',
				'Connection'    => 'keep-alive',
				'Content-Type'  => 'application/json',
				'user-agent'    => 'WPForms',
			],
		];
	}
}
