<?php

namespace WPFormsMailerlite\Api\Http;

/**
 * Wrapper class for HTTP requests.
 *
 * @since 1.0.0
 */
class Request {

	/**
	 * Request parameters.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $params;

	/**
	 * Request constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param array $params Additional parameters.
	 */
	public function __construct( $params ) {

		$this->params = $params;
	}

	/**
	 * Send a GET request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function get( $url, $args = [] ) {

		$url = add_query_arg( $args, $url );

		return $this->request( 'GET', $url, [] );
	}

	/**
	 * Send a POST request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function post( $url, $args = [] ) {

		$args = ! empty( $args ) ? [ 'body' => $args ] : [];

		return $this->request( 'POST', $url, $args );
	}

	/**
	 * Send a PUT request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $uri  URI to request.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function put( $uri, $args = [] ) {

		$args = ! empty( $args ) ? [ 'body' => $args ] : [];

		return $this->request( 'PUT', $uri, $args );
	}

	/**
	 * Send a DELETE request.
	 *
	 * @since 1.0.0
	 *
	 * @param string $url  Request URL.
	 * @param array  $args Arguments for the request.
	 *
	 * @return Response
	 */
	public function delete( $url, $args = [] ) {

		return $this->request( 'DELETE', $url, $args );
	}

	/**
	 * Send a request based on method (main interface).
	 *
	 * @since 1.0.0
	 *
	 * @param string $method Request method.
	 * @param string $url    Request URL.
	 * @param array  $args   Options for the request.
	 *
	 * @return Response
	 */
	public function request( $method, $url, $args ) {

		// Merge options.
		$options = array_merge_recursive( $this->params, $args );

		// Set a request method.
		$options['method'] = $method;

		// Prepare a request body, as API expect it in a JSON format.
		if (
			! empty( $options['headers']['Content-Type'] ) &&
			$options['headers']['Content-Type'] !== 'application/x-www-form-urlencoded' &&
			! empty( $options['body'] )
		) {
			$options['body'] = wp_json_encode( $options['body'] );
		}

		/**
		 * Filter a request options array before it's sent.
		 *
		 * @since 1.0.0
		 *
		 * @param array   $options  Request options.
		 * @param string  $method   Request method.
		 * @param string  $url      Request URL.
		 * @param Request $instance Instance of Request class.
		 */
		$options = (array) apply_filters( 'wpforms_mailerlite_api_http_request_options', $options, $method, $url, $this );

		// Retrieve the raw response from a safe HTTP request.
		return new Response( wp_safe_remote_request( $url, $options ) );
	}
}
