<?php

namespace WPFormsMailerlite\Api;

use WPFormsMailerlite\Api\Http\Request;
use WPFormsMailerlite\Api\Http\Response;

/**
 * MailerLite API class.
 *
 * @since 1.0.0
 */
class MailerLiteAPI implements PlatformInterface {

	/**
	 * URL for the MailerLite API.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const API_URL = 'https://api.mailerlite.com/api/v2';

	/**
	 * Request.
	 *
	 * @since 1.0.0
	 *
	 * @var Request
	 */
	private $request;

	/**
	 * List of allowed keys for custom fields.
	 *
	 * @since 1.0.0
	 *
	 * @var string[]
	 */
	private $allowed_keys = [ 'id', 'title', 'key', 'type' ];

	/**
	 * API key.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private $api_key;

	/**
	 * MailerLiteAPI constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param string $api_key API key.
	 */
	public function __construct( $api_key ) {

		$this->api_key = $api_key;
		$this->request = new Request( $this->get_default_parameters() );
	}

	/**
	 * Retrieve an account information.
	 *
	 * @since 1.0.0
	 *
	 * @return Response
	 */
	public function get_account() {

		return $this->request->get( self::API_URL . '/me/' );
	}

	/**
	 * Get contact by email.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email Email.
	 *
	 * @return Response
	 */
	public function get_subscriber( $email ) {

		return $this->request->get( self::API_URL . sprintf( '/subscribers/%s', $email ) );
	}

	/**
	 * Create contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email  Email.
	 * @param array  $groups Groups.
	 * @param array  $fields Fields.
	 * @param string $type   Type.
	 *
	 * @return Response
	 */
	public function create_contact( $email, $groups, $fields, $type ) {

		$data = [
			'email'            => $email,
			'fields'           => (object) $fields,
			'type'             => $type,
			'date_unsubscribe' => $type === 'unsubscribed' ? gmdate( 'Y-m-d H:i:s' ) : null,
		];

		$response = $this->request->post(
			self::API_URL . '/subscribers/',
			$data
		);

		foreach ( (array) $groups as $item ) {
			$this->set_group( $email, $item );
		}

		return $response;
	}

	/**
	 * Updating contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email      Existing contact email.
	 * @param array  $properties Properties.
	 *
	 * @return Response
	 */
	public function update_contact( $email, $properties ) {

		return $this->request->put( sprintf( self::API_URL . '/subscribers/%s', rawurlencode( $email ) ), $properties );
	}

	/**
	 * Delete contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id Contact ID.
	 *
	 * @return Response
	 */
	public function delete_contact( $id ) {

		return $this->request->delete( sprintf( self::API_URL . '/subscribers/%s', $id ) );
	}

	/**
	 * Delete contact from a group.
	 *
	 * @since 1.0.0
	 *
	 * @param int $id       Contact ID.
	 * @param int $group_id Group ID.
	 *
	 * @return Response
	 */
	public function delete_contact_from_group( $id, $group_id ) {

		return $this->request->delete( sprintf( self::API_URL . '/groups/%s/subscribers/%s', $group_id, $id ) );
	}

	/**
	 * Get a list of groups.
	 *
	 * @since 1.0.0
	 *
	 * @param int $limit  Limit of records.
	 * @param int $offset Offset for lists select.
	 *
	 * @return Response
	 */
	public function get_groups( $limit, $offset = 0 ) {

		return $this->request->get(
			self::API_URL . '/groups/',
			[
				'limit'  => absint( $limit ),
				'offset' => absint( $offset ),
			]
		);
	}

	/**
	 * Get all groups.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_all_groups() {

		$groups   = [];
		$offset   = 0;
		$per_page = 100; // It's possible to get a maximum of 100 groups per request.

		do {
			$data = $this->get_groups( $per_page, $offset )->get_data();

			if ( ! is_array( $data ) ) {
				break;
			}

			$count    = count( $data );
			$groups[] = $data;

			$offset += $per_page;
		} while ( $count !== 0 );

		return array_merge( [], ...$groups );
	}


	/**
	 * Add single subscriber to specified group.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email    Email.
	 * @param int    $group_id Group ID.
	 *
	 * @return Response
	 */
	public function set_group( $email, $group_id ) {

		return $this->request->post(
			sprintf( self::API_URL . '/groups/%s/subscribers', $group_id ),
			[
				'email' => $email,
			]
		);
	}

	/**
	 * Get list of custom fields.
	 *
	 * @since 1.0.0
	 */
	public function get_fields() {

		$response = $this->request->get( self::API_URL . '/fields/' );
		$body     = $response->get_body();
		$fields   = [];

		if ( empty( $body ) ) {
			return [];
		}

		foreach ( $body as $field ) {

			$fields[] = array_intersect_key( $field, array_flip( $this->allowed_keys ) );
		}

		return $fields;
	}

	/**
	 * Retrieve default parameters for request.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_default_parameters() {

		return [
			'method'  => 'GET',
			'timeout' => 5,
			'headers' => [
				'X-MailerLite-ApiKey' => $this->api_key,
				'Accept'              => 'application/json',
				'Connection'          => 'keep-alive',
				'Content-Type'        => 'application/json',
			],
		];
	}
}
