<?php

namespace WPFormsMailerlite;

use stdClass;
use WPForms_Updater;
use WPForms\Providers\Providers;
use WPFormsMailerlite\Api\Client;
use WPFormsMailerlite\Provider\Core;
use WPFormsMailerlite\Provider\Account;
use WPFormsMailerlite\Provider\Template;
use WPFormsMailerlite\Tasks\ProcessActionTask;
use WPFormsMailerlite\Provider\Settings\Helper;

/**
 * WPForms MailerLite main class.
 *
 * @since 1.0.0
 */
class Plugin {

	/**
	 * Provider slug.
	 *
	 * @since 1.0.0
	 */
	const SLUG = 'mailerlite';

	/**
	 * Manage API accounts.
	 *
	 * @since 1.0.0
	 *
	 * @var Account
	 */
	private $account;

	/**
	 * Get some addon templates.
	 *
	 * @since 1.0.0
	 *
	 * @var Template
	 */
	private $template;

	/**
	 * Client for MailerLite API.
	 *
	 * @since 1.0.0
	 *
	 * @var Client
	 */
	private $client;

	/**
	 * Helper class.
	 *
	 * @since 1.0.0
	 *
	 * @var Helper
	 */
	private $helper;

	/**
	 * Plugin constructor.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {}

	/**
	 * Initialize plugin.
	 *
	 * @since 1.0.0
	 * @since 1.1.0 Returns Plugin instance.
	 *
	 * @return Plugin
	 */
	public function init() {

		$this->hooks();
		$this->load_dependencies();

		return $this;
	}

	/**
	 * Plugin hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_updater', [ $this, 'updater' ] );
	}

	/**
	 * Get property.
	 *
	 * @since 1.0.0
	 *
	 * @param string $property_name Property name.
	 *
	 * @return mixed
	 */
	public function get( $property_name ) {

		return property_exists( $this, $property_name ) ? $this->{$property_name} : new stdClass();
	}

	/**
	 * Get a single instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return Plugin
	 */
	public static function get_instance() {

		static $instance;

		if ( ! $instance ) {
			$instance = new self();

			$instance->init();
		}

		return $instance;
	}

	/**
	 * All the actual plugin loading is done here.
	 *
	 * @since 1.0.0
	 */
	private function load_dependencies() {

		$this->account  = new Account();
		$this->template = new Template();
		$this->client   = new Client();
		$this->helper   = new Helper();

		$this->template->hooks();

		( new ProcessActionTask() )->hooks();

		Providers::get_instance()->register(
			Core::get_instance()
		);
	}

	/**
	 * Load the plugin updater.
	 *
	 * @since 1.0.0
	 *
	 * @param string $key License key.
	 */
	public function updater( $key ) {

		new WPForms_Updater(
			[
				'plugin_name' => 'WPForms MailerLite',
				'plugin_slug' => 'wpforms-mailerlite',
				'plugin_path' => plugin_basename( WPFORMS_MAILERLITE_FILE ),
				'plugin_url'  => trailingslashit( WPFORMS_MAILERLITE_URL ),
				'remote_url'  => WPFORMS_UPDATER_API,
				'version'     => WPFORMS_MAILERLITE_VERSION,
				'key'         => $key,
			]
		);
	}
}
