<?php

namespace WPFormsMailerlite\Provider;

use WPFormsMailerlite\Plugin;
// phpcs:ignore WPForms.PHP.UseStatement.UnusedUseStatement
use WPFormsMailerlite\Api\Connection;
use WPFormsMailerlite\Api\Http\Response;
use WPFormsMailerlite\Api\Exceptions\InvalidKeyException;
use WPFormsMailerlite\Api\Exceptions\AccountAlreadyExistsException;

/**
 * Class Account.
 *
 * @since 1.0.0
 */
class Account {

	/**
	 * Check if account with this API key already exists.
	 *
	 * @since 1.0.0
	 *
	 * @param string $api_key Api key.
	 *
	 * @throws AccountAlreadyExistsException The account already exists.
	 */
	private function is_already_exist( $api_key ) {

		$options = wpforms_get_providers_options( Plugin::SLUG );
		$keys    = array_column( $options, 'api_key' );

		if ( in_array( $api_key, $keys, true ) ) {
			throw new AccountAlreadyExistsException();
		}
	}

	/**
	 * Validate an API response.
	 *
	 * @since 1.0.0
	 *
	 * @param Response $account Account API response.
	 *
	 * @return void
	 *
	 * @throws InvalidKeyException Invalid key error message.
	 */
	private function validate_account( $account ) {

		$status = $account->get_response_code();

		// Request error.
		if ( $status !== 200 ) {
			throw new InvalidKeyException();
		}
	}

	/**
	 * Get list of available accounts.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_accounts() {

		$accounts = wpforms_get_providers_options( Plugin::SLUG );

		if ( empty( $accounts ) ) {
			return [];
		}

		foreach ( $accounts as $account_id => $account ) {
			if ( empty( $account['api_key'] ) ) {
				$this->remove_account( $account_id );

				continue;
			}

			$accounts[ $account_id ] = $account['label'];
		}

		return $accounts;
	}

	/**
	 * Get connection.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 *
	 * @return Connection|null
	 */
	public function get_connection( $account_id ) {

		$accounts = wpforms_get_providers_options( Plugin::SLUG );

		if ( empty( $accounts[ $account_id ]['api_key'] ) ) {
			return null;
		}

		return wpforms_mailerlite()
			->get( 'client' )
			->new_connection( $accounts[ $account_id ]['api_key'] );
	}

	/**
	 * Remove an account.
	 *
	 * @since 1.0.0
	 *
	 * @param string $account_id Account ID.
	 */
	public function remove_account( $account_id ) {

		$providers = wpforms_get_providers_options();

		if ( empty( $providers[ Plugin::SLUG ][ $account_id ] ) ) {
			return;
		}

		unset( $providers[ Plugin::SLUG ][ $account_id ] );

		update_option( 'wpforms_providers', $providers );
	}

	/**
	 * Get an account by an API key.
	 *
	 * @since 1.0.0
	 *
	 * @param string $api_key Api key.
	 *
	 * @return Response
	 *
	 * @throws InvalidKeyException Invalid key error message.
	 */
	public function get_account_by_api_key( $api_key ) {

		$account = wpforms_mailerlite()
			->get( 'client' )
			->new_connection( sanitize_text_field( $api_key ) )
			->get_account();

		$this->validate_account( $account );

		return $account;
	}

	/**
	 * Save a new account.
	 *
	 * @since 1.0.0
	 *
	 * @param string $api_key      API key.
	 * @param string $account_name Account name.
	 *
	 * @return array
	 *
	 * @throws InvalidKeyException           Invalid key error message.
	 * @throws AccountAlreadyExistsException Account already exists.
	 */
	public function add( $api_key, $account_name ) {

		$account = $this->get_account_by_api_key( $api_key );

		$this->is_already_exist( $api_key );

		$data = $account->get_data();

		if ( empty( $account_name ) ) {
			$name         = ! empty( $data['account'] ) ? $data['account']['name'] : $data['name'];
			$account_name = sanitize_text_field( $name );
		}

		$key = uniqid( '', true );

		$options = [
			'api_key' => $api_key,
			'label'   => $account_name,
			'date'    => time(),
		];

		// Save this connection for the provider.
		wpforms_update_providers_options( Plugin::SLUG, $options, $key );

		$options['key'] = $key;

		return $options;
	}
}
