<?php

namespace WPFormsMailerlite\Provider\Actions;

use WPFormsMailerlite\Api\Http\Response;
use WPFormsMailerlite\Api\Exceptions\ResponseException;
use WPFormsMailerlite\Api\Exceptions\RequiredArgumentMissingException;

/**
 * Class SubscribeAction.
 *
 * @since 1.0.0
 */
class SubscribeAction extends Action {

	/**
	 * Run action.
	 *
	 * @since 1.0.0
	 *
	 * @throws ResponseException Invalid response.
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	public function run() {

		$this->subscribe_contact();
	}

	/**
	 * Create contact.
	 *
	 * @since 1.0.0
	 *
	 * @throws ResponseException Invalid response.
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 * phpcs:ignore Squiz.Commenting.FunctionCommentThrowTag.WrongNumber
	 */
	private function subscribe_contact() {

		$email  = $this->field_mapper->get_field_value_by_id( $this->field_mapper->get_required_field_value( 'email', 'field_id' ) );
		$groups = $this->field_mapper->get_group_field_value( 'groups', 'group_id' );
		$type   = $this->field_mapper->get_required_field_value( 'type', 'type_id' );
		$fields = $this->map_custom_fields();

		$response = $this->create_contact( $email, $groups, $fields, $type );

		if ( $response->has_errors() ) {
			throw new ResponseException( $response->get_response_message() );
		}
	}

	/**
	 * Create a contact.
	 *
	 * @since 1.0.0
	 *
	 * @param string $email  Contact's email.
	 * @param array  $groups Group ID for contact.
	 * @param array  $fields Custom fields.
	 * @param string $type   Contact type.
	 *
	 * @return Response
	 */
	private function create_contact( $email, $groups, $fields, $type ) {

		return $this->connection->create_contact(
			$email,
			$groups,
			$fields,
			$type
		);
	}

	/**
	 * Get list of registered attributes in MailerLite.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_fields() {

		$cache_key = 'wpforms_mailerlite_fields_' . $this->connection_data['account_id'];
		$fields    = get_transient( $cache_key );

		if ( ! empty( $fields ) ) {
			return $fields;
		}

		$fields = $this->connection->get_fields();

		foreach ( $fields as $key => $field ) {

			$fields[ $field['key'] ] = $field;

			unset( $fields[ $key ] );
		}

		set_transient( $cache_key, $fields, 15 * MINUTE_IN_SECONDS );

		return $fields;
	}

	/**
	 * Map MailerLite form fields.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function map_custom_fields() {

		if ( empty( $this->connection_data['fields_meta'] ) ) {
			return [];
		}

		$custom_fields = [];
		$fields        = $this->get_fields();

		foreach ( $this->connection_data['fields_meta'] as $field ) {

			if ( ! isset( $field['name'] ) ) {
				continue;
			}

			$field_id = $this->field_mapper->get_custom_field_value( $field['name'] );

			if ( wpforms_is_empty_string( $field_id ) ) {
				continue;
			}

			$value = $this->field_mapper->get_formatted_value( $field_id, $fields[ $field['name'] ] );

			if ( $value === null || wpforms_is_empty_string( $value ) ) {
				continue;
			}

			$custom_fields[ $field['name'] ] = $value;
		}

		return $custom_fields;
	}
}
