<?php

namespace WPFormsMailerlite\Provider;

use WPFormsMailerlite\Api\Exceptions\RequiredArgumentMissingException;

/**
 * Field Mapper class.
 *
 * @since 1.0.0
 */
class FieldMapper {

	/**
	 * List of fields.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $fields;

	/**
	 * Form data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $form_data;

	/**
	 * Connection data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $connection_data;

	/**
	 * FieldMapper constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields          Array of form fields.
	 * @param array $form_data       Form data and settings.
	 * @param array $connection_data Connection data.
	 */
	public function __construct( $fields, $form_data, $connection_data ) {

		$this->fields          = $fields;
		$this->form_data       = $form_data;
		$this->connection_data = $connection_data;
	}

	/**
	 * Get required field value.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Field name.
	 * @param string $key  Field key.
	 *
	 * @return string
	 *
	 * @throws RequiredArgumentMissingException Required argument is missing.
	 */
	public function get_required_field_value( $name, $key ) {

		if ( empty( $this->connection_data['fields_required'][ $name ][ $key ] ) ) {
			throw new RequiredArgumentMissingException( $name );
		}

		return $this->connection_data['fields_required'][ $name ][ $key ];
	}

	/**
	 * Get not required field value.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Field name.
	 * @param string $key  Field key.
	 *
	 * @return string
	 */
	public function get_not_required_field_value( $name, $key ) {

		if ( ! isset( $this->connection_data['fields_required'][ $name ][ $key ] ) ) {
			return '';
		}

		return (string) $this->connection_data['fields_required'][ $name ][ $key ];
	}

	/**
	 * Get groups value.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Field name.
	 * @param string $key  Field key.
	 *
	 * @return array
	 */
	public function get_group_field_value( $name, $key ) {

		if ( ! isset( $this->connection_data[ $name ][ $key ] ) ) {
			return [];
		}

		return (array) $this->connection_data[ $name ][ $key ];
	}

	/**
	 * Get field value by field ID.
	 *
	 * @since 1.0.0
	 *
	 * @param int $field_id Field ID.
	 *
	 * @return string
	 */
	public function get_field_value_by_id( $field_id ) {

		if ( empty( $this->form_data['fields'][ $field_id ] ) ) {
			return '';
		}

		return (string) $this->fields[ $field_id ]['value'];
	}

	/**
	 * Get custom field value.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Field name.
	 *
	 * @return string
	 */
	public function get_custom_field_value( $name ) {

		$custom_fields = $this->get_custom_fields();

		return isset( $custom_fields[ $name ] ) ? $custom_fields[ $name ] : '';
	}

	/**
	 * Convert custom fields in key => value format.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_custom_fields() {

		if ( empty( $this->connection_data['fields_meta'] ) ) {
			return [];
		}

		return wp_list_pluck( $this->connection_data['fields_meta'], 'field_id', 'name' );
	}

	/**
	 * Get custom field value formatted to MailerLite properties.
	 *
	 * @since 1.0.0
	 *
	 * @uses FieldMapper::prepare_date_format
	 * @uses FieldMapper::prepare_number_format
	 *
	 * @param int   $field_id Field ID.
	 * @param array $property Property data.
	 *
	 * @return string|null
	 */
	public function get_formatted_value( $field_id, $property ) {

		$type = ! empty( $property['type'] ) ? strtolower( $property['type'] ) : '';

		$mapping_method = 'prepare_' . $type . '_format';

		if ( method_exists( $this, $mapping_method ) ) {

			$value = wpforms_decode_string( $this->$mapping_method( $this->fields[ $field_id ]['value'] ) );

			return str_replace( [ "\r\n", "\r", "\n" ], ', ', $value );
		}

		return wpforms_decode_string( $this->fields[ $field_id ]['value'] );
	}

	/**
	 * Date format.
	 *
	 * @since 1.0.0
	 *
	 * @param string $value Field value.
	 *
	 * @return string|null
	 */
	private function prepare_date_format( $value ) {

		$time = strtotime( $value );

		return $time ? gmdate( 'Y-m-d', $time ) : null;
	}

	/**
	 * Number format.
	 *
	 * @since 1.0.0
	 *
	 * @param string $value Field value.
	 *
	 * @return float|null
	 */
	private function prepare_number_format( $value ) {

		return is_numeric( $value ) ? (float) $value : null;
	}
}
