<?php

namespace WPFormsMailerlite\Provider\Settings;

use Exception;
use WPFormsMailerlite\Plugin;
use WPForms\Providers\Provider\Settings\FormBuilder as FormBuilderAbstract;

/**
 * Class FormBuilder handles functionality in the Form Builder.
 *
 * @since 1.0.0
 */
class FormBuilder extends FormBuilderAbstract {

	/**
	 * Register all hooks (actions and filters).
	 *
	 * @since 1.0.0
	 */
	protected function init_hooks() {

		parent::init_hooks();

		$this->hooks();
	}

	/**
	 * Register hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		static $ajax_events = [
			'ajax_account_save',
			'ajax_accounts_get',
			'ajax_connections_get',
			'ajax_account_template_get',
		];

		array_walk(
			$ajax_events,
			static function ( $ajax_event, $key, $instance ) {

				add_filter(
					"wpforms_providers_settings_builder_{$ajax_event}_{$instance->core->slug}",
					[ $instance, $ajax_event ]
				);
			},
			$this
		);

		add_filter( 'wpforms_save_form_args', [ $this, 'save_form' ], 11, 3 );
		add_action( 'wpforms_builder_enqueues', [ $this, 'enqueue_inline_css' ], 10 );
	}

	/**
	 * Pre-process provider data before saving it in form_data when editing form.
	 *
	 * @since 1.0.0
	 *
	 * @param array $form Form array, usable with wp_update_post.
	 * @param array $data Data retrieved from $_POST and processed.
	 * @param array $args Update form arguments.
	 *
	 * @return array
	 */
	public function save_form( $form, $data, $args ) {

		$form_data = json_decode( stripslashes( $form['post_content'] ), true );

		if ( ! empty( $form_data['providers'][ Plugin::SLUG ] ) ) {
			$modified_form_data = $this->modify_form_data( $form_data );

			if ( ! empty( $modified_form_data ) ) {
				$form['post_content'] = wpforms_encode( $modified_form_data );

				return $form;
			}
		}

		/*
		 * This part works when modification is locked or current filter was called on NOT Providers panel.
		 * Then we need to restore provider connections from the previous form content.
		 */

		// Get a "previous" form content (current content is still not saved).
		$prev_form = ! empty( $data['id'] ) ? wpforms()->get( 'form' )->get( $data['id'], [ 'content_only' => true ] ) : [];

		if ( ! empty( $prev_form['providers'][ Plugin::SLUG ] ) ) {
			$provider = $prev_form['providers'][ Plugin::SLUG ];

			if ( ! isset( $form_data['providers'] ) ) {
				$form_data = array_merge( $form_data, [ 'providers' => [] ] );
			}

			$form_data['providers'] = array_merge( (array) $form_data['providers'], [ Plugin::SLUG => $provider ] );
			$form['post_content']   = wpforms_encode( $form_data );
		}

		return $form;
	}

	/**
	 * Prepare modifications for the form content, if it's not locked.
	 *
	 * @since 1.0.0
	 *
	 * @param array $form_data Form content.
	 *
	 * @return array
	 */
	protected function modify_form_data( $form_data ) {

		$lock = '__lock__';

		/**
		 * Connection is locked.
		 * Why? User clicked the "Save" button when one of AJAX requests
		 * for retrieving data from API was in progress or failed.
		 */
		if (
			isset( $form_data['providers'][ Plugin::SLUG ][ $lock ] ) &&
			absint( $form_data['providers'][ Plugin::SLUG ][ $lock ] ) === 1
		) {
			return [];
		}

		// Modify content as we need, done by reference.
		foreach ( $form_data['providers'][ Plugin::SLUG ] as $connection_id => $connection ) {
			if ( $connection_id === $lock ) {
				unset( $form_data['providers'][ Plugin::SLUG ][ $lock ] );
				continue;
			}

			$form_data['providers'][ Plugin::SLUG ][ $connection_id ] = wpforms_mailerlite()->get( 'helper' )->sanitize_connection( $connection );
		}

		return $form_data;
	}

	/**
	 * Get the list of all saved connections.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function ajax_connections_get() {

		$connections = [
			'connections'    => ! empty( $this->get_connections_data() ) ? array_reverse( $this->get_connections_data(), true ) : [],
			'conditionals'   => [],
			'actions'        => $this->get_actions(),
			'actions_fields' => [
				'subscribe'   => $this->get_subscribe_fields(),
				'unsubscribe' => $this->get_unsubscribe_fields(),
				'delete'      => $this->get_delete_fields(),
			],
		];

		foreach ( $connections['connections'] as $connection ) {
			if ( empty( $connection['id'] ) ) {
				continue;
			}

			// This will either return an empty placeholder or complete set of rules, as a DOM.
			$connections['conditionals'][ $connection['id'] ] = wpforms_conditional_logic()
				->builder_block(
					[
						'form'       => $this->form_data,
						'type'       => 'panel',
						'parent'     => 'providers',
						'panel'      => Plugin::SLUG,
						'subsection' => $connection['id'],
						'reference'  => esc_html__( 'Marketing provider connection', 'wpforms-mailerlite' ),
					],
					false
				);
		}

		$accounts = $this->ajax_accounts_get();

		return array_merge( $connections, $accounts );
	}

	/**
	 * Get accounts data.
	 *
	 * @since 1.0.0
	 *
	 * @return array May return an empty sub-array.
	 */
	public function ajax_accounts_get() {

		$accounts = wpforms_mailerlite()->get( 'account' )->get_accounts();
		$groups   = [];
		$fields   = [];

		foreach ( $accounts as $account_id => $account_name ) {
			$groups[ $account_id ] = $this->get_groups( $account_id );
			$fields[ $account_id ] = $this->get_fields( $account_id );
		}

		return [
			'accounts' => $accounts,
			'groups'   => $groups,
			'fields'   => $fields,
		];
	}

	/**
	 * Content for Add New Account modal.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function ajax_account_template_get() {

		$content = wpforms_mailerlite()
			->get( 'template' )
			->get_settings_template(
				'new-account-form',
				[
					'core_name' => esc_html__( 'MailerLite', 'wpforms-mailerlite' ),
				]
			);

		return [
			'title'      => esc_html__( 'New MailerLite Account', 'wpforms-mailerlite' ),
			'content'    => $content,
			'connection' => '',
			'type'       => 'blue',
		];
	}

	/**
	 * Save the data for a new account and validate it.
	 *
	 * @since 1.0.0
	 */
	public function ajax_account_save() {

		// phpcs:ignore WordPress.Security.NonceVerification, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$data         = ! empty( $_POST['data'] ) ? wp_unslash( $_POST['data'] ) : [];
		$api_key      = ! empty( $data['api_key'] ) ? sanitize_text_field( $data['api_key'] ) : '';
		$account_name = ! empty( $data['account_name'] ) ? sanitize_text_field( $data['account_name'] ) : '';

		try {
			wpforms_mailerlite()
				->get( 'account' )
				->add( $api_key, $account_name );

			wp_send_json_success();
		} catch ( Exception $e ) {
			wp_send_json_error( [ 'error_msg' => esc_html( $e->getMessage() ) ] );
		}
	}

	/**
	 * Get fields for the subscribe action.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_subscribe_fields() {

		return [
			'required' => [
				'email'  => [
					'label'    => esc_html__( 'Email', 'wpforms-mailerlite' ),
					'type'     => 'email_select',
					'required' => true,
				],
				'type'   => [
					'label'    => esc_html__( 'Type', 'wpforms-mailerlite' ),
					'type'     => 'select',
					'required' => true,
					'options'  => [
						'active'       => esc_html__( 'Active', 'wpforms-mailerlite' ),
						'unsubscribed' => esc_html__( 'Unsubscribed', 'wpforms-mailerlite' ),
						'unconfirmed'  => esc_html__( 'Unconfirmed', 'wpforms-mailerlite' ),
					],
				],
				'groups' => [
					'label'    => esc_html__( 'Groups', 'wpforms-mailerlite' ),
					'type'     => 'multiselect',
					'required' => false,
				],
			],
		];
	}

	/**
	 * Get fields for the unsubscribe action.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_unsubscribe_fields() {

		return [
			'required' => [
				'email'  => [
					'label'    => esc_html__( 'Email', 'wpforms-mailerlite' ),
					'type'     => 'email_select',
					'required' => true,
				],
				'groups' => [
					'label'    => esc_html__( 'Groups', 'wpforms-mailerlite' ),
					'type'     => 'multiselect',
					'required' => true,
				],
			],
		];
	}

	/**
	 * Get fields for the delete action.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_delete_fields() {

		return [
			'required' => [
				'email' => [
					'label'    => esc_html__( 'Email', 'wpforms-mailerlite' ),
					'type'     => 'email_select',
					'required' => true,
				],
			],
		];
	}

	/**
	 * Retrieve saved provider connections data.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_connections_data() {

		return isset( $this->form_data['providers'][ Plugin::SLUG ] ) ? $this->form_data['providers'][ Plugin::SLUG ] : [];
	}

	/**
	 * Get all groups for account.
	 *
	 * @since 1.0.0
	 *
	 * @param int $account_id Account ID.
	 *
	 * @return array
	 */
	private function get_groups( $account_id ) {

		$cache_key = 'wpforms_mailerlite_groups_' . $account_id;
		$groups    = get_transient( $cache_key );

		if ( ! empty( $groups ) && is_array( $groups ) ) {
			return $groups;
		}

		$connection = wpforms_mailerlite()
			->get( 'account' )
			->get_connection( $account_id );

		$groups = $connection !== null ? $connection->get_all_groups() : [];

		if ( empty( $groups ) ) {
			return [];
		}

		$groups = wp_list_pluck( $groups, 'name', 'id' );

		set_transient( $cache_key, $groups, 15 * MINUTE_IN_SECONDS );

		return $groups;
	}

	/**
	 * Get fields for account.
	 *
	 * @since 1.0.0
	 *
	 * @param int $account_id Account ID.
	 *
	 * @return array
	 */
	private function get_fields( $account_id ) {

		$cache_key = 'wpforms_mailerlite_fields_' . $account_id;
		$fields    = get_transient( $cache_key );

		if ( $fields && is_array( $fields ) ) {
			return $fields;
		}

		$connection = wpforms_mailerlite()
			->get( 'account' )
			->get_connection( $account_id );

		if ( ! $connection ) {
			return [];
		}

		$fields = [];

		foreach ( $connection->get_fields() as $item ) {

			if ( $item['key'] === 'email' ) {
				continue;
			}

			$fields[ $item['key'] ] = $item['title'];
		}

		set_transient( $cache_key, $fields, 15 * MINUTE_IN_SECONDS );

		return $fields;
	}

	/**
	 * Get list of actions.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_actions() {

		return [
			'subscribe'   => esc_html__( 'Subscribe', 'wpforms-mailerlite' ),
			'unsubscribe' => esc_html__( 'Unsubscribe', 'wpforms-mailerlite' ),
			'delete'      => esc_html__( 'Delete', 'wpforms-mailerlite' ),
		];
	}

	/**
	 * Use this method to register own templates for form builder.
	 * Make sure, that you have `tmpl-` in template name in `<script id="tmpl-*">`.
	 *
	 * @since 1.0.0
	 */
	public function builder_custom_templates() {

		$templates = [
			'required-email-select-field',
			'required-select-field',
			'required-multiselect-field',
			'required-text-field',
			'custom-fields-label',
			'error',
			'lock',
		];

		?>
		<!-- Single MailerLite connection. -->
		<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection">
			<?php
			// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo wpforms_mailerlite()->get( 'template' )->get_builder_template( 'connection' );
			?>
		</script>

		<?php
		foreach ( $templates as $template ) :
			$template_name = strtoupper( str_replace( '-', ' ', $template ) );
			?>
			<!-- Single MailerLite connection block: <?php echo esc_attr( $template_name ); ?>. -->
			<script type="text/html" id="tmpl-wpforms-<?php echo esc_attr( Plugin::SLUG ); ?>-builder-content-connection-<?php echo esc_attr( $template ); ?>">
				<?php
				// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo wpforms_mailerlite()->get( 'template' )->get_builder_template( $template );
				?>
			</script>
			<?php
		endforeach;
	}

	/**
	 * Enqueue JavaScript and CSS files.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_assets() {

		parent::enqueue_assets();

		$min = wpforms_get_min_suffix();

		wp_enqueue_script(
			'wpforms-mailerlite-admin-builder',
			WPFORMS_MAILERLITE_URL . "assets/js/mailerlite-builder{$min}.js",
			[ 'wpforms-admin-builder-providers', 'choicesjs' ],
			WPFORMS_MAILERLITE_VERSION,
			true
		);

		wp_localize_script(
			'wpforms-mailerlite-admin-builder',
			'wpformsMailerLiteBuilderVars',
			[
				'i18n' => [
					'provider_placeholder' => esc_html__( '--- Select MailerLite Field ---', 'wpforms-mailerlite' ),
				],
			]
		);
	}

	/**
	 * Enqueue an inline CSS styles.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_inline_css() {

		wp_add_inline_style(
			'wpforms-builder',
			/** @lang CSS */
			'#wpforms-panel-providers .choices.wpforms-error .choices__inner {
				border: 1px solid #d63638; border-radius: 4px;
			}'
		);
	}
}
