<?php

namespace WPFormsMailerlite\Provider\Settings;

use WPFormsMailerlite\Plugin;

/**
 * Class for helper methods needed inside the addon.
 *
 * @since 1.0.0
 */
class Helper {

	/**
	 * Sanitize connection.
	 *
	 * @since 1.0.0
	 *
	 * @param array $connection_data Connection data.
	 *
	 * @return array
	 */
	public function sanitize_connection( $connection_data ) {

		if ( ! empty( $connection_data['fields_required'] ) ) {
			$connection_data['fields_required'] = $this->sanitize_required_fields( $connection_data['fields_required'] );
		}

		if ( ! empty( $connection_data['fields_meta'] ) ) {
			$connection_data['fields_meta'] = $this->sanitize_custom_fields( $connection_data['fields_meta'] );
		}

		// Groups are handled separately.
		if ( ! empty( $connection_data['groups'] ) ) {
			$connection_data['groups'] = $this->sanitize_groups( $connection_data );
		}

		return $connection_data;
	}

	/**
	 * Sanitize required fields.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields List of fields.
	 *
	 * @return array
	 */
	private function sanitize_required_fields( $fields ) {

		foreach ( $fields as $key => $field ) {

			if ( isset( $field['field_id'] ) ) {

				if ( $field['field_id'] !== '' ) {
					$fields[ $key ]['field_id'] = absint( $field['field_id'] );
				}

				continue;
			}

			if ( ! isset( $field['value'] ) || wpforms_is_empty_string( $field['value'] ) ) {
				continue;
			}

			$fields[ $key ]['value'] = $field['name'] === 'note' ? sanitize_textarea_field( $field['value'] ) : sanitize_text_field( $field['value'] );
		}

		return $fields;
	}

	/**
	 * Sanitize custom fields.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields List of fields.
	 *
	 * @return array
	 */
	private function sanitize_custom_fields( $fields ) {

		if ( empty( $fields ) ) {
			return [];
		}

		foreach ( $fields as $key => $field ) {

			// Remove field if name or field_id is empty.
			if (
				! isset( $field['name'], $field['field_id'] )
				|| wpforms_is_empty_string( $field['name'] )
				|| wpforms_is_empty_string( $field['field_id'] )
			) {
				unset( $fields[ $key ] );
				continue;
			}

			$fields[ $key ]['name']     = sanitize_text_field( $field['name'] );
			$fields[ $key ]['field_id'] = absint( $field['field_id'] );
		}

		return $fields;
	}

	/**
	 * Sanitize groups.
	 *
	 * @since 1.0.0
	 *
	 * @param array $connection_data Connection data.
	 *
	 * @return array
	 */
	private function sanitize_groups( $connection_data ) {

		// We use groups data from $_POST variable.
		$provider_slug = Plugin::SLUG;
		$connection_id = $connection_data['id'];

		// phpcs:ignore WordPress.Security.NonceVerification, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$form_post = ! empty( $_POST['data'] ) ? json_decode( wp_unslash( $_POST['data'] ), true ) : [];
		$fields    = $connection_data['groups'];

		$groups = wpforms_chain( $form_post )
			->map(
				static function( $post_pair ) use ( $provider_slug, $connection_id ) {

					if (
						empty( $post_pair['name'] ) ||
						$post_pair['name'] !== "providers[{$provider_slug}][{$connection_id}][groups][group_id][]"
					) {
						return '';
					}

					return $post_pair['value'];
				}
			)
			->array_filter()
			->array_values()
			->value();

		if ( ! empty( $groups ) ) {
			$fields['group_id'] = $groups;
		}

		return $fields;
	}
}
