<?php

namespace WPFormsOfflineForms;

use WPForms_Builder_Panel_Settings;
use WPForms_Updater;

/**
 * Offline Forms.
 *
 * @since 1.0.0
 */
class Plugin {

	/**
	 * Key used to save and retrieve from form_data.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	const KEY = 'offline_form';

	/**
	 * Get a single instance of the class.
	 *
	 * @since 1.3.0
	 *
	 * @return Plugin
	 */
	public static function get_instance() {

		static $instance;

		if ( ! $instance ) {
			$instance = new self();

			$instance->init();
		}

		return $instance;
	}

	/**
	 * Init class.
	 *
	 * @since 1.3.0
	 *
	 * @return Plugin
	 */
	private function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Perform all required add_action() and add_filter().
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_form_settings_general', [ $this, 'admin_register_setting' ] );
		add_action( 'wpforms_wp_footer', [ $this, 'frontend_enqueue' ] );

		add_action( 'wpforms_updater', [ $this, 'updater' ] );
	}

	/**
	 * Load the plugin updater.
	 *
	 * @since 1.0.0
	 *
	 * @param string $key License key.
	 */
	public function updater( $key ) {

		new WPForms_Updater(
			[
				'plugin_name' => 'WPForms Offline Forms',
				'plugin_slug' => 'wpforms-offline-forms',
				'plugin_path' => plugin_basename( WPFORMS_OFFLINE_FORMS_FILE ),
				'plugin_url'  => trailingslashit( WPFORMS_OFFLINE_FORMS_URL ),
				'remote_url'  => WPFORMS_UPDATER_API,
				'version'     => WPFORMS_OFFLINE_FORMS_VERSION,
				'key'         => $key,
			]
		);
	}

	/**
	 * Register an option for forms to enable offline functionality.
	 *
	 * @since 1.0.0
	 *
	 * @param WPForms_Builder_Panel_Settings $settings Settings panel object.
	 */
	public function admin_register_setting( $settings ) {

		wpforms_panel_field(
			'toggle',
			'settings',
			self::KEY,
			$settings->form_data,
			esc_html__( 'Enable offline mode for the form', 'wpforms-offline-forms' ),
			[
				'tooltip' => esc_html__( 'This will allow users to submit forms even without internet connection. Form data will be saved into their browsers and will give them ability to resubmit once internet is back with just 1 click of a button.', 'wpforms-offline-forms' ),
			]
		);
	}

	/**
	 * Enqueue assets on front-end.
	 *
	 * @since 1.0.0
	 *
	 * @param array $forms List of forms.
	 */
	public function frontend_enqueue( $forms ) {

		$offline_forms = [];

		// Check if form(s) on a page has an enabled offline functionality.
		foreach ( $forms as $form ) {
			if ( $this->is_enabled( $form ) ) {
				$offline_forms[] = (int) $form['id'];
			}
		}

		if ( empty( $offline_forms ) ) {
			return;
		}

		$min = wpforms_get_min_suffix();

		/*
		 * JavaScript.
		 */
		wp_enqueue_script(
			'wpforms-offline-forms-store',
			WPFORMS_OFFLINE_FORMS_URL . "assets/js/store2{$min}.js",
			[ 'wpforms' ],
			'2.5.3'
		);
		wp_enqueue_script(
			'wpforms-offline-forms-deserialize',
			WPFORMS_OFFLINE_FORMS_URL . "assets/js/jquery.deserialize{$min}.js",
			[ 'wpforms' ],
			'2.0.0-rc1'
		);
		wp_enqueue_script(
			'wpforms-offline-forms',
			WPFORMS_OFFLINE_FORMS_URL . "assets/js/wpforms-offline-forms{$min}.js",
			[ 'wpforms' ],
			WPFORMS_OFFLINE_FORMS_VERSION
		);

		// phpcs:ignore WPForms.Comments.PHPDocHooks.RequiredHookDocumentation, WPForms.PHP.ValidateHooks.InvalidHookName
		$data = apply_filters(
			'wpforms_offline_forms_frontend_localize_data',
			[
				'offline_form_ids'          => $offline_forms,
				'check_connection_url'      => WPFORMS_OFFLINE_FORMS_URL . 'assets/connection.txt', // should point to own domain unless CORS is configured.
				'check_connection_interval' => 30, // seconds.
				'check_connection_timeout'  => 3, // seconds.
				'text_offline'              => wpautop( esc_html__( "You appear to be offline right now. In order to save your progress, please submit this form.\n\nYour data will be saved in your browser, even if you close this page. Once your internet connection is restored, be sure to return here and complete the form submission.", 'wpforms-offline-forms' ) ),
				'text_restore_single'       => '<p>' . esc_html__( 'We\'ve saved your information from your previous attempt to complete this form. You can restore your saved data when your internet connection is active.', 'wpforms-offline-forms' ) . '</p>',
				'text_restore_plural'       => '<p>' . esc_html__( 'We see you previously attempted to complete this form. Would you like to restore then to submit one by one? After restoring, your browser will no longer have this data saved.', 'wpforms-offline-forms' ) . '</p>',
				'text_restore_btn'          => esc_html__( 'Restore', 'wpforms-offline-forms' ),
				'text_clear_btn'            => esc_html__( 'Clear', 'wpforms-offline-forms' ),
				'text_clear_all_btn'        => esc_html__( 'Clear All', 'wpforms-offline-forms' ),
			]
		);

		wp_localize_script( 'wpforms-offline-forms', 'wpforms_offline_forms', $data );
	}

	/**
	 * Helper function that checks if offline form functionality is enabled on a form.
	 * Making it static to be able to reuse elsewhere without class initialization.
	 *
	 * @since 1.0.0
	 *
	 * @param array $form_data Form data.
	 *
	 * @return bool
	 */
	public function is_enabled( $form_data ) {

		return ! empty( $form_data['settings'][ self::KEY ] );
	}
}
