<?php

namespace WPFormsPaypalCommerce\Process;

use WPFormsPaypalCommerce\Admin\Connect;
use WPFormsPaypalCommerce\Connection;
use WPFormsPaypalCommerce\Helpers;

/**
 * Base payment processing.
 *
 * @since 1.0.0
 */
abstract class Base {

	/**
	 * Form ID.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected $form_id = 0;

	/**
	 * Sanitized submitted field values and data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $fields = [];

	/**
	 * Form data and settings.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $form_data = [];

	/**
	 * Payment amount.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $amount = '';

	/**
	 * Payment currency.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $currency = '';

	/**
	 * Connection data.
	 *
	 * @since 1.0.0
	 *
	 * @var Connection
	 */
	protected $connection;

	/**
	 * PayPal Commerce form errors.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected $errors = [];

	/**
	 * Check form settings, fields, etc.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	protected function is_form_ok() {

		if ( ! $this->connection ) {
			$error_title    = esc_html__( 'This payment cannot be processed because the account connection is missing.', 'wpforms-paypal-commerce' );
			$this->errors[] = $error_title;

			$this->log_errors( $error_title );

			return false;
		}

		if ( $this->connection->is_access_token_expired() ) {
			Connect::refresh_access_token( $this->connection );
		}

		if ( ! $this->connection->is_usable() ) {
			$error_title    = esc_html__( 'This payment cannot be processed because the account connection is expired or invalid.', 'wpforms-paypal-commerce' );
			$this->errors[] = $error_title;

			$this->log_errors( $error_title );

			return false;
		}

		return true;
	}

	/**
	 * Check if conditional logic check passes for the given settings.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings Conditional logic settings to process.
	 *
	 * @return bool
	 */
	protected function is_conditional_logic_ok( $settings ) {

		if (
			empty( $settings['conditional_logic'] ) ||
			empty( $settings['conditional_type'] ) ||
			empty( $settings['conditionals'] )
		) {
			return true;
		}

		$process = wpforms_conditional_logic()->process( $this->fields, $this->form_data, $settings['conditionals'] );

		if ( $settings['conditional_type'] === 'stop' ) {
			$process = ! $process;
		}

		return $process;
	}

	/**
	 * Retrieve a payment currency.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	protected function get_currency() {

		return strtoupper( wpforms_get_currency() );
	}

	/**
	 * Retrieve a payment amount.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	protected function get_amount() {

		$amount = wpforms_get_total_payment( $this->fields );

		return $amount === false ? 0 : $amount;
	}

	/**
	 * Log payment errors.
	 *
	 * @since 1.0.0
	 *
	 * @param string       $title    Error title.
	 * @param array|string $messages Error messages.
	 * @param string       $level    Error level to add to 'payment' error level.
	 */
	protected function log_errors( $title, $messages = [], $level = 'error' ) {

		Helpers::log_errors( $title, $this->form_id, $messages, $level );
	}

	/**
	 * Determine if address field is valid.
	 *
	 * @since 1.9.0
	 *
	 * @param array  $submitted_data Submitted data.
	 * @param string $address_field  Address field id.
	 *
	 * @return bool
	 */
	protected function is_address_field_valid( array $submitted_data, string $address_field ): bool {

		return ! empty( $submitted_data['fields'][ $address_field ]['address1'] ) &&
			! empty( $submitted_data['fields'][ $address_field ]['city'] ) &&
			! empty( $submitted_data['fields'][ $address_field ]['postal'] ) &&
			( $this->form_data['fields'][ $address_field ]['scheme'] !== 'international' || ! empty( $submitted_data['fields'][ $address_field ]['country'] ) );
	}
}
