/* global wpforms_builder */

'use strict';

/**
 * WPForms Square builder function.
 *
 * @since 1.0.0
 */
const WPFormsBuilderSquare = window.WPFormsBuilderSquare || ( function( document, window, $ ) {

	/**
	 * Elements holder.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	const el = {};

	/**
	 * Public functions and properties.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	const app = {

		/**
		 * Start the engine.
		 *
		 * @since 1.0.0
		 */
		init: function() {

			$( app.ready );
		},

		/**
		 * Initialized once the DOM and Providers are fully loaded.
		 *
		 * @since 1.0.0
		 */
		ready: function() {

			const $providerHolder = $( '#square-provider' );

			// Cache DOM elements.
			el.$singlePaymentControl  = $( '#wpforms-panel-field-square-enable' );
			el.$singlePaymentSettings = $providerHolder.find( '.wpforms-square-panel-fields' );
			el.$AJAXSubmitOption      = $( '#wpforms-panel-field-settings-ajax_submit' );
			el.$cardButton            = $( '#wpforms-add-fields-square' );
			el.$alert                 = $( '#square-credit-card-alert' );

			app.bindUIActions();
		},

		/**
		 * Process various events.
		 *
		 * @since 1.0.0
		 */
		bindUIActions: function() {

			$( document ).on( 'wpformsSaved', app.ajaxRequiredCheck );
			$( document ).on( 'wpformsSaved', app.paymentsEnabledCheck );

			$( document ).on( 'wpformsFieldAdd', app.fieldAdded );
			$( document ).on( 'wpformsFieldDelete', app.fieldDeleted );

			el.$cardButton.on( 'click', app.connectionCheck );
			el.$singlePaymentControl.on( 'change', app.singleSettingsToggle );
		},

		/**
		 * Notify user if AJAX submission is not required.
		 *
		 * @since 1.0.0
		 */
		ajaxRequiredCheck: function() {

			if ( ! $( '#wpforms-panel-fields .wpforms-field.wpforms-field-square' ).length ) {
				return;
			}

			if ( app.isAJAXSubmitEnabled() ) {
				return;
			}

			$.alert( {
				title: wpforms_builder.heads_up,
				content: wpforms_builder.square_ajax_required,
				icon: 'fa fa-exclamation-circle',
				type: 'orange',
				buttons: {
					confirm: {
						text: wpforms_builder.ok,
						btnClass: 'btn-confirm',
						keys: [ 'enter' ],
					},
				},
			} );
		},

		/**
		 * Notify user if Square Payments are not enabled.
		 *
		 * @since 1.0.0
		 */
		paymentsEnabledCheck: function() {

			if ( ! $( '#wpforms-panel-fields .wpforms-field.wpforms-field-square' ).length ) {
				return;
			}

			if ( app.isPaymentsEnabled() ) {
				return;
			}

			$.alert( {
				title: wpforms_builder.heads_up,
				content: wpforms_builder.square_payments_enabled_required,
				icon: 'fa fa-exclamation-circle',
				type: 'orange',
				buttons: {
					confirm: {
						text: wpforms_builder.ok,
						btnClass: 'btn-confirm',
						keys: [ 'enter' ],
					},
				},
			} );
		},

		/**
		 * Notify user if Square connection are missing.
		 *
		 * @since 1.0.0
		 *
		 * @returns {boolean} False if button clicks should be prevented.
		 */
		connectionCheck: function() {

			if ( $( this ).hasClass( 'wpforms-add-fields-button-disabled' ) ) {
				return false;
			}

			if ( ! $( this ).hasClass( 'square-connection-required' ) ) {
				return true;
			}

			$.alert( {
				title: wpforms_builder.heads_up,
				content: wpforms_builder.square_connection_required,
				icon: 'fa fa-exclamation-circle',
				type: 'orange',
				buttons: {
					confirm: {
						text: wpforms_builder.ok,
						btnClass: 'btn-confirm',
						keys: [ 'enter' ],
					},
				},
			} );
		},

		/**
		 * We have to do several actions when the "Square" field is added.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} e Event object.
		 * @param {number} id Field ID.
		 * @param {string} type Field type.
		 */
		fieldAdded: function( e, id, type ) {

			if ( type === 'square' ) {
				app.cardButtonToggle( true );
				app.settingsToggle( true );
				app.paymentsEnabledCheck();
			}
		},

		/**
		 * We have to do several actions for UI when the "Square" credit card field is deleted.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} e Event object.
		 * @param {number} id Field ID.
		 * @param {string} type Field type.
		 */
		fieldDeleted: function( e, id, type ) {

			if ( type === 'square' ) {
				app.cardButtonToggle( false );
				app.settingsToggle( false );
				app.disableNotifications();
			}
		},

		/**
		 * Enable or disable the "Square" field in the fields list.
		 *
		 * @since 1.0.0
		 *
		 * @param {boolean} isDisabled If true then a card button will be disabled.
		 */
		cardButtonToggle: function( isDisabled ) {

			el.$cardButton
				.prop( 'disabled', isDisabled )
				.toggleClass( 'wpforms-add-fields-button-disabled', isDisabled );
		},

		/**
		 * Toggle visibility of the Square payment settings.
		 *
		 * If the "Square" field has been added then reveal the settings,
		 * otherwise hide them.
		 *
		 * @since 1.0.0
		 *
		 * @param {boolean} display Show or hide settings.
		 */
		settingsToggle: function( display ) {

			if ( ! el.$alert.length ) {
				return;
			}

			el.$alert.toggleClass( 'wpforms-hidden', display );
			$( '#wpforms-panel-field-square-enable-wrap' ).toggleClass( 'wpforms-hidden', ! display );

			// Uncheck the Payments > Square > Enable Square Payments setting.
			if ( ! display ) {
				el.$singlePaymentControl.prop( 'checked', false ).trigger( 'change' );
			}
		},

		/**
		 * Toggle visibility of the Square Single Payments settings.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} e Event object.
		 */
		singleSettingsToggle: function( e ) {

			el.$singlePaymentSettings.toggleClass( 'wpforms-hidden', ! e.target.checked );
		},

		/**
		 * Disable notifications.
		 *
		 * @since 1.0.0
		 */
		disableNotifications: function() {

			const $notificationWrap = $( '.wpforms-panel-content-section-notifications [id*="-square-wrap"]' );

			$notificationWrap.find( 'input[id*="-square"]' ).prop( 'checked', false );
			$notificationWrap.addClass( 'wpforms-hidden' );
		},

		/**
		 * Determine whether payments are enabled in the Payments > Square panel.
		 *
		 * @since 1.0.0
		 *
		 * @returns {boolean} Payments are enabled.
		 */
		isPaymentsEnabled: function() {

			return el.$singlePaymentControl.is( ':checked' );
		},

		/**
		 * Determine whether AJAX form submission is enabled in the Settings > General.
		 *
		 * @since 1.0.0
		 *
		 * @returns {boolean} AJAX form submission is enabled.
		 */
		isAJAXSubmitEnabled: function() {

			return el.$AJAXSubmitOption.is( ':checked' );
		},
	};

	// Provide access to public functions/properties.
	return app;

}( document, window, jQuery ) );

// Initialize.
WPFormsBuilderSquare.init();
