/* global wpforms_admin, WPFormsAdmin */

'use strict';

/**
 * WPForms Square settings function.
 *
 * @since 1.0.0
 */
const WPFormsSettingsSquare = window.WPFormsSettingsSquare || ( function( document, window, $ ) {

	/**
	 * Elements.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	const $el = {
		sandboxModeCheckbox: $( '#wpforms-setting-square-sandbox-mode' ),
		sandboxConnectionStatusBlock: $( '#wpforms-setting-row-square-connection-status-sandbox' ),
		productionConnectionStatusBlock: $( '#wpforms-setting-row-square-connection-status-production' ),
		sandboxLocationBlock: $( '#wpforms-setting-row-square-location-id-sandbox' ),
		productionLocationBlock: $( '#wpforms-setting-row-square-location-id-production' ),
		refreshBtn: $( '.wpforms-square-refresh-btn' ),
	};

	/**
	 * Public functions and properties.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	const app = {

		/**
		 * Start the engine.
		 *
		 * @since 1.0.0
		 */
		init: function() {

			$( app.ready );
		},

		/**
		 * Document ready.
		 *
		 * @since 1.0.0
		 */
		ready: function() {

			app.events();
		},

		/**
		 * Register JS events.
		 *
		 * @since 1.0.0
		 */
		events: function() {

			$el.sandboxModeCheckbox.on( 'change', app.credentialsFieldsDisplay );
			$el.refreshBtn.on( 'click', app.refreshTokensCallback );
		},

		/**
		 * Refresh tokens.
		 *
		 * @since 1.0.0
		 */
		refreshTokensCallback: function() {

			const $btn        = $( this );
			const buttonWidth = $btn.outerWidth();
			const buttonLabel = $btn.text();
			const settings    = {
				url: wpforms_admin.ajax_url,
				type: 'post',
				dataType: 'json',
				data: {
					action: 'wpforms_square_refresh_connection',
					nonce: wpforms_admin.nonce,
					mode: $btn.data( 'mode' ),
				},
				beforeSend: function() {

					$btn.css( 'width', buttonWidth ).html( WPFormsAdmin.settings.iconSpinner ).prop( 'disabled', true );
				},
			};

			let errorMessage = wpforms_admin.square.refresh_error;

			// Perform an Ajax request.
			$.ajax( settings )
				.done( function( response ) {

					if ( response.success ) {
						$btn
							.css( 'pointerEvents', 'none' )
							.removeClass( 'wpforms-btn-light-grey' )
							.addClass( 'wpforms-btn-grey' )
							.html( 'Refreshed!' );

						$btn.closest( 'form' ).css( 'cursor', 'wait' );

						window.location = $btn.data( 'url' );

						return;
					}

					if (
						Object.prototype.hasOwnProperty.call( response, 'data' ) &&
						response.data !== ''
					) {
						errorMessage = response.data;
					}

					$btn
						.css( 'width', 'auto' )
						.html( buttonLabel )
						.prop( 'disabled', false );
					app.modals.refreshTokensError( errorMessage );
				} )
				.fail( function() {

					$btn
						.css( 'width', 'auto' )
						.html( buttonLabel )
						.prop( 'disabled', false );
					app.modals.refreshTokensError( errorMessage );
				} );
		},

		/**
		 * Conditionally show Square mode switch warning.
		 *
		 * @since 1.0.0
		 */
		credentialsFieldsDisplay: function() {

			const sandboxModeEnabled = $el.sandboxModeCheckbox.is( ':checked' );

			if ( sandboxModeEnabled ) {
				$el.sandboxConnectionStatusBlock.show();
				$el.sandboxLocationBlock.show();

				$el.productionConnectionStatusBlock.hide();
				$el.productionLocationBlock.hide();
			} else {
				$el.sandboxConnectionStatusBlock.hide();
				$el.sandboxLocationBlock.hide();

				$el.productionConnectionStatusBlock.show();
				$el.productionLocationBlock.show();
			}

			if ( sandboxModeEnabled && $el.sandboxConnectionStatusBlock.find( '.wpforms-square-connected' ).length ) {
				return;
			}

			if ( ! sandboxModeEnabled && $el.productionConnectionStatusBlock.find( '.wpforms-square-connected' ).length ) {
				return;
			}

			app.modals.modeChangedWarning();
		},

		/**
		 * Modals.
		 *
		 * @since 1.0.0
		 */
		modals: {

			/**
			 * Show the warning modal when Square mode is changed.
			 *
			 * @since 1.0.0
			 */
			modeChangedWarning: function() {

				$.alert( {
					title: wpforms_admin.heads_up,
					content: wpforms_admin.square.mode_update,
					icon: 'fa fa-exclamation-circle',
					type: 'orange',
					buttons: {
						confirm: {
							text: wpforms_admin.ok,
							btnClass: 'btn-confirm',
							keys: [ 'enter' ],
						},
					},
				} );
			},

			/**
			 * Refresh tokens error handling.
			 *
			 * @since 1.0.0
			 *
			 * @param {string} error Error message.
			 */
			refreshTokensError: function( error ) {

				$.alert( {
					title: false,
					content: error,
					icon: 'fa fa-exclamation-circle',
					type: 'orange',
					buttons: {
						confirm: {
							text: wpforms_admin.ok,
							btnClass: 'btn-confirm',
							keys: [ 'enter' ],
						},
					},
				} );
			},
		},
	};

	// Provide access to public functions/properties.
	return app;

}( document, window, jQuery ) );

// Initialize.
WPFormsSettingsSquare.init();
