/* global Square, wpforms, wpforms_settings, wpforms_square, WPForms */

'use strict';

/**
 * WPForms Square function.
 *
 * @since 1.0.0
 */
const WPFormsSquare = window.WPFormsSquare || ( function( document, window, $ ) {

	/**
	 * Holder for original form submit handler.
	 *
	 * @since 1.2.0
	 *
	 * @type {Function}
	 */
	let originalSubmitHandler;

	/**
	 * Credit card data.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	const cardData = {
		cardNumber: {
			empty: true,
			valid: false,
		},
		expirationDate: {
			empty: true,
			valid: false,
		},
		cvv: {
			empty: true,
			valid: false,
		},
		postalCode: {
			empty: true,
			valid: false,
		},
	};

	/**
	 * Public functions and properties.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	const app = {

		/**
		 * Square payments object.
		 *
		 * @since 1.0.0
		 *
		 * @type {object}
		 */
		payments: null,

		/**
		 * Number of page locked to switch.
		 *
		 * @since 1.4.0
		 *
		 * @type {int}
		 */
		lockedPageToSwitch: 0,

		/**
		 * Start the engine.
		 *
		 * @since 1.0.0
		 */
		init: function() {

			app.payments = app.getPaymentsInstance();

			// Bail if a Square payments object isn't initialized.
			if ( app.payments === null ) {
				return;
			}

			$( document )
				.on( 'wpformsReady', app.setupForms )
				.on( 'wpformsBeforePageChange', app.pageChange )
				.on( 'wpformsPageChange', app.afterPageChange )
				.on( 'wpformsProcessConditionalsField', app.conditionalLogicHandler );
		},

		/**
		 * Setup and configure Square forms.
		 *
		 * @since 1.0.0
		 */
		setupForms: function() {

			if ( typeof $.fn.validate === 'undefined' ) {
				return;
			}

			$( '.wpforms-square form' )
				.filter( ( _, form ) => typeof $( form ).data( 'formid' ) === 'number' ) // filter out forms which are locked (formid changed to 'locked-...').
				.each( app.updateSubmitHandler );
		},

		/**
		 * Update submitHandler for the forms containing Square.
		 *
		 * @since 1.0.0
		 */
		updateSubmitHandler: async function() {

			const $form     = $( this );
			const validator = $form.data( 'validator' );

			if ( ! validator ) {
				return;
			}

			// Store the original submitHandler.
			originalSubmitHandler = validator.settings.submitHandler;

			// Replace the default submit handler.
			validator.settings.submitHandler = app.submitHandler;

			// Get a new Card object.
			await app.getCardInstance( $form );
		},

		/**
		 * Trigger resize event if Square field has been conditionally unhidden.
		 *
		 * Allows Square Card field to resize itself (fixes the issue with doubled field height on some screen resolutions).
		 *
		 * @since 1.4.0
		 *
		 * @param {Event} e Event.
		 * @param {int} formID Form ID.
		 * @param {int} fieldID Field ID.
		 * @param {boolean} pass Pass condition logic.
		 * @param {string} action Action name.
		 */
		conditionalLogicHandler: function( e, formID, fieldID, pass, action ) {

			if ( ! app.isVisibleField( pass, action ) ) {
				return;
			}

			const el = document.getElementById( 'wpforms-' + formID + '-field_' + fieldID );

			if ( ! el || ! el.classList.contains( 'wpforms-field-square-cardnumber' ) ) {
				return;
			}

			window.dispatchEvent( new Event( 'resize' ) );
		},

		/**
		 * Determine if the field is visible after being triggered by Conditional Logic.
		 *
		 * @since 1.4.0
		 *
		 * @param {boolean} pass Pass condition logic.
		 * @param {string} action Action name.
		 *
		 * @returns {boolean} The field is visible.
		 */
		isVisibleField: function( pass, action ) {

			return ( action === 'show' && pass ) || ( action === 'hide' && ! pass );
		},

		/**
		 * Update submitHandler for forms containing Square.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} form JS form element.
		 */
		submitHandler: function( form ) {

			const $form     = $( form );
			const validator = $form.data( 'validator' );
			const validForm = validator.form();
			const card      = $form.find( '.wpforms-square-credit-card-hidden-input' ).data( 'square-card' );

			if ( ! validForm || typeof card === 'undefined' || ! app.isProcessedCard( $form ) ) {
				originalSubmitHandler( $form );
				return;
			}

			app.tokenize( $form, card );
		},

		/**
		 * Tokenize a card payment.
		 *
		 * @param {jQuery} $form Form element.
		 * @param {object} card  Square Card object.
		 */
		tokenize: async function( $form, card ) {

			app.disableSubmitBtn( $form );

			const sourceId = await app.getSourceId( $form, card );

			if ( sourceId === null ) {
				app.enableSubmitBtn( $form );
				return;
			}

			const verifyToken = await app.getVerificationToken( $form, sourceId, 'charge' );

			if ( verifyToken === null ) {
				app.enableSubmitBtn( $form );
				return;
			}

			app.submitForm( $form );
		},

		/**
		 * Initialize a Square payments object and retrieve it.
		 *
		 * @since 1.0.0
		 *
		 * @returns {object|null} Square payments object or null.
		 */
		getPaymentsInstance: function() {

			if ( ! window.Square ) {
				app.displaySdkError( $( '.wpforms-square form' ), wpforms_square.i18n.missing_sdk_script );

				return null;
			}

			try {
				return Square.payments( wpforms_square.client_id, wpforms_square.location_id );

			} catch ( e ) {

				const message = ( typeof e === 'object' && Object.prototype.hasOwnProperty.call( e, 'message' ) ) ?
					e.message :
					wpforms_square.i18n.missing_creds;

				app.displaySdkError( $( '.wpforms-square form' ), message );

				return null;
			}
		},

		/**
		 * Try to retrieve a Square Card object.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 *
		 * @returns {object|null} Square Card object or null.
		 */
		getCardInstance: async function( $form ) {

			// Applying the modern styles to the card config if needed.
			wpforms_square.card_config.style = wpforms_square.card_config.style ? wpforms_square.card_config.style : app.getModernMarkupCardStyles( $form );

			try {
				const card = await app.payments.card( wpforms_square.card_config );

				// Attach the Card form to the page.
				await card.attach( $form.find( '.wpforms-field-square-cardnumber' ).get( 0 ) );

				const eventsList   = [ 'focusClassAdded', 'focusClassRemoved' ];
				const eventsLength = eventsList.length;
				let counter        = 0;

				// Bind the Card events.
				for ( ; counter < eventsLength; counter++ ) {
					card.addEventListener( eventsList[ counter ], function( e ) {

						// Card field is filled.
						cardData[ e.detail.field ].empty = e.detail.currentState.isEmpty;
						cardData[ e.detail.field ].valid = e.detail.currentState.isCompletelyValid;
					} );
				}

				$form.find( '.wpforms-square-credit-card-hidden-input' ).data( 'square-card', card );

				return card;

			} catch ( e ) {
				app.displaySdkError( $form, wpforms_square.i18n.card_init_error );
				console.log( 'Error:', e );
				console.log( 'Config', wpforms_square.card_config );

				return null;
			}
		},

		/**
		 * Retrieve a source ID (card nonce).
		 *
		 * @param {jQuery} $form Form element.
		 * @param {object} card  Square Card object.
		 *
		 * @returns {string|null} The source ID or null.
		 */
		getSourceId: async function( $form, card ) {

			try {
				const response = await card.tokenize();

				$form.find( '.wpforms-square-payment-source-id' ).remove();

				if ( response.status !== 'OK' || ! response.token ) {
					app.displayFormError( app.getCreditCardInput( $form ), app.getResponseError( response ) );

					return null;
				}

				$form.append( '<input type="hidden" name="wpforms[square][source_id]" class="wpforms-square-payment-source-id" value="' + app.escapeTextString( response.token ) + '">' );

				return response.token;

			} catch ( e ) {
				app.displayFormError( app.getCreditCardInput( $form ), wpforms_square.i18n.token_process_fail );
			}

			return null;
		},

		/**
		 * Retrieve a response error message.
		 *
		 * @param {object} response The response received from a tokenization call.
		 *
		 * @returns {string} The response error message.
		 */
		getResponseError: function( response ) {

			const hasErrors = response.errors && Array.isArray( response.errors ) && response.errors.length;

			return hasErrors ?
				response.errors[0].message :
				wpforms_square.i18n.token_status_error + ' ' + response.status;
		},

		/**
		 * Retrieve a verification token.
		 *
		 * @param {jQuery} $form  Form element.
		 * @param {string} source The nonce received from a tokenization call.
		 * @param {string} intent Transactional intent of the payment.
		 *
		 * @returns {string|null} The verification token or null.
		 */
		getVerificationToken: async function( $form, source, intent ) {

			try {
				const intents = [ 'store', 'charge' ];

				if ( ! intents.includes( intent ) ) {
					app.displayFormError( app.getCreditCardInput( $form ), wpforms_square.i18n.buyer_verify_error );

					return null;
				}

				const args     = intent === 'store' ? app.getStoreVerifyBuyerDetails() : app.getChargeVerifyBuyerDetails( $form );
				const response = await app.payments.verifyBuyer( source, args );

				$form.find( '.wpforms-square-payment-verify-' + intent ).remove();

				if ( response.token ) {
					$form.append( '<input type="hidden" name="wpforms[square][verify][' + intent + ']" class="wpforms-square-payment-verify-' + intent + '" value="' + app.escapeTextString( response.token ) + '">' );

					return response.token;
				}

				app.displayFormError( app.getCreditCardInput( $form ), wpforms_square.i18n.buyer_verify_error );

			} catch ( e ) {
				app.displayFormError( app.getCreditCardInput( $form ), wpforms_square.i18n.buyer_verify_error );
			}

			return null;
		},

		/**
		 * Retrieve details about the buyer for a charge.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 *
		 * @returns {object} Buyer details.
		 */
		getChargeVerifyBuyerDetails: function( $form ) {

			const args = {
				amount: wpforms.amountFormat( wpforms.amountTotalCalc( $form ) ),
				billingContact: {},
				currencyCode: wpforms_settings.currency_code,
				intent: 'CHARGE',
			};

			if (
				typeof wpforms_square === 'object' &&
				Object.prototype.hasOwnProperty.call( wpforms_square, 'verify_buyer_details' ) &&
				Object.prototype.hasOwnProperty.call( wpforms_square.verify_buyer_details, 'charge' )
			) {
				$.extend( args, wpforms_square.verify_buyer_details.charge );
			}

			return args;
		},

		/**
		 * Retrieve details about the buyer for saving a credit card.
		 *
		 * @since 1.0.0
		 *
		 * @returns {object} Buyer details.
		 */
		getStoreVerifyBuyerDetails: function() {

			const args = {
				billingContact: {},
				intent: 'STORE',
			};

			if (
				typeof wpforms_square === 'object' &&
				Object.prototype.hasOwnProperty.call( wpforms_square, 'verify_buyer_details' ) &&
				Object.prototype.hasOwnProperty.call( wpforms_square.verify_buyer_details, 'store' )
			) {
				$.extend( args, wpforms_square.verify_buyer_details.store );
			}

			return args;
		},

		/**
		 * Retrieve a jQuery selector for Credit Card hidden input.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 *
		 * @returns {jQuery} Credit Card hidden input.
		 */
		getCreditCardInput: function( $form ) {

			return $form.find( '.wpforms-square-credit-card-hidden-input' );
		},

		/**
		 * Submit the form using the original submitHandler.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 */
		submitForm: function( $form ) {

			const validator = $form.data( 'validator' );

			if ( validator ) {
				originalSubmitHandler( $form );
			}
		},

		/**
		 * Determine if a credit card should be processed.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 *
		 * @returns {boolean} True if a credit card should be processed.
		 */
		isProcessedCard: function( $form ) {

			const $squareDiv = $form.find( '.wpforms-field-square-cardnumber' );
			const condHidden = $squareDiv.closest( '.wpforms-field-square' ).hasClass( 'wpforms-conditional-hide' );
			const ccRequired = !! $squareDiv.data( 'required' );

			if ( condHidden ) {
				return false;
			}

			return ccRequired || app.isCardDataNotEmpty();
		},

		/**
		 * Determine if card data not empty.
		 *
		 * @since 1.0.0
		 *
		 * @returns {boolean} True if at least one credit card sub-field is filled.
		 */
		isCardDataNotEmpty: function() {

			return ! cardData.cardNumber.empty || ! cardData.expirationDate.empty || ! cardData.cvv.empty || ! cardData.postalCode.empty;
		},

		/**
		 * Determine if card data is completely valid.
		 *
		 * @since 1.4.0
		 *
		 * @returns {boolean} True if at least one credit card sub-field is filled.
		 */
		isCardDataValid: function() {

			return cardData.cardNumber.valid && cardData.expirationDate.valid && cardData.cvv.valid && cardData.postalCode.valid;
		},

		/**
		 * Display a SDK error.
		 *
		 * @param {jQuery} $form   Form element.
		 * @param {string} message Error messages.
		 *
		 * @since 1.0.0
		 */
		displaySdkError: function( $form, message ) {

			$form
				.find( '.wpforms-square-credit-card-hidden-input' )
				.closest( '.wpforms-field-square-number' )
				.append( $( '<label></label>', {
					text: message,
					class: 'wpforms-error',
				} ) );
		},

		/**
		 * Display a field error using jQuery Validate library.
		 *
		 * @param {jQuery} $field  Form element.
		 * @param {string} message Error messages.
		 *
		 * @since 1.0.0
		 */
		displayFormError: function( $field, message ) {

			const fieldName = $field.attr( 'name' );
			const $form     = $field.closest( 'form' );
			const error     = {};

			error[ fieldName ] = message;

			wpforms.displayFormAjaxFieldErrors( $form, error );
		},

		/**
		 * Disable submit button for the form.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 */
		disableSubmitBtn: function( $form ) {

			$form.find( '.wpforms-submit' ).prop( 'disabled', true );
		},

		/**
		 * Enable submit button for the form.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 */
		enableSubmitBtn: function( $form ) {

			$form.find( '.wpforms-submit' ).prop( 'disabled', false );
		},

		/**
		 * Replaces &, <, >, ", `, and ' with their escaped counterparts.
		 *
		 * @since 1.0.0
		 *
		 * @param {string} string String to escape.
		 *
		 * @returns {string} Escaped string.
		 */
		escapeTextString: function( string ) {

			return $( '<span></span>' ).text( string ).html();
		},

		/**
		 * Callback for a page changing.
		 *
		 * @since 1.4.0
		 *
		 * @param {Event}  event       Event.
		 * @param {int}    currentPage Current page.
		 * @param {jQuery} $form       Current form.
		 * @param {string} action      The navigation action.
		 */
		pageChange: function( event, currentPage, $form, action ) {

			const $squareDiv = $form.find( '.wpforms-field-square-cardnumber' );

			// Stop navigation through page break pages.
			if (
				! $squareDiv.is( ':visible' ) ||
				( ! $squareDiv.data( 'required' ) && ! app.isCardDataNotEmpty() ) ||
				( app.lockedPageToSwitch && app.lockedPageToSwitch !== currentPage ) ||
				action === 'prev'
			) {
				return;
			}

			if ( app.isCardDataValid() ) {
				$squareDiv.parent().find( 'label.wpforms-error' ).remove();

				return;
			}

			app.lockedPageToSwitch = currentPage;

			app.displayFormError( app.getCreditCardInput( $form ), wpforms_square.i18n.empty_details );
			event.preventDefault();
		},

		/**
		 * Callback for a after page changing.
		 *
		 * @since 1.8.0
		 */
		afterPageChange() {
			window.dispatchEvent( new Event( 'resize' ) );
		},

		/**
		 * Get modern card styles.
		 *
		 * @since 1.4.0
		 *
		 * @param {jQuery} $form Current form.
		 *
		 * @return {Object} Card styles object.
		 */
		getModernMarkupCardStyles( $form ) {
			if ( ! window.WPForms || ! WPForms.FrontendModern ) {
				return {};
			}

			const cssVars = WPForms.FrontendModern.getCssVars( $form );

			return {
				'input': {
					color: WPForms.FrontendModern.getColorWithOpacity( cssVars['field-text-color'], '1' ),
					fontSize: cssVars['field-size-font-size'],
				},
				'input::placeholder': {
					color: WPForms.FrontendModern.getColorWithOpacity( cssVars['field-text-color'], '0.5' ),
				},
				'input.is-error': {
					color: WPForms.FrontendModern.getColorWithOpacity( cssVars['label-error-color'], '1' ),
				},
			};
		},
	};

	// Provide access to public functions/properties.
	return app;

}( document, window, jQuery ) );

// Initialize.
WPFormsSquare.init();
