<?php

namespace WPFormsSquare\Admin;

use WPForms_Builder_Panel_Settings;
use WPFormsSquare\Helpers;

/**
 * Square Form Builder related functionality.
 *
 * @since 1.0.0
 */
class Builder {

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Builder hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_builder_enqueues', [ $this, 'enqueues' ] );
		add_filter( 'wpforms_builder_strings',  [ $this, 'javascript_strings' ] );
		add_action( 'wpforms_form_settings_notifications_single_after', [ $this, 'notification_settings' ], 10, 2 );
	}

	/**
	 * Enqueue assets for the builder.
	 *
	 * @since 1.0.0
	 */
	public function enqueues() {

		$min = wpforms_get_min_suffix();

		wp_enqueue_script(
			'wpforms-builder-square',
			WPFORMS_SQUARE_URL . "assets/js/admin/builder-square{$min}.js",
			[ 'wpforms-builder' ],
			WPFORMS_SQUARE_VERSION,
			true
		);

		wp_enqueue_style(
			'wpforms-square-placeholder',
			WPFORMS_SQUARE_URL . "assets/css/wpforms-square-card-placeholder{$min}.css",
			[],
			WPFORMS_SQUARE_VERSION
		);

		/**
		 * Currently, we would like to limit number of the Square Credit Card fields
		 * and allows only one field per form cause there is technical issue with the Web Payments SDK.
		 */
		wp_add_inline_style( 'wpforms-builder', '.wpforms-add-fields-group .wpforms-add-fields-button-disabled:hover { background-color: #036aab; cursor: no-drop; }' );
	}

	/**
	 * Add localized strings to be available in the form builder.
	 *
	 * @since 1.0.0
	 *
	 * @param array $strings Form builder JS strings.
	 *
	 * @return array
	 */
	public function javascript_strings( $strings ) {

		$strings['square_connection_required'] = wp_kses(
			__( '<p>Square account connection is required when using the Square field.</p><p>To proceed, please go to <strong>WPForms Settings » Payments » Square</strong> and press <strong>Connect with Square</strong> button.</p>', 'wpforms-square' ),
			[
				'p'      => [],
				'strong' => [],
			]
		);

		$strings['square_payments_enabled_required'] = wp_kses(
			__( '<p>Square Payments must be enabled when using the Square field.</p><p>To proceed, please go to <strong>Payments » Square</strong> and check <strong>Enable Square Payments</strong>.</p>', 'wpforms-square' ),
			[
				'p'      => [],
				'strong' => [],
			]
		);

		$strings['square_ajax_required'] = wp_kses(
			__( '<p>AJAX form submissions are required when using the Square field.</p><p>To proceed, please go to <strong>Settings » General » Advanced</strong> and check <strong>Enable AJAX form submission</strong>.</p>', 'wpforms-square' ),
			[
				'p'      => [],
				'strong' => [],
			]
		);

		return $strings;
	}

	/**
	 * Add checkbox to form notification settings.
	 *
	 * @since 1.0.0
	 *
	 * @param WPForms_Builder_Panel_Settings $settings WPForms_Builder_Panel_Settings class instance.
	 * @param int                            $id       Subsection ID.
	 */
	public function notification_settings( $settings, $id ) {

		if ( empty( $settings->form_data ) ) {
			return;
		}

		wpforms_panel_field(
			'toggle',
			'notifications',
			'square',
			$settings->form_data,
			esc_html__( 'Enable for Square completed payments', 'wpforms-square' ),
			[
				'parent'      => 'settings',
				'class'       => ! Helpers::is_square_single_enabled( $settings->form_data ) ? 'wpforms-hidden' : '',
				'input_class' => 'wpforms-radio-group wpforms-radio-group-' . $id . '-notification-by-status wpforms-radio-group-item-square wpforms-notification-by-status-alert',
				'subsection'  => $id,
				'tooltip'     => wp_kses(
					__( 'When enabled this notification will <em>only</em> be sent when a Square payment has been successfully <strong>completed</strong>.', 'wpforms-square' ),
					[
						'em'     => [],
						'strong' => [],
					]
				),
				'data'        => [
					'radio-group'    => $id . '-notification-by-status',
					'provider-title' => esc_html__( 'Square completed payments', 'wpforms-square' ),
				],
			]
		);
	}
}
