<?php

namespace WPFormsSquare\Admin;

use WPFormsSquare\Helpers;

/**
 * Square admin entries.
 *
 * @since 1.0.0
 */
class Entries {

	/**
	 * Init the class.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Entries hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_filter( 'wpforms_has_payment_gateway', [ $this, 'has_payment_gateway' ], 10, 2 );
	}

	/**
	 * Make Square payment gateway work on the admin entries page.
	 *
	 * @since 1.0.0
	 *
	 * @param bool  $result    Initial value.
	 * @param array $form_data Form data and settings.
	 *
	 * @return bool
	 */
	public function has_payment_gateway( $result, $form_data ) {

		if ( Helpers::is_square_single_enabled( $form_data ) ) {
			return true;
		}

		return $result;
	}

	/**
	 * Change gateway name in Entry Payment Details metabox.
	 *
	 * @since 1.0.0
	 * @deprecated 1.5.0
	 *
	 * @param string $gateway    Initial gateway name.
	 * @param array  $entry_meta Entry meta data.
	 *
	 * @return string
	 */
	public function entry_details_payment_gateway( $gateway, $entry_meta ) {

		_deprecated_function( __METHOD__, '1.5.0 of the Square addon.' );

		if ( ! $this->is_square_entry_meta_payment_type( $entry_meta ) ) {
			return $gateway;
		}

		return sprintf(
			'<a href="%s" target="_blank" rel="noopener noreferrer">%s</a>',
			esc_url( $this->get_seller_dashboard_url( $entry_meta ) ),
			esc_html__( 'Square', 'wpforms-square' )
		);
	}

	/**
	 * Change transaction ID in Entry Payment Details metabox.
	 *
	 * @since 1.0.0
	 * @deprecated 1.5.0
	 *
	 * @param string $transaction Initial transaction ID.
	 * @param array  $entry_meta  Entry meta data.
	 *
	 * @return string
	 */
	public function entry_details_payment_transaction( $transaction, $entry_meta ) {

		_deprecated_function( __METHOD__, '1.5.0 of the Square addon.' );

		if (
			! $this->is_square_entry_meta_payment_type( $entry_meta ) ||
			empty( $entry_meta['payment_transaction'] )
		) {
			return $transaction;
		}

		return $this->get_transaction_link( $entry_meta );
	}

	/**
	 * Change customer ID in Entry Payment Details metabox.
	 *
	 * @since 1.0.0
	 * @deprecated 1.5.0
	 *
	 * @param string $customer   Initial customer ID.
	 * @param array  $entry_meta Entry meta data.
	 *
	 * @return string
	 */
	public function entry_details_payment_customer( $customer, $entry_meta ) {

		_deprecated_function( __METHOD__, '1.5.0 of the Square addon.' );

		if (
			! $this->is_square_entry_meta_payment_type( $entry_meta ) ||
			empty( $entry_meta['payment_customer'] )
		) {
			return $customer;
		}

		return $this->get_customer_url( $entry_meta );
	}

	/**
	 * Change transaction total in Entry Payment Details metabox.
	 *
	 * @since 1.0.0
	 * @deprecated 1.5.0
	 *
	 * @param string $total      Initial transaction total.
	 * @param array  $entry_meta Entry meta data.
	 *
	 * @return string
	 */
	public function entry_details_payment_total( $total, $entry_meta ) {

		_deprecated_function( __METHOD__, '1.5.0 of the Square addon.' );

		if ( ! $this->is_square_entry_meta_payment_type( $entry_meta ) ) {
			return $total;
		}

		if ( ! empty( $entry_meta['payment_period'] ) ) {
			$total .= ' <span style="font-weight:400; color:#999; display:inline-block; margin-left:4px;"><i class="fa fa-refresh" aria-hidden="true"></i> ' . $entry_meta['payment_period'] . '</span>';
		}

		return $total;
	}

	/**
	 * Extend the Entry Payment Details metabox.
	 *
	 * @since 1.0.0
	 * @deprecated 1.5.0
	 *
	 * @param object $entry     Submitted entry values.
	 * @param array  $form_data Form data and settings.
	 */
	public function extend_entry_payment_details( $entry, $form_data ) {

		_deprecated_function( __METHOD__, '1.5.0 of the Square addon.' );

		$entry_meta = json_decode( $entry->meta, true );

		if ( ! $this->is_square_entry_meta_payment_type( $entry_meta ) ) {
			return;
		}

		if ( ! empty( $entry_meta['receipt_number'] ) ) {
			printf( '<p class="wpforms-entry-payment-receipt-number">%s <strong>%s</strong></p>', esc_html__( 'Receipt Number:', 'wpforms-square' ), wp_kses_post( $entry_meta['receipt_number'] ) );
		}
	}

	/**
	 * Check if Square is set as a payment type inside entry meta.
	 *
	 * @since 1.0.0
	 *
	 * @param array $entry_meta Entry meta data.
	 *
	 * @return bool
	 */
	private function is_square_entry_meta_payment_type( $entry_meta ) {

		return ! empty( $entry_meta['payment_type'] ) && $entry_meta['payment_type'] === 'square';
	}

	/**
	 * Determine if a transaction was performed on the Sandbox environment.
	 *
	 * @since 1.0.0
	 *
	 * @param array $entry_meta Entry data.
	 *
	 * @return bool
	 */
	private function is_sandbox_mode( $entry_meta ) {

		return ! empty( $entry_meta['payment_mode'] ) && $entry_meta['payment_mode'] === 'test';
	}

	/**
	 * Retrieve the seller dashboard URL.
	 *
	 * @since 1.0.0
	 *
	 * @param array $entry_meta Entry data.
	 *
	 * @return string
	 */
	private function get_seller_dashboard_url( $entry_meta ) {

		return $this->is_sandbox_mode( $entry_meta ) ? 'https://squareupsandbox.com/dashboard' : 'https://squareup.com/dashboard';
	}

	/**
	 * Retrieve the transaction link.
	 *
	 * @since 1.0.0
	 *
	 * @param array $entry_meta Entry data.
	 *
	 * @return string
	 */
	private function get_transaction_link( $entry_meta ) {

		$id  = $entry_meta['payment_transaction'];
		$url = $this->get_seller_dashboard_url( $entry_meta ) . '/sales/transactions/' . $id;

		return sprintf(
			'<a href="%1$s" target="_blank" rel="noopener noreferrer">%2$s</a>',
			esc_url( $url ),
			esc_html( $id )
		);
	}

	/**
	 * Retrieve the customer dashboard URL.
	 *
	 * @since 1.0.0
	 *
	 * @param array $entry_meta Entry data.
	 *
	 * @return string
	 */
	private function get_customer_url( $entry_meta ) {

		$id  = $entry_meta['payment_customer'];
		$url = $this->get_seller_dashboard_url( $entry_meta ) . '/customers/directory/customer/' . $id;

		return sprintf(
			'<a href="%1$s" target="_blank" rel="noopener noreferrer">%2$s</a>',
			esc_url( $url ),
			esc_html( $entry_meta['payment_customer'] )
		);
	}
}
