<?php

namespace WPFormsSquare\Admin;

use Square\Environment;
use WPForms\Admin\Notice;
use WPFormsSquare\Connection;
use WPFormsSquare\Helpers;

/**
 * Square addon settings.
 *
 * @since 1.0.0
 */
class Settings {

	/**
	 * Determine if Square account is connected.
	 *
	 * @since 1.0.0
	 *
	 * @var bool
	 */
	private $is_connected;

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 *
	 * @return Settings
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Settings hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_settings_enqueue',  [ $this, 'enqueue_scripts' ] );
		add_filter( 'wpforms_admin_strings',     [ $this, 'javascript_strings' ] );

		add_filter( 'wpforms_settings_defaults', [ $this, 'register' ], 12 );
		add_action( 'wpforms_settings_init',     [ $this, 'display_notice' ] );
		add_action( 'wpforms_settings_updated',  [ $this, 'reset_transients' ] );
	}

	/**
	 * Enqueue Settings scripts.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_scripts() {

		$min = wpforms_get_min_suffix();

		wp_enqueue_script(
			'wpforms-admin-settings-square',
			WPFORMS_SQUARE_URL . "assets/js/admin/settings-square{$min}.js",
			[ 'jquery' ],
			WPFORMS_SQUARE_VERSION,
			true
		);
	}

	/**
	 * Localize needed strings.
	 *
	 * @since 1.0.0
	 *
	 * @param array $strings JS strings.
	 *
	 * @return array
	 */
	public function javascript_strings( $strings ) {

		$strings['square'] = [
			'mode_update'   => wp_kses(
				__(
					'<p>Switching sandbox/production modes requires Square account reconnection.</p><p>Press the <em>"Connect with Square"</em> button after saving the settings to reconnect.</p>',
					'wpforms-square'
				),
				[
					'p'  => [],
					'em' => [],
				]
			),
			'refresh_error' => esc_html__( 'Something went wrong while performing a refresh tokens request.', 'wpforms-square' ),
		];

		return $strings;
	}

	/**
	 * Register Settings fields.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings Array of current form settings.
	 *
	 * @return array
	 */
	public function register( $settings ) {

		$settings['payments']['square-heading'] = [
			'id'       => 'square-heading',
			'content'  => $this->get_heading_content(),
			'type'     => 'content',
			'no_label' => true,
			'class'    => [ 'section-heading' ],
		];

		foreach ( Helpers::get_available_modes() as $mode ) {

			$this->is_connected = false;

			$settings['payments'][ 'square-connection-status-' . $mode ] = [
				'id'        => 'square-connection-status-' . $mode,
				'name'      => esc_html__( 'Connection Status', 'wpforms-square' ),
				'content'   => $this->get_connection_status_content( $mode ),
				'type'      => 'content',
				'is_hidden' => Helpers::get_mode() !== $mode,
			];

			if ( $this->is_connected ) {
				$settings['payments'][ 'square-location-id-' . $mode ] = [
					'id'        => 'square-location-id-' . $mode,
					'name'      => esc_html__( 'Business Location', 'wpforms-square' ),
					'desc'      => esc_html__( 'Only active locations that support credit card processing in Square can be chosen.', 'wpforms-square' ),
					'type'      => 'select',
					'choicesjs' => true,
					'options'   => $this->get_location_options( $mode ),
					'is_hidden' => Helpers::get_mode() !== $mode,
				];
			}
		}

		$settings['payments']['square-sandbox-mode'] = [
			'id'     => 'square-sandbox-mode',
			'name'   => esc_html__( 'Test Mode', 'wpforms-square' ),
			'desc'   => sprintf(
				wp_kses( /* translators: %s - WPForms.com URL for Square payment with more details. */
					__( 'Prevent Square from processing live transactions. <a href="%s" target="_blank" rel="noopener noreferrer" class="wpforms-learn-more">Learn More</a>', 'wpforms-square' ),
					[
						'a' => [
							'href'   => [],
							'target' => [],
							'rel'    => [],
							'class'  => [],
						],
					]
				),
				esc_url( wpforms_utm_link( 'https://wpforms.com/docs/how-to-test-square-payments-on-your-site/', 'Settings - Payments', 'Square Test Documentation' ) )
			),
			'type'   => 'toggle',
			'status' => true,
		];

		return $settings;
	}

	/**
	 * Reset transients on settings save.
	 *
	 * @since 1.0.0
	 */
	public function reset_transients() {

		array_map( '\WPFormsSquare\Helpers::detete_transients', Helpers::get_available_modes() );
	}

	/**
	 * Display admin error notice if something wrong with the Square settings.
	 *
	 * @since 1.0.0
	 */
	public function display_notice() {

		$connection = Connection::get();

		if ( ! $connection ) {
			return;
		}

		$this->maybe_display_connection_notice( $connection );
	}

	/**
	 * Display admin error notice if a connection exists, but is not ready to use.
	 *
	 * @since 1.0.0
	 *
	 * @param Connection $connection Connection data.
	 */
	private function maybe_display_connection_notice( $connection ) {

		if ( ! $connection->is_configured() ) {
			Notice::error( esc_html__( 'Heads up! Square account connection is missing required data. You must reconnect your Square account.', 'wpforms-square' ) );

			return;
		}

		if ( ! $connection->is_valid() ) {
			Notice::error( esc_html__( 'Heads up! Square account connection is invalid. You must reconnect your Square account.', 'wpforms-square' ) );

			return;
		}

		if ( $connection->is_expired() ) {
			Notice::error( esc_html__( 'Heads up! Square account connection is expired. Tokens must be refreshed.', 'wpforms-square' ) );

			return;
		}

		if ( empty( Helpers::get_location_id() ) ) {
			Notice::error( esc_html__( 'Heads up! No Square business location was chosen.', 'wpforms-square' ) );

			return;
		}

		if ( ! $connection->is_currency_matched() ) {
			Notice::error(
				sprintf(
					wp_kses( /* translators: %1$s - WPForms currency; %2$s - Location currency. */
						__( 'Heads up! WPForms Settings » Payments » Currency is <strong>%1$s</strong> but your configured Square business location currency is <strong>%2$s</strong>, so payments cannot be processed. Please choose a different business location or change WPForms currency.', 'wpforms-square' ),
						[
							'strong' => [],
						]
					),
					esc_html( wpforms_get_currency() ),
					esc_html( $connection->get_currency() )
				)
			);

			return;
		}
	}

	/**
	 * Retrieve a section header content.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_heading_content() {

		return '<h4>' . esc_html__( 'Square', 'wpforms-square' ) . '</h4><p>' .
			sprintf(
				wp_kses( /* translators: %s - WPForms.com Square documentation article URL. */
					__( 'Easily collect credit card payments with Square. For getting started and more information, see our <a href="%s" target="_blank" rel="noopener noreferrer">Square documentation</a>.', 'wpforms-square' ),
					[
						'a' => [
							'href'   => [],
							'target' => [],
							'rel'    => [],
						],
					]
				),
				esc_url( wpforms_utm_link( 'https://wpforms.com/docs/how-to-install-and-use-the-square-addon-with-wpforms/', 'Settings - Payments', 'Square Documentation' ) )
			) .
			'</p>';
	}

	/**
	 * Retrieve a Connection Status setting content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_connection_status_content( $mode ) {

		$this->is_connected = false;
		$connection         = Connection::get( $mode );

		if ( ! $connection ) {
			return $this->get_disconnected_status_content( $mode );
		}

		$content = $this->get_disabled_status_content( $connection );

		if ( ! empty( $content ) ) {
			return $content;
		}

		$this->is_connected = true;

		return $this->get_enabled_status_content( $connection );
	}

	/**
	 * Retrieve setting content when a connection is disabled.
	 *
	 * @since 1.0.0
	 *
	 * @param Connection $connection Connection data.
	 *
	 * @return string
	 */
	private function get_disabled_status_content( $connection ) {

		if ( ! $connection->is_configured() ) {
			return $this->get_missing_status_content( $connection->get_mode() );
		}

		if ( ! $connection->is_valid() ) {
			return $this->get_invalid_status_content( $connection->get_mode() );
		}

		return '';
	}

	/**
	 * Retrieve setting content when a connection is enabled.
	 *
	 * @since 1.0.0
	 *
	 * @param Connection $connection Connection data.
	 *
	 * @return string
	 */
	private function get_enabled_status_content( $connection ) {

		if ( $connection->is_expired() ) {
			return $this->get_expired_status_content( $connection->get_mode() );
		}

		if ( empty( Helpers::get_location_id( $connection->get_mode() ) ) ) {
			return $this->get_no_locations_status_content( $connection->get_mode() );
		}

		if ( ! $connection->is_currency_matched() ) {
			return $this->get_currency_mismatch_status_content( $connection->get_mode() );
		}

		return '<div class="wpforms-square-connected"><span class="wpforms-success-icon"></span>' . $this->get_connected_status_content( $connection->get_mode() ) . $this->get_disconnect_button( $connection->get_mode() ) . '</div>';
	}

	/**
	 * Retrieve a Connected Status setting content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_connected_status_content( $mode ) {

		$account_data = wpforms_square()->get_connect()->get_connected_account( $mode );
		$account_name = empty( $account_data['business_name'] ) ? '' : $account_data['business_name'];

		if ( empty( $account_name ) ) {
			return sprintf(
				wp_kses( /* translators: %s - Square mode. */
					__( 'Connected to Square in <strong>%s</strong> mode.', 'wpforms-square' ),
					[
						'strong' => [],
					]
				),
				$mode === Environment::SANDBOX ? esc_html__( 'Sandbox', 'wpforms-square' ) : esc_html__( 'Production', 'wpforms-square' )
			);
		}

		return sprintf(
			wp_kses( /* translators: %1$s - Square connected account name; %2$s - Square mode. */
				__( 'Connected to Square as <em>%1$s</em> in <strong>%2$s</strong> mode.', 'wpforms-square' ),
				[
					'strong' => [],
					'em'     => [],
				]
			),
			esc_html( $account_name ),
			$mode === Environment::SANDBOX ? esc_html__( 'Sandbox', 'wpforms-square' ) : esc_html__( 'Production', 'wpforms-square' )
		);
	}

	/**
	 * Retrieve a Disconnected Status setting content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_disconnected_status_content( $mode ) {

		return $this->get_connect_button( $mode, false ) .
			'<p class="desc">' .
			sprintf(
				wp_kses( /* translators: %s - WPForms.com Square documentation article URL. */
					__( 'Securely connect to Square with just a few clicks to begin accepting payments! <a href="%s" target="_blank" rel="noopener noreferrer" class="wpforms-learn-more">Learn More</a>', 'wpforms-square' ),
					[
						'a' => [
							'href'   => [],
							'target' => [],
							'rel'    => [],
							'class'  => [],
						],
					]
				),
				esc_url( wpforms_utm_link( 'https://wpforms.com/docs/how-to-install-and-use-the-square-addon-with-wpforms/#connect-square', 'Settings - Payments', 'Square Learn More' ) )
			) .
			'</p>';
	}

	/**
	 * Retrieve a connection is missing status content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_missing_status_content( $mode ) {

		return '<div class="wpforms-square-connected">' . $this->get_warning_icon() . esc_html__( 'Your connection is missing required data. You must reconnect your Square account.', 'wpforms-square' ) . $this->get_disconnect_button( $mode ) . '</div>';
	}

	/**
	 * Retrieve a connection invalid status content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_invalid_status_content( $mode ) {

		return '<div class="wpforms-square-connected">' . $this->get_warning_icon() . $this->get_connected_status_content( $mode ) .
			'<p>' . esc_html__( 'It appears your connection may be invalid. You must refresh tokens or reconnect your account.', 'wpforms-square' ) . '</p>' .
			'<p>' . $this->get_refresh_button( $mode ) . $this->get_disconnect_button( $mode, false ) . '</p></div>';
	}

	/**
	 * Retrieve a connection expired status content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_expired_status_content( $mode ) {

		return '<div class="wpforms-square-connected">' . $this->get_warning_icon() . $this->get_connected_status_content( $mode ) .
			'<p>' . esc_html__( 'Your connection is expired. You must refresh tokens or reconnect your account.', 'wpforms-square' ) . '</p>' .
			'<p>' . $this->get_refresh_button( $mode ) . $this->get_disconnect_button( $mode, false ) . '</p></div>';
	}

	/**
	 * Retrieve no locations status content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_no_locations_status_content( $mode ) {

		return '<div class="wpforms-square-connected">' . $this->get_warning_icon() . $this->get_connected_status_content( $mode ) .
			'<p>' . esc_html__( 'No Business Location was chosen.', 'wpforms-square' ) . '</p></div>' .
			$this->get_disconnect_button( $mode );
	}

	/**
	 * Retrieve a currency mismatch status content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_currency_mismatch_status_content( $mode ) {

		return '<div class="wpforms-square-connected">' . $this->get_warning_icon() . $this->get_connected_status_content( $mode ) .
			'<p>' . esc_html__( 'WPForms currency and Business Location currency are not matched.', 'wpforms-square' ) . '</p></div>' .
			$this->get_disconnect_button( $mode );
	}

	/**
	 * Retrieve the Connect button.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 * @param bool   $wrap Optional. Wrap a button HTML element or not.
	 *
	 * @return string
	 */
	private function get_connect_button( $mode, $wrap = true ) {

		$button = sprintf(
			'<a class="wpforms-btn wpforms-btn-md wpforms-btn-light-grey" href="%1$s" title="%2$s">%3$s</a>',
			esc_url( wpforms_square()->get_connect()->get_connect_url( $mode ) ),
			esc_attr__( 'Connect Square account', 'wpforms-square' ),
			esc_html__( 'Connect with Square', 'wpforms-square' )
		);

		return $wrap ? '<p>' . $button . '</p>' : $button;
	}

	/**
	 * Retrieve the Disconnect button.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 * @param bool   $wrap Optional. Wrap a button HTML element or not.
	 *
	 * @return string
	 */
	private function get_disconnect_button( $mode, $wrap = true ) {

		$button = sprintf(
			'<a class="wpforms-btn wpforms-btn-md wpforms-btn-light-grey" href="%1$s" title="%2$s">%3$s</a>',
			esc_url( wpforms_square()->get_connect()->get_disconnect_url( $mode ) ),
			esc_attr__( 'Disconnect Square account', 'wpforms-square' ),
			esc_html__( 'Disconnect', 'wpforms-square' )
		);

		return $wrap ? '<p>' . $button . '</p>' : $button;
	}

	/**
	 * Retrieve the Refresh tokens button.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	private function get_refresh_button( $mode ) {

		return sprintf(
			'<button class="wpforms-btn wpforms-btn-md wpforms-btn-light-grey wpforms-square-refresh-btn" type="button" data-mode="%1$s" data-url="%2$s" title="%3$s">%4$s</button>',
			esc_attr( $mode ),
			esc_url( Helpers::get_settings_page_url() ),
			esc_attr__( 'Refresh connection tokens', 'wpforms-square' ),
			esc_html__( 'Refresh tokens', 'wpforms-square' )
		);
	}

	/**
	 * Retrieve the Warning icon emoji.
	 *
	 * @since 1.0.0
	 * @since 1.8.0 Removed emoji and added by CSS styles.
	 *
	 * @return string
	 */
	private function get_warning_icon() {

		return '<span class="wpforms-warning-icon"></span>';
	}

	/**
	 * Retrieve Business Location options.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return array
	 */
	private function get_location_options( $mode ) {

		$locations = wpforms_square()->get_connect()->get_connected_locations( $mode );

		return is_array( $locations ) && ! empty( $locations ) ? array_column( $locations, 'name', 'id' ) : [ '' => esc_html__( 'No locations were found', 'wpforms-square' ) ];
	}
}
