<?php

namespace WPFormsSquare\Admin;

use WPForms_Payment;
use WPFormsSquare\Connection;
use WPFormsSquare\Helpers;

/**
 * Square Form Builder payment registration.
 *
 * @since 1.0.0
 */
class SquarePayment extends WPForms_Payment {

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->version  = WPFORMS_SQUARE_VERSION;
		$this->name     = 'Square';
		$this->slug     = 'square';
		$this->priority = 12;
		$this->icon     = WPFORMS_SQUARE_URL . 'assets/images/addon-icon-square.png';
	}

	/**
	 * Display content inside the panel content area.
	 *
	 * @since 1.0.0
	 */
	public function builder_content() {

		if ( ! $this->check_connection() ) {
			return;
		}

		$has_square_field = Helpers::has_square_field( $this->form_data );

		$this->single_payment_content( $has_square_field );
	}

	/**
	 * Display Single payment content inside the panel content area.
	 *
	 * @since 1.0.0
	 *
	 * @param bool $has_square_field True if Square field is in the form.
	 */
	private function single_payment_content( $has_square_field ) {

		$alert_css_classes     = $has_square_field ? 'wpforms-alert wpforms-alert-info wpforms-hidden' : 'wpforms-alert wpforms-alert-info';
		$single_enable_classes = $has_square_field ? 'wpforms-square-panel-field' : 'wpforms-square-panel-field wpforms-hidden';
		$single_fields_classes = $has_square_field && Helpers::is_square_single_enabled( $this->form_data ) ? 'wpforms-square-panel-fields' : 'wpforms-square-panel-fields wpforms-hidden';

		printf( '<p class="%s" id="square-credit-card-alert">%s</p>', esc_attr( $alert_css_classes ), esc_html__( 'To use Square payments you need to add a Square field to the form', 'wpforms-square' ) );

		wpforms_panel_field(
			'toggle',
			'square',
			'enable',
			$this->form_data,
			esc_html__( 'Enable Square payments', 'wpforms-square' ),
			[
				'parent'  => 'payments',
				'default' => '0',
				'class'   => $single_enable_classes,
			]
		);

		printf( '<div class="%s">', esc_attr( $single_fields_classes ) );

			wpforms_panel_field(
				'text',
				'square',
				'payment_description',
				$this->form_data,
				esc_html__( 'Payment Description', 'wpforms-square' ),
				[
					'parent'  => 'payments',
					'tooltip' => esc_html__( 'Enter your payment description. Eg: Donation for the soccer team.', 'wpforms-square' ),
				]
			);

			wpforms_panel_field(
				'select',
				'square',
				'buyer_email',
				$this->form_data,
				esc_html__( 'Buyer Email', 'wpforms-square' ),
				[
					'parent'      => 'payments',
					'field_map'   => [ 'email' ],
					'placeholder' => esc_html__( '--- Select Buyer Email ---', 'wpforms-square' ),
					'tooltip'     => esc_html__( "Select the field that contains the buyer's email address. This field is optional.", 'wpforms-square' ),
				]
			);

			wpforms_panel_field(
				'select',
				'square',
				'billing_name',
				$this->form_data,
				esc_html__( 'Billing Name', 'wpforms-square' ),
				[
					'parent'      => 'payments',
					'field_map'   => [ 'name' ],
					'placeholder' => esc_html__( '--- Select Billing Name ---', 'wpforms-square' ),
					'tooltip'     => esc_html__( "Select the field that contains the billing's name. This field is optional.", 'wpforms-square' ),
				]
			);

			wpforms_panel_field(
				'select',
				'square',
				'billing_address',
				$this->form_data,
				esc_html__( 'Billing Address', 'wpforms-square' ),
				[
					'parent'      => 'payments',
					'field_map'   => [ 'address' ],
					'placeholder' => esc_html__( '--- Select Billing Address ---', 'wpforms-square' ),
					'tooltip'     => esc_html__( "Select the field that contains the billing's address. This field is optional.", 'wpforms-square' ),
				]
			);

			wpforms_conditional_logic()->builder_block(
				[
					'form'        => $this->form_data,
					'type'        => 'panel',
					'panel'       => 'square',
					'parent'      => 'payments',
					'actions'     => [
						'go'   => esc_html__( 'Process', 'wpforms-square' ),
						'stop' => esc_html__( "Don't process", 'wpforms-square' ),
					],
					'action_desc' => esc_html__( 'this charge if', 'wpforms-square' ),
					'reference'   => esc_html__( 'Square payment', 'wpforms-square' ),
				]
			);

		echo '</div>';
	}

	/**
	 * Check if connection is exist and ready to use.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function check_connection() {

		$connection = Connection::get();

		if ( ! $connection ) {
			echo '<p class="wpforms-alert wpforms-alert-info">';
			printf(
				wp_kses( /* translators: %s - Admin area Payments settings page URL. */
					__( "Heads up! Square payments can't be enabled yet. First, please connect to your Square account on the <a href='%s'>WPForms Settings</a> page.", 'wpforms-square' ),
					[
						'a' => [
							'href' => [],
						],
					]
				),
				esc_url( Helpers::get_settings_page_url() . '#wpforms-setting-row-square-heading' )
			);
			echo '</p>';

			return false;
		}

		if ( ! $connection->is_usable() ) {
			echo '<p class="wpforms-alert wpforms-alert-info">';
			printf(
				wp_kses( /* translators: %s - the WPForms Payments settings page URL. */
					__( "Heads up! Square payments can't be processed because of something wrong with account connection. First, please resolve the connection issue on the <a href='%2\$s'>WPForms Settings</a> page.", 'wpforms-square' ),
					[
						'a' => [
							'href' => [],
						],
					]
				),
				Helpers::is_sandbox_mode() ? esc_html__( 'Sandbox', 'wpforms-square' ) : esc_html__( 'Production', 'wpforms-square' ),
				esc_url( Helpers::get_settings_page_url() . '#wpforms-setting-row-square-heading' )
			);
			echo '</p>';

			return false;
		}

		if ( $connection->is_expired() ) {
			echo '<p class="wpforms-alert wpforms-alert-warning">';
			printf(
				wp_kses( /* translators: %s - the WPForms Payments settings page URL. */
					__( "Heads up! Square account connection is expired. Tokens must be refreshed. First, please refresh them on the <a href='%2\$s'>WPForms Settings</a> page.", 'wpforms-square' ),
					[
						'a' => [
							'href' => [],
						],
					]
				),
				Helpers::is_sandbox_mode() ? esc_html__( 'Sandbox', 'wpforms-square' ) : esc_html__( 'Production', 'wpforms-square' ),
				esc_url( Helpers::get_settings_page_url() . '#wpforms-setting-row-square-heading' )
			);
			echo '</p>';

			return true;
		}

		return true;
	}
}
