<?php

namespace WPFormsSquare\Api;

use Square\Exceptions\ApiException;
use Square\Http\ApiResponse;
use Square\Models\Address;
use Square\Models\CreateOrderRequest;
use Square\Models\CreatePaymentRequest;
use Square\Models\CreatePaymentResponse;
use Square\Models\Location;
use Square\Models\Merchant;
use Square\Models\Money;
use Square\Models\Order;
use Square\Models\OrderLineItem;
use Square\Models\OrderLineItemDiscount;
use Square\Models\OrderSource;
use Square\Models\OrderState;
use Square\Models\Payment;
use Square\Models\UpdateOrderRequest;
use Square\SquareClient;
use WPFormsSquare\Connection;
use WPFormsSquare\Plugin;

/**
 * WPForms Square API class.
 *
 * @since 1.0.0
 */
class Api {

	/**
	 * Square API client instance.
	 *
	 * @since 1.0.0
	 *
	 * @var SquareClient
	 */
	private $client;

	/**
	 * Payment token (card nonce) generated by the Web Payments SDK.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private $source_id;

	/**
	 * Identifying tokens generated by the Web Payments SDK.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $verify_tokens;

	/**
	 * Last API call response.
	 *
	 * @since 1.0.0
	 *
	 * @var ApiResponse
	 */
	private $response;

	/**
	 * Last API call exception.
	 *
	 * @since 1.0.0
	 *
	 * @var ApiException
	 */
	private $exception;

	/**
	 * API errors.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $errors;

	/**
	 * Constructs the main Square API wrapper class.
	 *
	 * @since 1.0.0
	 *
	 * @param Connection $connection Connection object.
	 */
	public function __construct( $connection ) {

		$this->client = new SquareClient(
			[
				'accessToken' => $connection->get_access_token(),
				'environment' => $connection->get_mode(),
			]
		);
	}

	/**
	 * Set tokens from a submitted form data.
	 *
	 * @since 1.0.0
	 *
	 * @param array $entry Copy of original $_POST.
	 */
	public function set_payment_tokens( $entry ) {

		if ( ! empty( $entry['square']['source_id'] ) ) {
			$this->source_id = $entry['square']['source_id'];
		}

		if ( ! empty( $entry['square']['verify']['store'] ) ) {
			$this->verify_tokens['store'] = $entry['square']['verify']['store'];
		}

		if ( ! empty( $entry['square']['verify']['charge'] ) ) {
			$this->verify_tokens['charge'] = $entry['square']['verify']['charge'];
		}
	}

	/**
	 * Check if OAuth connection is present and ready to use.
	 *
	 * @since 1.0.0
	 */
	private function check_connection() {

		$connection = Connection::get();

		if ( ! $connection || ! $connection->is_configured() ) {
			$this->errors[] = esc_html__( 'Square account connection is missing.', 'wpforms-square' );

			return;
		}

		if ( ! $connection->is_valid() ) {
			$this->errors[] = esc_html__( 'Square account connection is invalid.', 'wpforms-square' );

			return;
		}

		if ( ! $connection->is_currency_matched() ) {
			$this->errors[] = esc_html__( 'The currency associated with the payment is not valid for the provided business location.', 'wpforms-square' );

			return;
		}
	}

	/**
	 * Check if single payment tokens are present.
	 *
	 * @since 1.0.0
	 */
	private function check_single_payment_tokens() {

		if ( empty( $this->source_id ) || empty( $this->verify_tokens['charge'] ) ) {
			$this->errors[] = esc_html__( 'Square payment stopped, missing card tokens.', 'wpforms-square' );
		}
	}

	/**
	 * Check if all required general arguments are present.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args Arguments to check.
	 */
	private function check_required_args_general( $args ) {

		if ( empty( $args['location_id'] ) ) {
			$this->errors[] = esc_html__( 'Missing location ID.', 'wpforms-square' );
		}

		if ( empty( $args['currency'] ) ) {
			$this->errors[] = esc_html__( 'Missing currency.', 'wpforms-square' );
		}

		if ( empty( $args['amount'] ) && ! is_numeric( $args['amount'] ) ) {
			$this->errors[] = esc_html__( 'Missing amount.', 'wpforms-square' );
		}
	}

	/**
	 * Check if all required single payment arguments are present.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args Arguments to check.
	 */
	private function check_required_args_single( $args ) {

		if ( empty( $args['order_items'] ) ) {
			$this->errors[] = esc_html__( 'Missing order/payment items.', 'wpforms-square' );
		}
	}

	/**
	 * Process single transaction.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args Payment arguments.
	 */
	public function process_single_transaction( $args ) {

		$this->check_connection();
		$this->check_single_payment_tokens();
		$this->check_required_args_general( $args );
		$this->check_required_args_single( $args );

		if ( $this->errors ) {
			return;
		}

		$result = $this->perform_single_transaction( $args );

		/**
		 * Fire when a single transaction is performed.
		 *
		 * @since 1.0.0
		 *
		 * @param array $result Single transaction result.
		 * @param array $args   Payment arguments.
		 * @param Api   $api    Api class instance.
		 */
		do_action( 'wpforms_square_api_process_single_transaction_after', $result, $args, $this );
	}

	/**
	 * Perform a single transaction.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args Payment arguments.
	 *
	 * @return array
	 */
	private function perform_single_transaction( $args ) {

		$result = [];

		// Create an order.
		$order_request = $this->prepare_create_order_request( $args );
		$order         = $this->send_create_order_request( $order_request );

		if ( ! $order instanceof Order ) {
			return $result;
		}

		$result['order'] = $order;

		// Create a payment.
		$payment_request = $this->prepare_create_payment_request( $order, $args );
		$payment         = $this->send_create_payment_request( $payment_request );

		// In this case we should cancel an order.
		if ( ! $payment instanceof Payment ) {
			$update_order_request = $this->prepare_update_order_request( $order );
			$updated_order        = $this->send_update_order_request( $order->getId(), $update_order_request );

			return $updated_order instanceof Order ? [ 'order' => $updated_order ] : $result;
		}

		$result['payment'] = $payment;

		return $result;
	}

	/**
	 * Prepare a create order request object for sending to API.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args Single payment arguments.
	 *
	 * @return CreateOrderRequest
	 */
	private function prepare_create_order_request( $args ) {

		$request = $this->get_create_order_request_object( $args );
		$request = $this->create_order_request_set_line_items( $request, $args );
		$request = $this->create_order_request_set_discounts( $request, $args );

		/**
		 * Filter a create order request object.
		 *
		 * @since 1.0.0
		 *
		 * @param CreateOrderRequest $request Create order request object.
		 * @param array              $args    Payment arguments.
		 * @param Api                $api     Api class instance.
		 */
		return apply_filters( 'wpforms_square_api_prepare_create_order_request', $request, $args, $this );
	}

	/**
	 * Prepare an update order request object for sending to API.
	 *
	 * @since 1.0.0
	 *
	 * @param Order $order Order object.
	 *
	 * @return UpdateOrderRequest
	 */
	private function prepare_update_order_request( $order ) {

		$request = $this->get_update_order_request_object( $order );

		/**
		 * Filter an update order request object.
		 *
		 * @since 1.0.0
		 *
		 * @param UpdateOrderRequest $request Update order request object.
		 * @param Order              $order   Order object.
		 * @param Api                $api     Api class instance.
		 */
		return apply_filters( 'wpforms_square_api_prepare_update_order_request', $request, $order, $this );
	}

	/**
	 * Prepare a create payment request object for sending to API.
	 *
	 * @since 1.0.0
	 *
	 * @param Order $order Order object.
	 * @param array $args  Payment arguments.
	 *
	 * @return CreatePaymentRequest
	 */
	private function prepare_create_payment_request( $order, $args ) {

		$request = $this->get_create_payment_request_object();
		$request = $this->create_payment_request_set_order( $request, $order );

		if ( ! empty( $args['billing'] ) ) {
			$address = $this->get_address_object( $args['billing'] );

			$request->setBillingAddress( $address );
		}

		if ( ! empty( $args['buyer_email'] ) ) {
			$request->setBuyerEmailAddress( $args['buyer_email'] );
		}

		if ( ! empty( $args['note'] ) ) {
			$request->setNote( $args['note'] );
		}

		/**
		 * Filter a create payment request object.
		 *
		 * @since 1.0.0
		 *
		 * @param CreateOrderRequest $request Create payment request object.
		 * @param array              $args    Payment arguments.
		 * @param Api                $api     Api class instance.
		 */
		return apply_filters( 'wpforms_square_api_prepare_create_payment_request', $request, $args, $this );
	}

	/**
	 * Retrieve a create order request object.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args Payment arguments.
	 *
	 * @return CreateOrderRequest
	 */
	private function get_create_order_request_object( $args ) {

		$request = new CreateOrderRequest();

		$request->setIdempotencyKey( $this->get_idempotency_key() );

		$request->setOrder( new Order( $args['location_id'] ) );
		$request->getOrder()->setSource( new OrderSource() );
		$request->getOrder()->getSource()->setName( Plugin::APP_NAME );

		return $request;
	}

	/**
	 * Retrieve an update order request object.
	 *
	 * @since 1.0.0
	 *
	 * @param Order $order Order object.
	 *
	 * @return UpdateOrderRequest
	 */
	private function get_update_order_request_object( $order ) {

		$request = new UpdateOrderRequest();

		$request->setIdempotencyKey( $this->get_idempotency_key() );
		$request->setOrder( $order );
		$request->getOrder()->setState( OrderState::CANCELED );

		return $request;
	}

	/**
	 * Retrieve a create payment request object.
	 *
	 * @since 1.0.0
	 *
	 * @return CreatePaymentRequest
	 */
	private function get_create_payment_request_object() {

		$request = new CreatePaymentRequest( $this->source_id, $this->get_idempotency_key(), new Money() );

		$request->setVerificationToken( $this->verify_tokens['charge'] );

		return $request;
	}

	/**
	 * Send a create order request object to API.
	 *
	 * @since 1.0.0
	 *
	 * @param CreateOrderRequest $request Create order request object.
	 *
	 * @return Order|null
	 */
	private function send_create_order_request( $request ) {

		try {
			$this->response = $this->client->getOrdersApi()->createOrder( $request );

			if ( ! $this->response->isSuccess() ) {
				$this->errors[] = esc_html__( 'Square fail: order was not created.', 'wpforms-square' );

				return null;
			}

			return $this->response->getResult()->getOrder();

		} catch ( ApiException $e ) {
			$this->exception = $e;
			$this->errors[]  = esc_html__( 'Square fail: order was not created.', 'wpforms-square' );

			return null;
		}
	}

	/**
	 * Send an update order request object to API.
	 *
	 * @since 1.0.0
	 *
	 * @param string             $order_id The ID of the order to update.
	 * @param UpdateOrderRequest $request  Update order request object.
	 *
	 * @return Order|null
	 */
	private function send_update_order_request( $order_id, $request ) {

		try {
			$response = $this->client->getOrdersApi()->updateOrder( $order_id, $request );

			if ( ! $response->isSuccess() ) {
				return null;
			}

			return $response->getResult()->getOrder();

		} catch ( ApiException $e ) {
			return null;
		}
	}

	/**
	 * Send a create payment request to API.
	 *
	 * @since 1.0.0
	 *
	 * @param CreatePaymentRequest $request Create payment request object.
	 *
	 * @return Payment|null
	 */
	private function send_create_payment_request( $request ) {

		try {
			$this->response = $this->client->getPaymentsApi()->createPayment( $request );

			if ( ! $this->response->isSuccess() ) {
				$this->errors[] = esc_html__( 'Square fail: payment was not processed.', 'wpforms-square' );

				return null;
			}

			return $this->response->getResult()->getPayment();

		} catch ( ApiException $e ) {
			$this->exception = $e;
			$this->errors[]  = esc_html__( 'Square fail: payment was not processed.', 'wpforms-square' );

			return null;
		}
	}

	/**
	 * Set line items to a create order request object.
	 *
	 * @since 1.0.0
	 *
	 * @param CreateOrderRequest $request Request to create an order.
	 * @param array              $args    Payment arguments.
	 *
	 * @return CreateOrderRequest
	 */
	private function create_order_request_set_line_items( $request, $args ) {

		$line_items = [];

		foreach ( (array) $args['order_items'] as $item ) {

			// Order item without variations.
			if ( empty( $item['variations'] ) ) {
				$line_items[] = $this->get_order_line_item_object( $item, $args );

				continue;
			}

			// Order item with variations (e.g. Small, Medium and Large).
			foreach ( (array) $item['variations'] as $variation ) {

				$order_line_item = $this->get_order_line_item_object( $variation, $args );

				$order_line_item->setVariationName( $variation['variation_name'] );

				$line_items[] = $order_line_item;
			}
		}

		$request->getOrder()->setLineItems( $line_items );

		return $request;
	}

	/**
	 * Set discounts to a create order request object.
	 *
	 * @since 1.6.0
	 *
	 * @param CreateOrderRequest $request Request to create an order.
	 * @param array              $args    Payment arguments.
	 *
	 * @return CreateOrderRequest
	 */
	private function create_order_request_set_discounts( $request, $args ) {

		$discounts = [];

		if ( empty( $args['discounts'] ) ) {
			return $request;
		}

		foreach ( (array) $args['discounts'] as $discount ) {

			$discounts[] = $this->get_order_discount_object( $discount, $args );
		}

		if ( ! empty( $discounts ) ) {
			$request->getOrder()->setDiscounts( $discounts );
		}

		return $request;
	}

	/**
	 * Set order to a create payment request object.
	 *
	 * @since 1.0.0
	 *
	 * @param CreatePaymentRequest $request Create payment request object.
	 * @param Order                $order   Order object.
	 *
	 * @return CreatePaymentRequest
	 */
	private function create_payment_request_set_order( $request, $order ) {

		$request->setOrderId( $order->getId() );
		$request->setLocationId( $order->getLocationId() );

		$request->getAmountMoney()->setAmount( $order->getTotalMoney()->getAmount() );
		$request->getAmountMoney()->setCurrency( $order->getTotalMoney()->getCurrency() );

		return $request;
	}

	/**
	 * Retrieve a single order line item object.
	 *
	 * @since 1.0.0
	 *
	 * @param array $item Order item data.
	 * @param array $args Payment arguments.
	 *
	 * @return OrderLineItem
	 */
	private function get_order_line_item_object( $item, $args ) {

		$order_line_items = new OrderLineItem( '1' );

		$order_line_items->setName( $item['name'] );
		$order_line_items->setBasePriceMoney( new Money() );
		$order_line_items->getBasePriceMoney()->setAmount( $item['amount'] );
		$order_line_items->getBasePriceMoney()->setCurrency( $args['currency'] );

		return $order_line_items;
	}

	/**
	 * Retrieve a single order discount object.
	 *
	 * @since 1.6.0
	 *
	 * @param array $discount Discount data.
	 * @param array $args     Payment arguments.
	 *
	 * @return OrderLineItemDiscount
	 */
	private function get_order_discount_object( $discount, $args ) {

		$order_discounts = new OrderLineItemDiscount();

		$order_discounts->setName( $discount['name'] );
		$order_discounts->setAmountMoney( new Money() );
		$order_discounts->getAmountMoney()->setAmount( $discount['amount'] );
		$order_discounts->getAmountMoney()->setCurrency( $args['currency'] );

		return $order_discounts;
	}

	/**
	 * Prepare and retrieve an address object.
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Address data.
	 *
	 * @return Address
	 */
	private function get_address_object( $data ) {

		$address = new Address();

		// The empty country value may occur API errors.
		if ( ! empty( $data['address']['country'] ) ) {
			$address->setAddressLine1( $data['address']['address1'] );
			$address->setAddressLine2( $data['address']['address2'] );
			$address->setLocality( $data['address']['city'] );
			$address->setAdministrativeDistrictLevel1( $data['address']['state'] );
			$address->setPostalCode( $data['address']['postal'] );
			$address->setCountry( $data['address']['country'] );
		}

		if ( ! empty( $data['first_name'] ) ) {
			$address->setFirstName( $data['first_name'] );
		}

		if ( ! empty( $data['last_name'] ) ) {
			$address->setLastName( $data['last_name'] );
		}

		return $address;
	}

	/**
	 * Retrieve information of all locations of a business.
	 *
	 * @since 1.0.0
	 *
	 * @return array|Location|null
	 */
	public function get_locations() {

		try {
			$this->response = $this->client->getLocationsApi()->listLocations();

			if ( ! $this->response->isSuccess() ) {
				return null;
			}

			return $this->response->getResult()->getLocations();

		} catch ( ApiException $e ) {
			$this->exception = $e;

			return null;
		}
	}

	/**
	 * Retrieve a Merchant object with details.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id The Merchant ID.
	 *
	 * @return Merchant|null
	 */
	public function get_merchant( $id ) {

		try {
			$this->response = $this->client->getMerchantsApi()->retrieveMerchant( $id );

			if ( ! $this->response->isSuccess() ) {
				return null;
			}

			return $this->response->getResult()->getMerchant();

		} catch ( ApiException $e ) {
			$this->exception = $e;

			return null;
		}
	}

	/**
	 * Retrieve an idempotency key.
	 *
	 * @since 1.0.0
	 *
	 * @link https://developer.squareup.com/docs/working-with-apis/idempotency
	 *
	 * @return string
	 */
	private function get_idempotency_key() {

		return uniqid( '', false );
	}

	/**
	 * Retrieve API errors.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_errors() {

		return $this->errors;
	}

	/**
	 * Retrieve last API call errors if are exist.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_response_errors() {

		if ( $this->response instanceof ApiResponse && ! $this->response->isSuccess() ) {
			$errors = [];

			foreach ( (array) $this->response->getErrors() as $error ) {
				$errors[] = $error->jsonSerialize();
			}

			return $errors;
		}

		if ( $this->exception instanceof ApiException ) {
			return [
				'code'    => $this->exception->getCode(),
				'message' => $this->exception->getMessage(),
			];
		}

		return [];
	}

	/**
	 * Retrieve last API call response resource.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_response_resource() {

		if ( ! $this->response instanceof ApiResponse ) {
			return [];
		}

		if ( $this->response->getResult() instanceof CreatePaymentResponse ) {
			return [ 'payment' => $this->response->getResult()->getPayment() ];
		}

		return [];
	}
}
