<?php

namespace WPFormsSquare;

/**
 * Square form frontend related functionality.
 *
 * @since 1.0.0
 */
class Frontend {

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Frontend hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_frontend_container_class', [ $this, 'form_container_class' ], 10, 2 );
		add_action( 'wpforms_wp_footer',                [ $this, 'enqueues' ] );
	}

	/**
	 * Add class to form container if Square is enabled.
	 *
	 * @since 1.0.0
	 *
	 * @param array $classes   Array of form classes.
	 * @param array $form_data Form data of current form.
	 *
	 * @return array
	 */
	public function form_container_class( $classes, $form_data ) {

		if ( ! Connection::get() ) {
			return $classes;
		}

		if ( ! Helpers::has_square_field( $form_data ) ) {
			return $classes;
		}

		if ( Helpers::is_square_single_enabled( $form_data ) ) {
			$classes[] = 'wpforms-square';
		}

		return $classes;
	}

	/**
	 * Enqueue assets in the frontend if Square is in use on the page.
	 *
	 * @since 1.0.0
	 *
	 * @param array $forms Form data of forms on current page.
	 */
	public function enqueues( $forms ) {

		$connection = Connection::get();

		if ( ! $connection || ! $connection->is_usable() ) {
			return;
		}

		if ( ! Helpers::has_square_field( $forms, true ) ) {
			return;
		}

		if ( ! Helpers::has_square_enabled( $forms ) ) {
			return;
		}

		$min = wpforms_get_min_suffix();

		// Include styles if the "Include Form Styling > No Styles" is not set.
		if ( wpforms_setting( 'disable-css', '1' ) !== '3' ) {
			wp_enqueue_style(
				'wpforms-square',
				WPFORMS_SQUARE_URL . "assets/css/wpforms-square{$min}.css",
				[],
				WPFORMS_SQUARE_VERSION
			);
		}

		wp_enqueue_script(
			'square-web-payments-sdk',
			Helpers::is_sandbox_mode() ? 'https://sandbox.web.squarecdn.com/v1/square.js' : 'https://web.squarecdn.com/v1/square.js',
			[],
			null,
			true
		);

		wp_enqueue_script(
			'wpforms-square',
			WPFORMS_SQUARE_URL . "assets/js/wpforms-square{$min}.js",
			[ 'jquery', 'square-web-payments-sdk' ],
			WPFORMS_SQUARE_VERSION,
			true
		);

		/**
		 * Filter details about a buyer to help verify their identify.
		 *
		 * @since 1.0.0
		 *
		 * @link https://developer.squareup.com/reference/sdks/web/payments#Payments.verifyBuyer
		 *
		 * @param array $verify_buyer_details Buyer verify details.
		 * @param array $forms                Form data of forms on current page.
		 */
		$verify_buyer_details = (array) apply_filters(
			'wpforms_square_frontend_enqueues_verify_buyer_details',
			[
				'charge' => [],
				'store'  => [],
			],
			$forms
		);

		/**
		 * This filter allows to set a card configuration and styles.
		 *
		 * @since 1.0.0
		 *
		 * @link https://developer.squareup.com/reference/sdks/web/payments/card-payments#Card.configure.options
		 *
		 * @param array $card_config Configuration and style options.
		 * @param array $forms       Form data of forms on current page.
		 */
		$card_config = (array) apply_filters( 'wpforms_square_frontend_enqueues_card_config', [], $forms );

		wp_localize_script(
			'wpforms-square',
			'wpforms_square',
			[
				'client_id'            => $connection->get_client_id(),
				'location_id'          => Helpers::get_location_id(),
				'verify_buyer_details' => $verify_buyer_details,
				'card_config'          => $card_config,
				'i18n'                 => [
					'missing_sdk_script' => esc_html__( 'Square.js failed to load properly.', 'wpforms-square' ),
					'general_error'      => esc_html__( 'An unexpected Square SDK error has occurred.', 'wpforms-square' ),
					'missing_creds'      => esc_html__( 'Client ID and/or Location ID is incorrect.', 'wpforms-square' ),
					'card_init_error'    => esc_html__( 'Initializing Card failed.', 'wpforms-square' ),
					'token_process_fail' => esc_html__( 'Tokenization of the payment card failed.', 'wpforms-square' ),
					'token_status_error' => esc_html__( 'Tokenization failed with status:', 'wpforms-square' ),
					'buyer_verify_error' => esc_html__( 'The verification was not successful. An issue occurred while verifying the buyer.', 'wpforms-square' ),
					'empty_details'      => esc_html__( 'Please fill out payment details to continue.', 'wpforms-square' ),
				],
			]
		);
	}
}
