<?php

namespace WPFormsSquare;

use Square\Environment;
use WPForms\Helpers\Transient;

/**
 * Square related helper methods.
 *
 * @since 1.0.0
 */
class Helpers {

	/**
	 * Determine if Square single payment is enabled for the form.
	 *
	 * @since 1.0.0
	 *
	 * @param array $form_data Form data and settings.
	 *
	 * @return bool
	 */
	public static function is_square_single_enabled( $form_data ) {

		return ! empty( $form_data['payments']['square']['enable'] );
	}

	/**
	 * Determine if Square is in use on the page.
	 *
	 * @since 1.0.0
	 *
	 * @param array $forms Forms data (e.g. forms on a current page).
	 *
	 * @return bool
	 */
	public static function has_square_enabled( $forms ) {

		foreach ( $forms as $form_data ) {
			if ( self::is_square_single_enabled( $form_data ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Determine if Square field is in the form.
	 *
	 * @since 1.0.0
	 *
	 * @param array $forms    Form data (e.g. forms on a current page).
	 * @param bool  $multiple Must be 'true' if $forms contain multiple forms.
	 *
	 * @return bool
	 */
	public static function has_square_field( $forms, $multiple = false ) {

		return wpforms_has_field_type( 'square', $forms, $multiple );
	}

	/**
	 * Determine whether Square is in sandbox mode.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	public static function is_sandbox_mode() {

		return self::get_mode() === Environment::SANDBOX;
	}

	/**
	 * Determine whether Square is in production mode.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	public static function is_production_mode() {

		return self::get_mode() === Environment::PRODUCTION;
	}

	/**
	 * Retrieve Square mode from WPForms settings.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public static function get_mode() {

		return wpforms_setting( 'square-sandbox-mode' ) ? Environment::SANDBOX : Environment::PRODUCTION;
	}

	/**
	 * Set/update Square mode from WPForms settings.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode that will be set.
	 *
	 * @return bool
	 */
	public static function set_mode( $mode ) {

		$key              = 'square-sandbox-mode';
		$settings         = (array) get_option( 'wpforms_settings', [] );
		$settings[ $key ] = $mode === Environment::SANDBOX;

		return update_option( 'wpforms_settings', $settings );
	}

	/**
	 * Retrieve Square Business Location ID from WPForms settings.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 *
	 * @return string
	 */
	public static function get_location_id( $mode = '' ) {

		$mode = self::validate_mode( $mode );

		return wpforms_setting( 'square-location-id-' . $mode, '' );
	}

	/**
	 * Set/update Square Business Location ID from WPForms settings.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id   The location ID.
	 * @param string $mode Square mode.
	 *
	 * @return bool
	 */
	public static function set_locataion_id( $id, $mode ) {

		$mode             = self::validate_mode( $mode );
		$key              = 'square-location-id-' . $mode;
		$settings         = (array) get_option( 'wpforms_settings', [] );
		$settings[ $key ] = (string) $id;

		return update_option( 'wpforms_settings', $settings );
	}

	/**
	 * Delete transients by mode.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode.
	 */
	public static function detete_transients( $mode ) {

		Transient::delete( 'wpforms_square_account_' . $mode );
		Transient::delete( 'wpforms_square_active_locations_' . $mode );
	}

	/**
	 * Retrieve Square available modes.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public static function get_available_modes() {

		return [ Environment::SANDBOX, Environment::PRODUCTION ];
	}

	/**
	 * Validate Square mode to ensure it's either 'production' or 'sandbox'.
	 * If given mode is invalid, fetches current Square mode.
	 *
	 * @since 1.0.0
	 *
	 * @param string $mode Square mode to validate.
	 *
	 * @return string
	 */
	public static function validate_mode( $mode ) {

		return in_array( $mode, self::get_available_modes(), true ) ? $mode : self::get_mode();
	}

	/**
	 * Retrieve the WPForms > Payments settings page URL.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public static function get_settings_page_url() {

		return add_query_arg(
			[
				'page' => 'wpforms-settings',
				'view' => 'payments',
			],
			admin_url( 'admin.php' )
		);
	}

	/**
	 * The `array_key_first` polyfill.
	 *
	 * @since 1.0.0
	 *
	 * @param array $arr Input array.
	 *
	 * @return mixed
	 */
	public static function array_key_first( $arr ) {

		if ( function_exists( 'array_key_first' ) ) {
			return array_key_first( $arr );
		}

		foreach ( (array) $arr as $key => $unused ) {
			return $key;
		}

		return null;
	}
}
