<?php

namespace WPFormsSquare;

use WPForms_Updater;
use WPFormsSquare\Migrations\Migrations;

/**
 * WPForms Square main class.
 *
 * @since 1.0.0
 */
final class Plugin {

	/**
	 * Square application name.
	 *
	 * @since 1.0.0
	 */
	const APP_NAME = 'WPForms';

	/**
	 * Connect handler instance.
	 *
	 * @since 1.0.0
	 *
	 * @var Admin\Connect
	 */
	private $connect;

	/**
	 * Square credit card field.
	 *
	 * @since 1.0.0
	 *
	 * @var Fields\Square;
	 */
	private $field;

	/**
	 * Plugin constructor.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {}

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 *
	 * @return Plugin
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Plugin hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_loaded',  [ $this, 'setup' ], 15 );
		add_action( 'wpforms_updater', [ $this, 'updater' ] );
	}

	/**
	 * All the actual plugin loading is done here.
	 *
	 * @since 1.0.0
	 */
	public function setup() {

		( new Migrations() )->init();

		$this->load_admin_entries();
		$this->load_settings();
		$this->load_connect();
		$this->load_field();
		$this->load_builder();
		$this->load_block_editor();
		$this->load_frontend();
		$this->load_processing();
		$this->load_admin_payments();
	}

	/**
	 * Load admin entries functionality.
	 *
	 * @since 1.0.0
	 */
	private function load_admin_entries() {

		if ( wpforms_is_admin_page( 'entries' ) ) {
			( new Admin\Entries() )->init();
		}
	}

	/**
	 * Load Square settings.
	 *
	 * @since 1.0.0
	 */
	private function load_settings() {

		if ( wpforms_is_admin_page( 'settings', 'payments' ) ) {
			( new Admin\Settings() )->init();
		}
	}

	/**
	 * Load connect handler.
	 *
	 * @since 1.0.0
	 */
	private function load_connect() {

		$this->connect = ( new Admin\Connect() )->init();
	}

	/**
	 * Load Square field.
	 *
	 * @since 1.0.0
	 */
	private function load_field() {

		//phpcs:disable WordPress.Security.NonceVerification
		$is_elementor =
			( ! empty( $_POST['action'] ) && $_POST['action'] === 'elementor_ajax' ) ||
			( ! empty( $_GET['action'] ) && $_GET['action'] === 'elementor' );
		// phpcs:enable WordPress.Security.NonceVerification

		if ( $is_elementor || ! is_admin() || wp_doing_ajax() || wpforms_is_admin_page( 'builder' ) ) {
			$this->field = new Fields\Square();
		}
	}

	/**
	 * Load builder functionality.
	 *
	 * @since 1.0.0
	 */
	private function load_builder() {

		if ( wpforms_is_admin_page( 'builder' ) ) {
			new Admin\SquarePayment();
			( new Admin\Builder() )->init();
		}
	}

	/**
	 * Load Block editor functionality.
	 *
	 * @since 1.0.0
	 */
	private function load_block_editor() {

		( new Admin\BlockEditor() )->init();
	}

	/**
	 * Load frontend functionality.
	 *
	 * @since 1.0.0
	 */
	private function load_frontend() {

		if ( ! is_admin() ) {
			( new Frontend() )->init();
		}
	}

	/**
	 * Load payment form processing.
	 *
	 * @since 1.0.0
	 */
	private function load_processing() {

		if ( ! is_admin() || wpforms_is_frontend_ajax() ) {
			( new Process() )->init();
		}
	}

	/**
	 * Initialize class for extending the Payments functionality.
	 *
	 * @since 1.5.0
	 */
	private function load_admin_payments() {

		if ( ! wpforms_is_admin_page( 'payments' ) ) {
			return;
		}

		( new Admin\Payments() )->hooks();
	}

	/**
	 * Load the plugin updater.
	 *
	 * @since 1.0.0
	 *
	 * @param string $key License key.
	 */
	public function updater( $key ) {

		new WPForms_Updater(
			[
				'plugin_name' => 'WPForms Square',
				'plugin_slug' => 'wpforms-square',
				'plugin_path' => plugin_basename( WPFORMS_SQUARE_FILE ),
				'plugin_url'  => trailingslashit( WPFORMS_SQUARE_URL ),
				'remote_url'  => WPFORMS_UPDATER_API,
				'version'     => WPFORMS_SQUARE_VERSION,
				'key'         => $key,
			]
		);
	}

	/**
	 * Retrieve a connect handler.
	 *
	 * @since 1.0.0
	 *
	 * @return Admin\Connect
	 */
	public function get_connect() {

		return $this->connect;
	}

	/**
	 * Retrieve a field instance.
	 *
	 * @since 1.0.0
	 *
	 * @return Fields\Square
	 */
	public function get_field() {

		return $this->field;
	}

	/**
	 * Retrieve an API instance.
	 *
	 * @since 1.0.0
	 *
	 * @param Connection $connection Connection object.
	 *
	 * @return Api\Api|null
	 */
	public function get_api( $connection ) {

		if ( ! $connection instanceof Connection ) {
			return null;
		}

		return new Api\Api( $connection );
	}

	/**
	 * Get a single instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return Plugin
	 */
	public static function get_instance() {

		static $instance;

		if ( ! $instance ) {
			$instance = new self();

			$instance->init();
		}

		return $instance;
	}
}
