<?php

namespace WPFormsSquare;

use Square\Models\Card;
use Square\Models\ErrorCode;
use WP_Post;

/**
 * Square payment processing.
 *
 * @since 1.0.0
 */
class Process {

	/**
	 * Form ID.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	public $form_id = 0;

	/**
	 * Sanitized submitted field values and data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $fields = [];

	/**
	 * Form submission data ($_POST).
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $entry = [];

	/**
	 * Form data and settings.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $form_data = [];

	/**
	 * Form Square payment settings.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $settings = [];

	/**
	 * Square credit card field.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $cc_field = [];

	/**
	 * Payment amount.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $amount = '';

	/**
	 * Payment currency.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $currency = '';

	/**
	 * Connection data.
	 *
	 * @since 1.0.0
	 *
	 * @var Connection
	 */
	public $connection;

	/**
	 * Main class that communicates with the Square API.
	 *
	 * @since 1.0.0
	 *
	 * @var Api\Api
	 */
	private $api;

	/**
	 * Square form errors.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $errors;

	/**
	 * Whether the payment has been processed.
	 *
	 * @since 1.7.0
	 *
	 * @var bool
	 */
	private $is_payment_processed = false;

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 *
	 * @return Process
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Process hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_process', [ $this, 'process_entry' ], 10, 3 );
		add_action( 'wpforms_process_complete', [ $this, 'update_entry_meta' ], 10, 4 );
		add_filter( 'wpforms_forms_submission_prepare_payment_data', [ $this, 'prepare_payment_data' ], 10, 3 );
		add_filter( 'wpforms_forms_submission_prepare_payment_meta', [ $this, 'prepare_payment_meta' ], 10, 3 );
		add_action( 'wpforms_process_payment_saved', [ $this, 'process_payment_saved' ], 10, 3 );
		add_filter( 'wpforms_entry_email_process', [ $this, 'process_email' ], 70, 4 );
		add_filter( 'wpforms_process_bypass_captcha', [ $this, 'bypass_captcha' ] );
	}

	/**
	 * Check if a payment exists with an entry, if so validate and process.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields    Final/sanitized submitted field data.
	 * @param array $entry     Copy of original $_POST.
	 * @param array $form_data Form data and settings.
	 */
	public function process_entry( $fields, $entry, $form_data ) {

		// Check if payment method exists and is enabled.
		if ( ! Helpers::is_square_single_enabled( $form_data ) ) {
			return;
		}

		$this->fields     = $fields;
		$this->entry      = $entry;
		$this->form_data  = $form_data;
		$this->form_id    = (int) $form_data['id'];
		$this->settings   = $form_data['payments']['square'];
		$this->cc_field   = $this->get_credit_card_field();
		$this->currency   = $this->get_currency();
		$this->amount     = $this->get_amount();
		$this->connection = Connection::get();

		// Before proceeding, check if any basic errors were detected.
		if ( ! $this->is_form_processed() ) {
			$this->display_errors();

			return;
		}

		// Set payment processing flag.
		$this->is_payment_processed = true;

		// Set an API instance.
		$this->api = wpforms_square()->get_api( $this->connection );

		// Set tokens provided by Web Payments SDK.
		$this->api->set_payment_tokens( $entry );

		// Proceed to executing the purchase.
		$this->process_payment();

		// Update the card field value to contain basic details.
		$this->update_credit_card_field_value();
	}

	/**
	 * Bypass captcha if payment has been processed.
	 *
	 * @since 1.7.0
	 *
	 * @param bool $bypass_captcha Whether to bypass captcha.
	 *
	 * @return bool
	 */
	public function bypass_captcha( $bypass_captcha ) {

		if ( $bypass_captcha ) {
			return $bypass_captcha;
		}

		return $this->is_payment_processed;
	}

	/**
	 * Check if form has errors before payment processing.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function is_form_processed() {

		// Bail in case there are form processing errors.
		if ( ! empty( wpforms()->get( 'process' )->errors[ $this->form_id ] ) ) {
			return false;
		}

		if ( ! $this->is_card_field_visibility_ok() ) {
			return false;
		}

		return $this->is_form_ok();
	}

	/**
	 * Check form settings, fields, etc.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function is_form_ok() {

		// Check for conditional logic.
		if ( ! $this->is_conditional_logic_ok( $this->settings ) ) {
			$title = esc_html__( 'Square payment stopped by conditional logic.', 'wpforms-square' );

			$this->log_errors( $title, $this->fields, 'conditional_logic' );

			return false;
		}

		// Check for Square connection.
		if ( ! $this->is_connection_ok() ) {
			$error_title    = esc_html__( 'Square payment stopped, account connection is missing.', 'wpforms-square' );
			$this->errors[] = $error_title;

			$this->log_errors( $error_title );

			return false;
		}

		// Check total charge amount.
		// Square has different minimum amount limits by country.
		if ( ! $this->is_amount_ok() ) {
			$error_title    = esc_html__( 'Square payment stopped, amount is smaller than the allowed minimum amount for a payment.', 'wpforms-square' );
			$this->errors[] = $error_title;

			$this->log_errors(
				$error_title,
				[
					'amount'   => $this->amount,
					'currency' => $this->currency,
				]
			);

			return false;
		}

		// Check that, despite how the form is configured, the form and
		// entry actually contain payment fields, otherwise no need to proceed.
		if ( empty( $this->cc_field ) || ! wpforms_has_payment( 'form', $this->fields ) ) {
			$error_title    = esc_html__( 'Square payment stopped, missing payment fields.', 'wpforms-square' );
			$this->errors[] = $error_title;

			$this->log_errors( $error_title );

			return false;
		}

		return true;
	}

	/**
	 * Check if there is at least one visible (not hidden by conditional logic) card field in the form.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function is_card_field_visibility_ok() {

		// If the form contains no Square credit card field.
		if ( empty( $this->cc_field ) ) {
			return false;
		}

		// If the form contains no fields with conditional logic the card field is visible by default.
		if ( empty( $this->form_data['conditional_fields'] ) ) {
			return true;
		}

		// If the field is NOT in array of conditional fields, it's visible.
		if ( ! in_array( $this->cc_field['id'], $this->form_data['conditional_fields'], true ) ) {
			return true;
		}

		// If the field IS in array of conditional fields and marked as visible, it's visible.
		if ( ! empty( $this->cc_field['visible'] ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Check if conditional logic check passes for the given settings.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings Conditional logic settings to process.
	 *
	 * @return bool
	 */
	private function is_conditional_logic_ok( $settings ) {

		// Check conditional logic settings.
		if (
			empty( $settings['conditional_logic'] ) ||
			empty( $settings['conditional_type'] ) ||
			empty( $settings['conditionals'] )
		) {
			return true;
		}

		// All conditional logic checks passed, continue with processing.
		$process = wpforms_conditional_logic()->process( $this->fields, $this->form_data, $settings['conditionals'] );

		if ( $settings['conditional_type'] === 'stop' ) {
			$process = ! $process;
		}

		return $process;
	}

	/**
	 * Check if connection is exists, configured and valid.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function is_connection_ok() {

		return $this->connection !== null && $this->connection->is_usable();
	}

	/**
	 * Check if an amount is greater than the minimum amount.
	 *
	 * @since 1.0.0
	 *
	 * @link https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts#monetary-amount-limits
	 *
	 * @return bool
	 */
	private function is_amount_ok() {

		$amount = $this->format_amount( $this->amount );

		if ( $amount < 1 && in_array( $this->currency, [ 'USD', 'CAD' ], true ) ) {
			return false;
		}

		if ( $amount < 100 && in_array( $this->currency, [ 'EUR', 'GBP', 'AUD', 'JPY' ], true ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Process a payment.
	 *
	 * @since 1.0.0
	 */
	private function process_payment() {

		$this->process_payment_single();
	}

	/**
	 * Process a single payment.
	 *
	 * @since 1.0.0
	 */
	private function process_payment_single() {

		$args = $this->get_payment_args_single();

		$this->api->process_single_transaction( $args );

		$this->process_api_errors( 'single' );
	}

	/**
	 * Retrieve single payment args.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_payment_args_single() {

		$args           = $this->get_payment_args_general();
		$customer_name  = $this->get_customer_name();
		$customer_email = $this->get_customer_email();

		// Billing Name.
		if ( isset( $customer_name['first_name'] ) ) {
			$args['billing']['first_name'] = sanitize_text_field( $customer_name['first_name'] );
		}

		if ( isset( $customer_name['last_name'] ) ) {
			$args['billing']['last_name'] = sanitize_text_field( $customer_name['last_name'] );
		}

		// Billing Address.
		if ( ! empty( $this->fields[ $this->settings['billing_address'] ] ) ) {
			$args['billing']['address'] = $this->fields[ $this->settings['billing_address'] ];
		}

		// Buyer Email.
		if ( ! empty( $customer_email ) ) {
			$args['buyer_email'] = sanitize_email( $customer_email );
		}

		// Payment description.
		$description = empty( $this->settings['payment_description'] ) ? $this->get_form_name() : html_entity_decode( $this->settings['payment_description'], ENT_COMPAT, 'UTF-8' );

		// The maximum length for the Square notes field is 500 characters.
		$args['note'] = wp_html_excerpt( Plugin::APP_NAME . ': ' . $description, 500 );

		// Order items.
		$args['order_items'] = $this->get_order_items();

		/**
		 * Filter single payment arguments.
		 *
		 * @since 1.0.0
		 *
		 * @param array   $args    The single payment arguments.
		 * @param Process $process The Process instance.
		 */
		return (array) apply_filters( 'wpforms_square_process_get_payment_args_single', $args, $this );
	}

	/**
	 * Retrieve arguments for any type of payment.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_payment_args_general() {

		/**
		 * Filter arguments for any type of payment.
		 *
		 * @since 1.0.0
		 *
		 * @param array   $args    The general payment arguments.
		 * @param Process $process The Process instance.
		 */
		return (array) apply_filters(
			'wpforms_square_process_get_payment_args_general',
			[
				'amount'      => $this->format_amount( $this->amount ),
				'currency'    => $this->currency,
				'location_id' => Helpers::get_location_id(),
			],
			$this
		);
	}

	/**
	 * Retrieve a payment currency.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_currency() {

		return strtoupper( wpforms_get_currency() );
	}

	/**
	 * Retrieve a payment amount.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_amount() {

		$amount = wpforms_get_total_payment( $this->fields );

		return $amount === false ? wpforms_sanitize_amount( 0 ) : $amount;
	}

	/**
	 * Retrieve order items.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_order_items() {

		/**
		 * Filter order items types.
		 *
		 * @since 1.6.0
		 *
		 * @param array $types The order items types.
		 */
		$types = apply_filters( 'wpforms_square_process_get_order_items_types', wpforms_payment_fields() );
		$items = [];

		foreach ( $this->fields as $field_id => $field ) {

			if (
				empty( $field['type'] ) ||
				! in_array( $field['type'], $types, true )
			) {
				continue;
			}

			// Skip payment field that is not filled in.
			if (
				! isset( $this->entry['fields'][ $field_id ] ) ||
				wpforms_is_empty_string( $this->entry['fields'][ $field_id ] )
			) {
				continue;
			}

			$items[] = $this->prepare_order_line_item( $field );
		}

		return $items;
	}

	/**
	 * Retrieve a Form Name.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_form_name() {

		if ( ! empty( $this->form_data['settings']['form_title'] ) ) {
			return sanitize_text_field( $this->form_data['settings']['form_title'] );
		}

		$form = wpforms()->get( 'form' )->get( $this->form_id );

		return $form instanceof WP_Post ? $form->post_title : sprintf( /* translators: %d - Form ID. */ esc_html__( 'Form #%d', 'wpforms-square' ), $this->form_id );
	}

	/**
	 * Retrieve a Square credit card field.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_credit_card_field() {

		foreach ( (array) $this->fields as $field ) {

			if ( $field['type'] !== 'square' ) {
				continue;
			}

			return $field;
		}

		return [];
	}

	/**
	 * Prepare order line item.
	 *
	 * @since 1.0.0
	 *
	 * @param array $field Field data.
	 *
	 * @return array
	 */
	private function prepare_order_line_item( $field ) {

		$field_id = absint( $field['id'] );
		$name     = empty( $field['name'] ) ? sprintf( /* translators: %d - Field ID. */ esc_html__( 'Field #%d', 'wpforms-square' ), $field_id ) : $field['name'];
		$item     = [
			'name' => $name,
		];

		if ( empty( $field['value_raw'] ) ) {
			$item['amount'] = $this->format_amount( $field['amount_raw'] );

			return $item;
		}

		$values     = explode( ',', $field['value_raw'] );
		$variations = [];

		foreach ( $values as $value ) {

			if ( empty( $this->form_data['fields'][ $field_id ]['choices'][ $value ] ) ) {
				continue;
			}

			$choice = $this->form_data['fields'][ $field_id ]['choices'][ $value ];

			$variations[] = [
				'name'           => $name,
				'variation_name' => empty( $choice['label'] ) ? sprintf( /* translators: %d - Choice ID. */ esc_html__( 'Choice %d', 'wpforms-square' ), absint( $value ) ) : $choice['label'],
				'amount'         => $this->format_amount( $choice['value'] ),
			];
		}

		if ( ! empty( $variations ) ) {
			$item['variations'] = $variations;
		}

		return $item;
	}

	/**
	 * Return a formatted amount required by Square API.
	 *
	 * @since 1.0.0
	 *
	 * @param string $amount Price amount.
	 *
	 * @return float|int
	 */
	private function format_amount( $amount ) {

		$amount          = wpforms_sanitize_amount( $amount );
		$amount_decimals = (int) str_pad( 1, wpforms_get_currency_decimals( $this->currency ) + 1, 0, STR_PAD_RIGHT );

		return $amount * $amount_decimals;
	}

	/**
	 * Display form errors.
	 *
	 * @since 1.0.0
	 *
	 * @param array $errors Errors to display.
	 */
	private function display_errors( $errors = [] ) {

		if ( ! $errors ) {
			$errors = $this->errors;
		}

		if ( ! $errors || ! is_array( $errors ) ) {
			return;
		}

		// Check if the form contains a required credit card. If it does
		// and there was an error, return the error to the user and prevent
		// the form from being submitted. This should not occur under normal
		// circumstances.
		if ( empty( $this->cc_field ) || empty( $this->form_data['fields'][ $this->cc_field['id'] ] ) ) {
			return;
		}

		if ( ! empty( $this->form_data['fields'][ $this->cc_field['id'] ]['required'] ) ) {
			wpforms()->get( 'process' )->errors[ $this->form_id ]['footer'] = implode( '<br>', $errors );
		}
	}

	/**
	 * Collect errors from API and turn it into form errors.
	 *
	 * @since 1.0.0
	 *
	 * @param string $type Payment type (e.g. 'single').
	 */
	private function process_api_errors( $type ) {

		$errors = $this->api->get_errors();

		if ( empty( $errors ) || ! is_array( $errors ) ) {
			return;
		}

		$this->display_errors( $errors );

		$title   = esc_html__( 'Square payment stopped', 'wpforms-square' );
		$_errors = $this->api->get_response_errors();

		if ( ! empty( $_errors ) ) {
			$this->process_api_errors_codes( $_errors );
			$errors[] = $_errors;
		}

		// Log transaction specific errors.
		$this->log_errors( $title, $errors );
	}

	/**
	 * Check specific error codes.
	 *
	 * @since 1.0.0
	 *
	 * @param array $errors The last API call errors.
	 */
	private function process_api_errors_codes( $errors ) {

		$codes = $this->get_oauth_error_codes();

		foreach ( $errors as $error ) {

			if (
				empty( $error['code'] ) ||
				! in_array( $error['code'], $codes, true )
			) {
				continue;
			}

			// If the error indicates that access token is bad, set a connection as invalid.
			$this->connection
				->set_status( Connection::STATUS_INVALID )
				->save();
		}
	}

	/**
	 * Retrieve OAuth-related errors.
	 *
	 * @since 1.0.0
	 *
	 * @link https://developer.squareup.com/docs/oauth-api/best-practices#ensure-api-calls-made-with-oauth-tokens-handle-token-based-errors-appropriately
	 *
	 * @return array
	 */
	private function get_oauth_error_codes() {

		return [ ErrorCode::ACCESS_TOKEN_EXPIRED, ErrorCode::ACCESS_TOKEN_REVOKED, ErrorCode::UNAUTHORIZED ];
	}

	/**
	 * Log payment errors.
	 *
	 * @since 1.0.0
	 *
	 * @param string       $title    Error title.
	 * @param array|string $messages Error messages.
	 * @param string       $level    Error level to add to 'payment' error level.
	 */
	private function log_errors( $title, $messages = [], $level = 'error' ) {

		wpforms_log(
			$title,
			$messages,
			[
				'type'    => [ 'payment', $level ],
				'form_id' => $this->form_id,
			]
		);
	}

	/**
	 * Update the credit card field value to contain basic details.
	 *
	 * @since 1.0.0
	 */
	private function update_credit_card_field_value() {

		if ( $this->errors || ! $this->api ) {
			return;
		}

		$resource = $this->api->get_response_resource();

		if ( empty( $resource ) ) {
			return;
		}

		$type = Helpers::array_key_first( $resource );

		// Get a card.
		$card = $resource[ $type ]->getCardDetails()->getCard();

		if ( empty( $card ) ) {
			return;
		}

		$details = [
			'brand'  => $card->getCardBrand(),
			'last4'  => $card->getLast4(),
			'holder' => $this->get_card_holder( $card ),
		];
		$details = implode( "\n", array_filter( $details ) );

		/**
		 * Filter a credit card field value by card details.
		 *
		 * @since 1.0.0
		 *
		 * @param string  $details Card details.
		 * @param Process $process Process object.
		 */
		$details = apply_filters(
			'wpforms_square_process_update_credit_card_field_value',
			$details,
			$this
		);

		wpforms()->get( 'process' )->fields[ $this->cc_field['id'] ]['value'] = $details;
	}

	/**
	 * Update entry details and add meta for a successful payment.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $fields    Final/sanitized submitted field data.
	 * @param array  $entry     Copy of original $_POST.
	 * @param array  $form_data Form data and settings.
	 * @param string $entry_id  Entry ID.
	 */
	public function update_entry_meta( $fields, $entry, $form_data, $entry_id ) {

		if ( empty( $entry_id ) || $this->errors || ! $this->api ) {
			return;
		}

		$resource = $this->api->get_response_resource();

		if ( empty( $resource ) ) {
			return;
		}

		wpforms()->get( 'entry' )->update(
			$entry_id,
			[
				'type' => 'payment',
			],
			'',
			'',
			[ 'cap' => false ]
		);

		/**
		 * Fire when entry details and add meta was successfully updated.
		 *
		 * @since 1.0.0
		 *
		 * @param array   $fields    Final/sanitized submitted field data.
		 * @param array   $form_data Form data and settings.
		 * @param string  $entry_id  Entry ID.
		 * @param array   $resource  Response resource data.
		 * @param Process $process   Process class instance.
		 */
		do_action( 'wpforms_square_process_update_entry_meta', $fields, $form_data, $entry_id, $resource, $this );
	}

	/**
	 * Add details to payment data.
	 *
	 * @since 1.5.0
	 *
	 * @param array $payment_data Payment data args.
	 * @param array $fields       Final/sanitized submitted field data.
	 * @param array $form_data    Form data and settings.
	 *
	 * @return array
	 */
	public function prepare_payment_data( $payment_data, $fields, $form_data ) {

		// If there are errors or API is not initialized, return the original payment data.
		if ( $this->errors || ! $this->api ) {
			return $payment_data;
		}

		$resource = $this->api->get_response_resource();

		// If the resource is empty, return the original payment meta.
		if ( empty( $resource ) ) {
			return $payment_data;
		}

		$type    = Helpers::array_key_first( $resource );
		$payment = $resource[ $type ];

		$payment_data['status']         = 'processed';
		$payment_data['gateway']        = 'square';
		$payment_data['mode']           = Helpers::is_sandbox_mode() ? 'test' : 'live';
		$payment_data['customer_id']    = sanitize_text_field( $payment->getCustomerId() );
		$payment_data['transaction_id'] = sanitize_text_field( $payment->getId() );
		$payment_data['title']          = $this->get_payment_title( $payment );

		return $payment_data;
	}

	/**
	 * Get Payment title.
	 *
	 * @since 1.5.0
	 *
	 * @param object $payment Payment object.
	 *
	 * @return string Payment title.
	 */
	private function get_payment_title( $payment ) {

		// Look for the cardholder name.
		$customer_name = $this->get_card_holder( $payment->getCardDetails()->getCard() );

		if ( $customer_name ) {
			return sanitize_text_field( $customer_name );
		}

		$customer_name = $this->get_customer_name();

		if ( $customer_name ) {
			return sanitize_text_field( implode( ' ', array_values( $customer_name ) ) );
		}

		$customer_email = $this->get_customer_email();

		if ( $customer_email ) {
			return sanitize_email( $customer_email );
		}

		return '';
	}

	/**
	 * Add payment meta for a successful one-time or subscription payment.
	 *
	 * @since 1.5.0
	 *
	 * @param array $payment_meta Payment meta.
	 * @param array $fields       Final/sanitized submitted field data.
	 * @param array $form_data    Form data and settings.
	 *
	 * @return array
	 */
	public function prepare_payment_meta( $payment_meta, $fields, $form_data ) {

		// If there are errors or API is not initialized, return the original payment meta.
		if ( $this->errors || ! $this->api ) {
			return $payment_meta;
		}

		$resource = $this->api->get_response_resource();

		// If the resource is empty, return the original payment meta.
		if ( empty( $resource ) ) {
			return $payment_meta;
		}

		$type                = Helpers::array_key_first( $resource );
		$payment             = $resource[ $type ];
		$credit_card_details = $payment->getCardDetails()->getCard();

		$payment_meta['method_type']    = strtolower( $payment->getSourceType() );
		$payment_meta['receipt_number'] = $payment->getReceiptNumber();

		if ( ! empty( $credit_card_details ) ) {
			$payment_meta['credit_card_last4']   = $credit_card_details->getLast4();
			$payment_meta['credit_card_expires'] = $credit_card_details->getExpMonth() . '/' . $credit_card_details->getExpYear();
			$payment_meta['credit_card_method']  = strtolower( $credit_card_details->getCardBrand() );
			$payment_meta['credit_card_name']    = $this->get_card_holder( $credit_card_details );
		}

		// Add a log indicating that the charge was successful.
		$payment_meta['log'] = $this->format_payment_log(
			sprintf(
				'Square order created. (Order ID: %s)',
				$payment->getOrderId()
			)
		);

		return $payment_meta;
	}

	/**
	 * Add payment info for successful payment.
	 *
	 * @since 1.5.0
	 *
	 * @param string $payment_id Payment ID.
	 * @param array  $fields     Final/sanitized submitted field data.
	 * @param array  $form_data  Form data and settings.
	 */
	public function process_payment_saved( $payment_id, $fields, $form_data ) {

		// If there are errors or API is not initialized, return the original payment meta.
		if ( $this->errors || ! $this->api ) {
			return;
		}

		$resource = $this->api->get_response_resource();

		// If the resource is empty, return the original payment meta.
		if ( empty( $resource ) ) {
			return;
		}

		$type = Helpers::array_key_first( $resource );

		// phpcs:disable WordPress.DB.SlowDBQuery.slow_db_query_meta_key, WordPress.DB.SlowDBQuery.slow_db_query_meta_value
		wpforms()->get( 'payment_meta' )->add(
			[
				'payment_id' => $payment_id,
				'meta_key'   => 'log',
				'meta_value' => $this->format_payment_log(
					sprintf(
						'Square payment completed. (Receipt ID: %s)',
						$resource[ $type ]->getReceiptNumber()
					)
				),
			]
		);
		// phpcs:enable WordPress.DB.SlowDBQuery.slow_db_query_meta_key, WordPress.DB.SlowDBQuery.slow_db_query_meta_value
	}

	/**
	 * Return payment log value.
	 *
	 * @since 1.5.0
	 *
	 * @param string $value Log value.
	 *
	 * @return string
	 */
	private function format_payment_log( $value ) {

		return wp_json_encode(
			[
				'value' => sanitize_text_field( $value ),
				'date'  => gmdate( 'Y-m-d H:i:s' ),
			]
		);
	}

	/**
	 * Get Customer name.
	 *
	 * @since 1.5.0
	 *
	 * @return array
	 */
	private function get_customer_name() {

		$customer_name = [];

		// Billing first name.
		if ( ! empty( $this->fields[ $this->settings['billing_name'] ]['first'] ) ) {
			$customer_name['first_name'] = $this->fields[ $this->settings['billing_name'] ]['first'];
		}

		// Billing last name.
		if ( ! empty( $this->fields[ $this->settings['billing_name'] ]['last'] ) ) {
			$customer_name['last_name'] = $this->fields[ $this->settings['billing_name'] ]['last'];
		}

		// If a Name field has the `Simple` format.
		if (
			empty( $customer_name['first_name'] ) &&
			empty( $customer_name['last_name'] ) &&
			! empty( $this->fields[ $this->settings['billing_name'] ]['value'] )
		) {
			$customer_name['first_name'] = $this->fields[ $this->settings['billing_name'] ]['value'];
		}

		return $customer_name;
	}

	/**
	 * Get Customer email.
	 *
	 * @since 1.5.0
	 *
	 * @return string
	 */
	private function get_customer_email() {

		return ! empty( $this->fields[ $this->settings['buyer_email'] ]['value'] ) ? $this->fields[ $this->settings['buyer_email'] ]['value'] : '';
	}

	/**
	 * Logic that helps decide if we should send completed payments notifications.
	 *
	 * @since 1.0.0
	 *
	 * @param bool  $process         Whether to process or not.
	 * @param array $fields          Form fields.
	 * @param array $form_data       Form data.
	 * @param int   $notification_id Notification ID.
	 *
	 * @return bool
	 */
	public function process_email( $process, $fields, $form_data, $notification_id ) {

		if ( ! $process ) {
			return false;
		}

		if ( ! Helpers::is_square_single_enabled( $form_data ) ) {
			return $process;
		}

		if ( empty( $form_data['settings']['notifications'][ $notification_id ]['square'] ) ) {
			return $process;
		}

		if ( ! $this->is_conditional_logic_ok( $this->settings ) ) {
			return false;
		}

		return ! $this->errors && $this->api && empty( $this->api->get_errors() ) && empty( $this->api->get_response_errors() );
	}

	/**
	 * Retrieve a Cardholder Name.
	 *
	 * @since 1.0.0
	 *
	 * @param Card $card Card object.
	 *
	 * @return string
	 */
	private function get_card_holder( $card ) {

		$holder = '';

		if ( $card instanceof Card ) {
			$holder = $card->getCardholderName();
		}

		if ( empty( $holder ) && isset( $this->cc_field['cardname'] ) ) {
			$holder = $this->cc_field['cardname'];
		}

		return $holder;
	}
}
